/***************************************************************************
                          applytemplate.cpp  -  description
                             -------------------
    begin                : Sat Oct 2 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "expr.h"
#include "xpathfactory.h"
#include "applytemplate.h"
#include "templatelauncher.h"
#include "xpathstring.h"

namespace limpid {

ApplyTemplate::ApplyTemplate(const DOMString& selectString,
      const DOMString& modeString, bool useImports):
      Template(selectString), expr_(XPathFactory::getExpr(selectString)),
      selectionManager_(useImports, modeString) {}

void ApplyTemplate::setResources(XSLResources& resources) {
  selectionManager_.setResources(resources);
  defaultTemplate_.setResources(resources);
}

bool ApplyTemplate::process(XSLContext& parentContext) {
  NodeSet resultSet = expr_.getNodeSet(parentContext);
  XSLContext context; // use new context for recursion
  context.setParent(parentContext);
  context.setNode(parentContext.getNode());
  context.setCurrentSet(resultSet);
  sortTemplates_.process(context);
//  size_t count = resultSet.getLength();
//  context.setCount(count);
  for (size_t index = 0; index < resultSet.getLength(); ++index) {
    Node& node = resultSet.getItem(index);

    context.setNode(node);
    context.setPosition(index + 1);
    TemplateLauncher(selectionManager_, defaultTemplate_, parameters_).process(context);
  }

  return true;
}

bool ApplyTemplate::processNode(XSLContext& parentContext, Node& parentNode) {
  NodeSet resultSet = expr_.getNodeSet(parentContext);
  XSLContext context; // use new context for recursion
  context.setParent(parentContext);
  context.setNode(parentContext.getNode());
  context.setCurrentSet(resultSet);
  sortTemplates_.process(context);

  size_t count = resultSet.getLength();
  context.setCount(count);

  for (size_t index = 0; index < count; ++index) {
    Node& node = resultSet.getItem(index);
    context.setNode(node);
    context.setPosition(index);
    TemplateLauncher(selectionManager_, defaultTemplate_, parameters_).processNode(context, parentNode);
  }

  return true;
}


}
