/***************************************************************************
                          arithmeticcomponent.cpp  -  description
                             -------------------
    begin                : Thu May 27 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "arithmeticcomponent.h"
#include "boolean.h"
#include "xpathstring.h"
#include "number.h"
#include "writer.h"

namespace limpid {

ArithmeticComponent::ArithmeticComponent(const DOMString& type): ParentComponent() {
  setType(type);
}

Boolean ArithmeticComponent::getBoolean(Context& context) {
  return getNumber(context).getBoolean(context);
}

XPathString ArithmeticComponent::getString(Context& context) {
  return getNumber(context).getString(context);
}

Number ArithmeticComponent::getNumber(Context& context) {
  size_t length = getLength();
  if (!length) return 0;
  else if (length == 1)
    return components->at(0)->getNumber(context);
  else {
    double firstVal = components->at(0)->getNumber(context).getValue();
    double secondVal = components->at(1)->getNumber(context).getValue();

    if (type_ == ADD) return firstVal + secondVal;
    if (type_ == SUBTRACT) return firstVal - secondVal;
    if (type_ == MULTIPLY) return  firstVal * secondVal;
    if (type_ == DIVIDE) return  firstVal / secondVal;
    if (type_ == MOD) return (int)firstVal % (int)secondVal;
    return 0;
  }
}

void ArithmeticComponent::setType(const DOMString& typeString) {
  type_ = ADD;
  if (typeString == "-") type_ = SUBTRACT;
  else if (typeString == "*") type_ = MULTIPLY;
  else if (typeString == "div") type_ = DIVIDE;
  else if (typeString == "mod") type_ = MOD;
}

}
