/***************************************************************************
                          attr.h  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ATTR_H
#define ATTR_H

#include <string>
#include "parentnode.h"
#include "attributecontent.h"

/**
Attr is a ParentNode and models the DOM Attr class.

  @author root <john_redmond@optusnet.com.au>
*/

namespace limpid {
  class Element;

  class Attr : public ParentNode  {
    friend class Element;

  public:
    Attr(const DOMString& name);
    Attr(const DOMString& namespaceURI, const DOMString& name);
    Attr(const Attr& refNode);
    Attr& operator=(const Attr& refNode);

    static Attr failAttr;

  //DOM:
    DOMString getName() { return getNodeName(); }
    bool getSpecified() { return ((AttributeContent *)nodeContent_)->specified_; }
    DOMString getValue() const;
    DOMString getNodeValue() const { return getValue(); }
    void setValue(const DOMString& value);
    void setNodeValue(DOMString& value) { setValue(value); }
    Element& getOwnerElement() const;
    DOMString getNamespaceURI() const;
    Node& cloneNode(bool deep = false) const;

  //non-DOM:
    bool isValid() const { return nodeContent_ != failAttr.nodeContent_; }
    bool operator!() const { return nodeContent_ == failAttr.nodeContent_; }
    DOMString getRawValue() const;
    DOMString getDisplayString() const;
    void setSpecified(bool flag) { ((AttributeContent *)nodeContent_)->specified_ = flag; }

  private:
  	Attr();
    Attr& copy() { return *(new Attr(*this)); }

    void setNamespaceURI(const DOMString& namespaceURI);
  };
}
#endif
