/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "attributeconverter.h"
#include "nodestream.h"
#include "liststream.h"
#include "element.h"
#include "attr.h"

namespace limpid {

AttributeConverter::AttributeConverter(const DOMString& attributeName):
 attributeName_(attributeName) {
}

Document&  AttributeConverter::processDocument(Document& inputDocument) {
  if (attributeName_.trim().empty())
    return inputDocument; // this must be a mistake

  NodeList matchedList;
  NodeStream nodeStream(inputDocument, false, false);

  while (true) {
    Node& node = nodeStream.nextNode();
    if (!node.isValid()) break;

    if (node.getNodeType() != Node::ELEMENT_NODE) continue;

    Element& element = static_cast<Element &>(node);
    Attr& attribute = element.getAttributeNode(attributeName_);

    if (attribute.isValid())
      matchedList.addNode(element);
  }

  ListStream listStream(matchedList);
  listStream.setForward(false);

  while (true) {
    Node& node = listStream.nextNode();
    if (!node.isValid()) break;

    Element& element = static_cast<Element &>(node);
    DOMString name = element.getAttribute(attributeName_);

    Element newElement(name);
    node.getParentNode().insertBefore(newElement, node);

    copyChildren(node, newElement);
    node.getParentNode().removeChild(node);
  }

  return inputDocument;
}

void AttributeConverter::copyChildren(Node& sourceNode, Node& destNode) {
  while (true) {
    Node& childNode = sourceNode.getFirstChild();
    if (!childNode.isValid()) break;	// no more children

    destNode.appendChild(childNode);
  }
}

}
