/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "attributestripper.h"
#include "nodestream.h"
#include "element.h"
#include "namednodemap.h"

namespace limpid {

Document& AttributeStripper::processDocument(Document& inputDocument) {
  NodeStream nodeStream(inputDocument, false, false);

  while (true) {
    Node& node = nodeStream.nextNode();
    if (!node.isValid()) break;

    if (node.getNodeType() != Node::ELEMENT_NODE) continue;

    Element& element = static_cast<Element &>(node);

    if (attrName_.empty()) { // remove all attributes
      NamedNodeMap& attributes = element.getAttributes();
      if (!attributes.getLength()) continue;

      // the DOM interfaces do not provide a more elegant way of doing this:
      while (attributes.getLength()) {
        Node& attributeNode = attributes.item(0);
        attributes.removeNamedItem(attributeNode.getNodeName());
      }
    }

    else { // remove a named attribute
      Node& attrNode = element.getAttributeNode(attrName_);
      if (attrNode.isValid())
        element.removeAttribute(attrName_);
    }
  }

  return inputDocument;
}


}
