/***************************************************************************
                          attributetemplate.cpp  -  description
                             -------------------
    begin                : Fri Jul 1 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "attributetemplate.h"
#include "xpathfactory.h"
#include "xslcontext.h"
#include "element.h"
#include "attr.h"

namespace limpid {

AttributeTemplate::AttributeTemplate(const DOMString& name, Expr& nameExpr,
      Expr& uriExpr, Expr& valueExpr): Template(name), nameExpr_(nameExpr),
    uriExpr_(uriExpr), valueExpr_(valueExpr) {}

AttributeTemplate::AttributeTemplate(const DOMString& name, Expr& nameExpr,
      Expr& valueExpr): Template(name), nameExpr_(nameExpr), valueExpr_(valueExpr) {}

bool AttributeTemplate::process(XSLContext& parentContext) {
  int parentType = parentContext.getNode().getNodeType();
  if (parentType != Node::ELEMENT_NODE && parentType != Node::DOCUMENT_NODE)
    return true;  // should not happen

  // note that there is no way of specifying a namespaceuri in text
  DOMString nameString = nameExpr_.getString(parentContext);
  DOMString valueString = valueExpr_.getString(parentContext);

  if (!parentContext.getEscapingAttributes())
    valueString.resolveEntities();

  parentContext.getWriter() << ' ' << nameString << "=\"" << valueString << "\"";

  return true;
}

bool AttributeTemplate::processNode(XSLContext& parentContext, Node& node) {
  if (node.getNodeType() != Node::ELEMENT_NODE) return true;  // should not happen

  DOMString nameString = nameExpr_.getString(parentContext);
  DOMString valueString = valueExpr_.getString(parentContext);

  if (!parentContext.getEscapingAttributes())
    valueString.resolveEntities();

  DOMString uriString;
  if (uriExpr_.isValid())
    uriString = uriExpr_.getString(parentContext);

  Attr attr = Attr(nameString);
  if (!uriString.empty()) attr = Attr(uriString, nameString);

  attr.setValue(valueString);
  static_cast<Element &>(node).setAttributeNode(attr);

  return true;
}

}
