/***************************************************************************
                          attributevalidator.h  -  description
                             -------------------
    begin                : Wed Jan 7 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ATTRIBUTEVALIDATOR_H
#define ATTRIBUTEVALIDATOR_H

#include <set>
#include <map>
#include "node.h"
#include "document.h"
#include "element.h"
#include "domstring.h"
#include "attributespec.h"
#include "charactervalidator.h"
#include "writer.h"
#include "converter.h"
#include "pointer.h"

/**
AttributeValidator is an internal class for validation of a document.

  @author root <john_redmond@optusnet.com.au>
*/

namespace limpid {
  typedef std::map<DOMString, Node*> SimpleMap;
  typedef std::map<DOMString, AttributeSpec> AttributeMap;
  typedef std::map<DOMString, AttributeMap> ElementMap;

  class AttributeValidator: protected Converter {
  public:
    AttributeValidator();

    void setup(Document& document);
    void validateNames(Element& element, CharacterValidator& characterValidator);
    void validateSpecified(Element& element);
    void validateValues(Element& element, CharacterValidator& characterValidator);
    void validateComplete(Element& element);

    void setWriter(Writer& wr) { writer = new Writer(wr); }
    size_t getSize();

  protected:
    Pointer<Writer> writer;
    Pointer <ElementMap> elementMap;

    void collectSimpleTypes(Document& document, SimpleMap& simpleTypes);
    void collectContent(Document& document, SimpleMap& simpleTypes);
    void addAttributeSpecs(Node& parentNode, SimpleMap& simpleTypes, AttributeMap& attributeMap);
    void addSimpleSpec(DOMString& key, Element& attrElement, AttributeMap& attributeMap);
    void addReferenceSpec(DOMString& key, Element& attrElement, Element& referredElement,
            AttributeMap& attributeMap);
    AttributeSpec::Type getType(DOMString& typeString);
    void validateEnumeration(DOMString& elementName, Enumeration& enumeration,
            DOMString& attrName, DOMString& value, int lineNo);
    void validateType(Element& element, AttributeSpec& spec,
            DOMString& attrValue, CharacterValidator& characterValidator);
    void outputSet(Enumeration& enumeration);
  };
}
#endif
