/***************************************************************************
                          builder.cpp  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "element.h"
#include "attr.h"
#include "endnode.h"
#include "builder.h"
#include "writer.h"

namespace limpid {

Builder::Builder(bool checkBalance): currentNode_(0), checkBalance_(checkBalance) {
  name_ = "builder";
  push(document_);
}

Builder::Builder(Node& rootNode, bool checkBalance): currentNode_(0), checkBalance_(checkBalance) {
  name_ = "builder";
  push(rootNode);
}

Node& Builder::process(Node& node) {
  if (!node.isValid()) {
    reportStackErrors();
    emptyStack();
    return node;  // source stream is no longer valid
  }

  int type = node.getNodeType();
  if (type == Node::SYSTEM_NODE) return node;  // HTML parse

  if (nodeStack_.back()->getNodeType() == Node::DOCUMENT_NODE &&
      type == Node::TEXT_NODE) return node; // handle XSLT mode

  if (type == Node::DOCUMENT_NODE) {
    // ignore document node and use the internal one
    return document_;
  }

  if (type == Node::END_NODE && !nodeStack_.empty()) {
    while (nodeStack_.size() > 2) {
      bool matched = node.getNodeName() == nodeStack_.back()->getNodeName(); 
if (!matched)
wrt::Errors << "Line " << node.getPosition() << ": " << node << " unbalanced with start node " << *(nodeStack_.back()) << " at " << nodeStack_.back()->getPosition() << wrt::endl; 
      pop();
      if (matched) break;
    }

    return node;
  }

  if (!nodeStack_.empty() && nodeStack_.back()->getNodeType() == Node::ELEMENT_NODE &&
      type == Node::ATTRIBUTE_NODE) {
    Element& element = static_cast<Element &>(*nodeStack_.back());
    element.setAttributeNode(static_cast<Attr &>(node));

    return node;
  }

  Node &resultNode = nodeStack_.back()->appendChild(node);

  if (type == Node::ELEMENT_NODE && (node.hasChildNodes() || node.isParent()))
    push(resultNode);


  return resultNode;
}

inline void Builder::push(Node& node) {
  nodeStack_.push_back(&node.copy());
}

inline void Builder::pop() {
  if (!nodeStack_.empty()) {
  nodeStack_.pop_back();
  }
///  else &document_;
}

void Builder::emptyStack() {
  while (!nodeStack_.empty()) {
    delete(nodeStack_.back());  
    nodeStack_.pop_back();
  }
}

Document& Builder::getDocument() {
///  reportStackErrors();
  return document_;
}

void Builder::reportStackErrors() {
  if (nodeStack_.size() == 1 || !checkBalance_) return;   // only the document is on the stack: correct

  wrt::Errors << nodeStack_.size() << " End tags were missing for the following start tags\
  (or one of their child tags)\n" <<\
  "and has produced incorrect structure in the document:" << wrt::endl;
  nodeStack_.pop_front(); // the document
  while (!nodeStack_.empty()) {
    wrt::Errors << "\tLine " << nodeStack_.front()->getPosition() << ": Tag: "\
    << *nodeStack_.front() << wrt::endl;
    nodeStack_.pop_front();
  }
}

}
