/***************************************************************************
 *   Copyright (C) 2006 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ccparser.h"
#include "node.h"
#include "element.h"
#include "text.h"
#include "liststream.h"
#include "writer.h"
#include "stringreader.h"
#include "domwriter.h"
#include "converter.h"

namespace limpid {

CCParser::CCParser(const DOMString& fileName):
    reader_(new Reader(fileName)), tis_(new TokenStream(*reader_)),
    lineNo_(1), isStarted_(false), nodeSent_(false) {
  configure();
}
/*
CCParser::CCParser(const Reader& charStream):
    reader_(new Reader(charStream)), tis_(new TokenStream(*reader_)),
    lineNo_(1), isStarted_(false), nodeSent_(false) {
  configure();
}
*/
Node& CCParser::nextNode() {
  if (nodeSent_) {
    nodeSent_ = false;
    return endNode_; // this will trigger denesting in an XML builder
  }
  
  if (!isStarted_) {
    isStarted_ = true;
    node_ = new Element("SHEET"); // this is the document element of the new document
    node_->addChildNodes();
    node_->setPosition(0);
    
    return *node_;
  }
  
  if (!setLine()) // the data stream is finished
    return Node::failNode;

  DOMString rowName = "ROW";
  
  parseLine(rowName);
  node_->setPosition(lineNo_);

  nodeSent_ = true;
  return *node_;
}

// create a row and add the column elements to it
void CCParser::parseLine(DOMString& rowName) {
  static DOMString delimiterString = delimiterChar_;
  static DOMString endString = "\n";
  
  if (true) {
//    line_.trim();
    if (line_[2] == '-') {
      node_ = new Element(line_.substring(0, 2));
      DOMString line = line_.substring(4, line_.size() - 4);
      Text text(line);
      node_->appendChild(text);
    }
    else {
      node_ = new Element(rowName);
      Text text(line_.trim());
      node_->appendChild(text);
    }
    return;
  }
  
  StringReader stringReader(line_);
  TokenStream tis(stringReader);
  tis.setDelimiters(delimiterString_);
  tis.setWhitespaceChars("");

  size_t index = 0;
  DOMString value;
  
  while (tis.isValid()) {
    DOMString token = tis.nextToken();

    if (token == delimiterString || token == endString) { // this triggers setting of the value
      setColumn(*node_, value, index);
      if (token == "\n") break;
      ++index; // bump if continuing
    }
    else
      value = token;
  }
}

// add a column with data to the entry element
void CCParser::setColumn(Node& entryElement, DOMString& value, size_t index) {
  Element col("COL");
  entryElement.appendChild(col);

  // add data only if it is not empty
  if (value.getLength()) {
//    value.escapeDelimiters().trim();
    Text valueText(value);
    col.appendChild(valueText);
  }
}

// establish a line to be parsed
bool CCParser::setLine() {
  while (true) {
    if (!tis_->isValid()) return false;

    line_ = tis_->nextToken();
    ++lineNo_;
    if (!line_.empty()) {
      line_ += '\n';
      return true;
    }
  }
}

void CCParser::configure() {
//  delimiterString_ += Char(delimiterChar_);
//  delimiterString_ += '\n';

  tis_->setWhitespaceChars("\n");
  tis_->setDelimiters("");
}

}
