/***************************************************************************
                          characterdata.cpp  -  description
                             -------------------
    begin                : Wed Sep 10 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "characterdata.h"
#include "writer.h"

namespace limpid {

CharacterData::CharacterData(NodeType nodeType): Node(nodeType) {
  try {
    nodeContent_ = new CharacterContent(nodeType);
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

CharacterData::CharacterData(NodeType nodeType, const DOMString& data):
    Node(nodeType) {
  try {
    nodeContent_ = new CharacterContent(nodeType, data);
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

CharacterData::CharacterData(const CharacterData& refData):
    Node(refData) {}

CharacterData& CharacterData::operator=(const CharacterData& refData) {
  replaceContent(refData);
  return *this;
}

DOMString CharacterData::getData() const {
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);
  if (contentPtr->data_) return contentPtr->data_;
  else return "";
}

void CharacterData::setData(const DOMString& newData) {
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);
  delete [] contentPtr->data_;

  contentPtr->data_ = newData.copyChars();
}

size_t CharacterData::getLength() const {
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);
  return DOMString::strLength(contentPtr->data_);
}

void CharacterData::appendData(const DOMString& extraData) {
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);
  DOMString d = contentPtr->data_;
  d += extraData;

  delete [] contentPtr->data_;
  contentPtr->data_ = d.copyChars();
}

void CharacterData::insertData(size_t offset, const DOMString& extraData) {
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);
  DOMString d = contentPtr->data_;
  d.insertString(offset, extraData);

  delete [] contentPtr->data_;
  contentPtr->data_ = d.copyChars();
}

void CharacterData::replaceData(size_t offset, size_t count, const DOMString& extraData) {
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);
  DOMString d = contentPtr->data_;
  d.change(offset, count, extraData);

  delete [] contentPtr->data_;
  contentPtr->data_ = d.copyChars();
}

DOMString CharacterData::substringData(size_t offset, size_t count) const {
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);
  DOMString d = contentPtr->data_;
  return d.substring(offset, count);
}

size_t CharacterData::getSize() {
  size_t size = sizeof(*this);
  size += getNodeValue().getSize() + sizeof(Char);

  return size;
}

}
