/***************************************************************************
                          characterwriter.cpp  -  description
                             -------------------
    begin                : Thu Jun 9 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "characterwriter.h"
#include "domstring.h"
#include "converter.h"
#include "node.h"

namespace limpid {

CharacterWriter::CharacterWriter() {}

// non-member functions:
#ifndef _NARROW_
CharacterWriter& operator<<(CharacterWriter& w, const Char c) {
  w.putChar(c);
  return w;
}
#endif

CharacterWriter& operator<<(CharacterWriter& w, const char c) {
  w.putChar(wchar_t(0) | c);
  return w;
}

CharacterWriter& operator<<(CharacterWriter& w, const DOMString& s) {
  return w.putString(s);
}

#ifndef _NARROW_
CharacterWriter& operator<<(CharacterWriter& w, const Char* s) {
  while (*s) w.putChar(int(*s++));
   return w;
}
#endif


CharacterWriter& operator<<(CharacterWriter& w, const char* s) {
  while (*s) w.putChar(wchar_t(0) | *s++);
   return w;
}

CharacterWriter& operator<<(CharacterWriter& w, int i) {
   return w.putString(Converter::toString(i));
}

CharacterWriter& operator<<(CharacterWriter& w, size_t i) {
   return w.putString(Converter::toString(i));
}

CharacterWriter& operator<<(CharacterWriter& w, long int i) {
   return w.putString(Converter::toString(i));
}

CharacterWriter& operator<<(CharacterWriter& w, unsigned long int i) {
   return w.putString(Converter::toString(i));
}

CharacterWriter& operator<<(CharacterWriter& w, double d) {
   return w.putString(Converter::toString(d));
}

CharacterWriter& operator<<(CharacterWriter& w, void* p) {
   return w << '@' << Converter::toHexString(p);
}

CharacterWriter& operator<<(CharacterWriter& w, const void* p) {
   return w << '@' << Converter::toHexString(p);
}

CharacterWriter& operator<<(CharacterWriter& w, bool flag) {
  if (flag) return w << "true";
  else return w << "false";
}

CharacterWriter& operator<<(CharacterWriter& w, const Node& node) {
   return w.putString(node.getDisplayString());
}

}
