/***************************************************************************
                          contentvalidator.h  -  description
                             -------------------
    begin                : Sat Dec 6 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTENTVALIDATOR_H
#define CONTENTVALIDATOR_H

#include <map>
#include <vector>
#include "document.h"
#include "element.h"
#include "liststream.h"
#include "nodestream.h"
#include "nodelist.h"
#include "namednodemap.h"
#include "writer.h"
#include "pointer.h"
#include "matchpoint.h"

/**
  *@author John Redmond
  */

namespace limpid {
//  typedef std::map<DOMString, GraphNavigator*> NavigatorMap;
  typedef std::map<DOMString, MatchPoint*> MatchMap;
  typedef std::vector<DOMString> StringList;
  typedef std::map<DOMString, DOMString> ChildrenMap;
  typedef std::map<DOMString, DOMString> TypeMap;

  class ContentValidator {
  public:
    ContentValidator();

    void setup(Document& document); // public for external access
    void setWriter(Writer& writer);
    void validate(Element& element);
    void validateChildren(Element& element);
    void setSimple(bool simple) { simple_ = simple; }
    size_t getSize();

  private:

    Pointer<Writer> writer_;
    Pointer<MatchMap> matchMap_;
    Pointer<ChildrenMap> childrenMap_;
    Pointer<TypeMap> typeMap_;
    bool simple_;

    void setTypes(Document& document);
    void addContentPoint(Element& element);
    void addChildrenList(Element& element);

    void validateContent(Element& element);
    void validateAttributes(Element& element);
    bool isXSDElement(Node& node);
};
}
#endif
