/***************************************************************************
                          context.cpp  -  description
                             -------------------
    begin                : Mon May 10 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "context.h"
#include "writer.h"

namespace limpid {

Context::Context():
      count_(0), position_(0), node_(0), current_(0),
      parentContext_(0) {
}

bool Context::setVariable(const DOMString& key, const Expr& value) {
  ExprMap::iterator p = variables_.find(key);
  if (p != variables_.end())  // remove the old variable definition
    variables_.erase(p);

  variables_.insert(make_pair(key, value));

  return true;
}

Expr& Context::getVariableExpr(const DOMString& key, bool imported) {
  if (!imported) { // try to get an unimported expression
    ExprMap::iterator p = variables_.find(key);
    if (p != variables_.end()) return p->second;
  }
  // try to get an imported expression
  ExprMap::iterator q = importedVariables_.find(key);
  if (q !=  importedVariables_.end()) return q->second;

  return failExpr;
}

Expr Context::getVariable(const DOMString& key, bool imported) {
  Context* activeContext = this;

  while (true) {
    Expr& expr = activeContext->getVariableExpr(key, imported);
    if (expr.isValid()) return expr;

    if (activeContext->parentContext_)
      activeContext = activeContext->parentContext_;
    else break;
  }

  return failExpr;
}

int Context::getDepth() {
  int depth = 0;
  Context *currentContext = this;

  while (true) {
    if (!currentContext->hasParent())
      break;
    ++depth;
    currentContext = &(currentContext->getParent());
  }
  return depth;
}

void Context::listVariables() {
  wrt::Errors << "KEYS: list of " << variables_.size() << " in " << this << wrt::endl;

  ExprMap::iterator p = variables_.begin();
  while (p != variables_.end()) {
    DOMString key = p->first;
    wrt::Errors << " KEY: = " << key << " for "
      << p->second.getString(*this) << " in " << this << wrt::endl;
    ++p;
  }
}

Expr Context::failExpr;
}
