/***************************************************************************
                          context.h  -  description
                             -------------------
    begin                : Mon May 10 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTEXT_H
#define CONTEXT_H

#include <map>
#include "node.h"
#include "pointer.h"
#include "expr.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class DOMString;
  class DecimalFormatMap;

  class Context {
  public:
  	Context();

    void setCount(int count) { count_ = count; }
    void setPosition(int position) { position_ = position; }
    void setNode(Node& node) { node_ = &node;  current_ = &node; }
    void setCurrentNode(Node& node) { current_ = &node; }
    void updateCurrentNode() { current_ = node_; }
    void setDecimalFormatMap(DecimalFormatMap& map) { formatMap_ = &map; }

    int getCount() { return count_; }
    int getPosition() { return position_; }
    Node& getNode() { if (node_) return *node_; else return Node::failNode; }
    Node& getCurrentNode() { if (current_) return *current_; else return Node::failNode; }
    DecimalFormatMap& getDecimalFormatMap() { return *formatMap_; }

    bool hasParent() { return parentContext_; }
    Context& getParent() { return *parentContext_; }

    void clearVariables() { variables_.clear(); }
    bool setVariable(const DOMString& key, const Expr& value);

    Expr getVariable(const DOMString& key, bool imported = false);
    size_t getVariableCount() { return variables_.size(); }
    int getDepth();
    void listVariables();

    static Expr failExpr;

   protected:
    typedef std::map<DOMString, Expr> ExprMap;

    int count_, position_;
    Node *node_, *current_;  // not managed pointers
    ExprMap variables_, importedVariables_;
    Context* parentContext_; // not a managed pointer
    DecimalFormatMap* formatMap_; // not a managed pointer
    bool verbose_;

    DOMString getLocalVariable(DOMString& key);
    Expr& getVariableExpr(const DOMString& key, bool imported);
  };
}

#endif

