/***************************************************************************
                          converter.cpp  -  description
                             -------------------
    begin                : Fri Apr 2 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <sstream>
#include "converter.h"

using namespace limpid;

namespace limpid {

Converter::Converter() {
}

int Converter::toInt(const DOMString& intString) {
  try {
    int result = 0;
    if (intString.empty()) return result;

    const char* cstring = intString.getCString();
    std::istringstream ss(cstring);
    ss >> result;

    delete cstring;
    return result;
  }
  catch (...) {
    return 0;
  }

}

double Converter::toDouble(const DOMString& dString) {
  try {
    double result = 0;
    if (dString.empty()) return result;

    const char* cstring = dString.getCString();
    std::istringstream ss(cstring);
    ss >> result;

    delete cstring;
    return result;
  }
  catch (...) {
    return 0;
  }
}

DOMString Converter::toString(int i) {
  try {
    std::ostringstream ss;
    ss << i;
    return ss.str().c_str();
  }
  catch (...) {
    return "NaN";
  }
}

DOMString Converter::toString(size_t i) {
  try {
    std::ostringstream ss;
    ss << i;
    return ss.str().c_str();
  }
  catch (...) {
    return "NaN";
  }
}

DOMString Converter::toString(long int i) {
  try {
    std::ostringstream ss;
    ss << i;
    return ss.str().c_str();
  }
  catch (...) {
    return "NaN";
  }
}

DOMString Converter::toString(unsigned long int i) {
  try {
    std::ostringstream ss;
    ss << i;
    return ss.str().c_str();
  }
  catch (...) {
    return "NaN";
  }
}

DOMString Converter::toString(double d) {
  try {
  std::ostringstream ss;
  ss << d;
  return ss.str().c_str();
  }
  catch (...) {
    return "NaN";
  }
}

DOMString Converter::toString(void *ptr) {
  return toString(size_t(ptr));
}

DOMString Converter::toHexString(int i) {
  try {
    std::ostringstream ss;
    ss << std::hex << i;
    return ss.str().c_str();
  }
  catch (...) {
    return "NaN";
  }
}

DOMString Converter::toHexString(size_t i) {
  try {
    std::ostringstream ss;
    ss << std::hex << i;
    return ss.str().c_str();
  }
  catch (...) {
    return "NaN";
  }
}

DOMString Converter::toHexString(double d) {
  try {
  std::ostringstream ss;
  ss << std::hex << d;
  return ss.str().c_str();
  }
  catch (...) {
    return "NaN";
  }
}

DOMString Converter::toHexString(const void *ptr) {
  return toHexString(size_t(ptr));
}

bool Converter::toBoolean(DOMString s) {  // work with a DOMString copy
  s.toUpperCase();
  return s == "TRUE" || s == "YES";
}

bool Converter::isNumber(const DOMString& token) {
  size_t size = token.size();
  for (size_t index = 0; index < size; index++)
    if (!isdigit(token[index]) && token[index] != '.') return false;

  return true;
}

int Converter::maxInt = INT_MAX;
}
