/***************************************************************************
                          copyoftemplate.cpp  -  description
                             -------------------
    begin                : Sat Jan 15 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "copyoftemplate.h"
#include "node.h"
#include "attr.h"
#include "element.h"
#include "document.h"
#include "xpathfactory.h"

namespace limpid {

CopyOfTemplate::CopyOfTemplate(const DOMString& selectString): Template(selectString),
     expr_(XPathFactory::getExpr(selectString)) {}

bool CopyOfTemplate::process(XSLContext& parentContext) {
  Expr expr = expr_.eval(parentContext); // dereference if expr_ is a variable
  NodeSet nodeSet = expr.getNodeSet(parentContext);
  size_t length = nodeSet.getLength();

  DOMWriter domWriter(parentContext.getWriter());
  domWriter.setStartAndFinish("", "");

  for (size_t index = 0; index < length; ++index) {
    domWriter.write(nodeSet.item(index));
  }

  return true;
}

bool CopyOfTemplate::processNode(XSLContext& parentContext, Node& node) {
  Expr expr = expr_.eval(parentContext); // dereference if expr_ is
  NodeSet& nodeSet = static_cast<NodeSet &>(expr.getContent());
  size_t length = nodeSet.getLength();

  for (size_t index = 0; index < length; ++index) {
    Node& childNode = nodeSet.item(index);
    int type = childNode.getNodeType();

    if (type == Node::ATTRIBUTE_NODE && node.getNodeType() == Node::ELEMENT_NODE) {
      Attr& attr = static_cast<Attr &>(childNode.cloneNode(true));
      node.getOwnerDocument().importNode(attr, false);
      static_cast<Element &>(node).setAttributeNode(attr);
    }
    else {
      node.appendChild(node.getOwnerDocument().importNode(childNode.cloneNode(true), false));
    }
  }

  return true;
}


}

