/***************************************************************************
                          copytemplate.cpp  -  description
                             -------------------
    begin                : Sun Jan 9 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "copytemplate.h"
#include "element.h"
#include "document.h"
#include "text.h"
#include "attr.h"
#include "writer.h"

namespace limpid {

CopyTemplate::CopyTemplate(bool atStart, bool withChildren):
      atStart_(atStart), withChildren_(withChildren) {}

bool CopyTemplate::process(XSLContext& parentContext) {
  CharacterWriter& writer = parentContext.getWriter();
  Node& node = parentContext.getNode();
  int type = node.getNodeType();

  if (type != Node::ELEMENT_NODE && type != Node::DOCUMENT_NODE) {
    nest(parentContext);
    writeFormat(parentContext);
    writer << node;
    deNest(parentContext);
    return true;
  }

  if (type == Node::DOCUMENT_NODE) {
    Document document;
    processNode(parentContext, document);
    writer << document;
    return true;
  }

  // it must be an element
  Element element(parentContext.getNode().getNodeName());
  attributes_.processNode(parentContext, element);
  templates_.processNode(parentContext, element);

  DOMWriter(parentContext.getWriter()).write(element);
  return true;
}

bool CopyTemplate::processNode(XSLContext& parentContext, Node& node) {
  Node& contextNode = parentContext.getNode();
  int type = contextNode.getNodeType();

  if (type == Node::ATTRIBUTE_NODE && node.getNodeType() == Node::ELEMENT_NODE) {
    DOMString attrName = contextNode.getNodeName();
    if (attrName == "xml:base") return true;  // do not copy this attribute

    DOMString attrValue = contextNode.getValue();
    static_cast<Element &>(node).setAttribute(attrName, attrValue);
  }
  else if (contextNode.getNodeType() == Node::ELEMENT_NODE) {
    Element element(contextNode.getNodeName());
    attributes_.processNode(parentContext, element);
    templates_.processNode(parentContext, element); // recurse for children of element
    node.appendChild(element);
  }
  else {
    node.appendChild(contextNode.cloneNode(false)); // no recursion here
  }

  return true;
}

void CopyTemplate::addTemplate(Template& tmp) {
  if (tmp.isAttribute()) attributes_.addTemplate(tmp);
  else templates_.addTemplate(tmp); }

}

