/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDCSSPARSER_H
#define LIMPIDCSSPARSER_H

#include <deque>
#include "parser.h"
#include "stringreader.h"
#include "pointer.h"
#include "reader.h"
#include "node.h"
#include "element.h"
#include "text.h"
#include "endnode.h"
#include "domstring.h"

namespace limpid {

/**
  @author John Redmond <john_redmond@optusnet.com.au>
*/
class CSSParser : public Parser
{
public:
explicit  CSSParser(const DOMString& fileName);
  CSSParser(CharacterReader& reader);
  CSSParser& copy() { return *(new CSSParser(*this)); }

  Node& nextNode();

private:
    typedef std::deque<DOMString> NameStack;
    typedef std::deque<int> IntegerStack;
    NameStack tagStack_;
    IntegerStack nestingStack_;

    Pointer<Node> node_;
    Pointer<EndNode> endNode_;
    Pointer<CharacterReader> reader_;
    Element* lastElement_;
    Node* root_;
    DOMString rootName_, tagName_, mediaName_, defaultString_;
    int state_, lineNo_;

    void pushElement (const DOMString& name, bool isParent);
    int getState();
    void setTargets();
    void setSpecs();

    DOMString getTagString();
    DOMString getCommentString();
    void skipWhitespace();
    DOMString getLine();
    DOMString getDelimitedString(const DOMString& delimiters);
    Char peekChar();
    Char getChar();
    DOMString normalizePath(Element& parent, DOMString& sourceString);

};

}

#endif
