/***************************************************************************
                          csvparser.h  -  description
                             -------------------
    begin                : Wed Mar 29 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSVPARSER_H
#define CSVPARSER_H

#include <vector>
#include "endnode.h"
#include "parser.h"
#include "pointer.h"
#include "reader.h"
#include "tokenstream.h"
#include "node.h"
#include "domstring.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class DOMString;

  class CSVParser : public Parser  {
  public:
    CSVParser(const DOMString& fileName, int delimiterChar = ',', bool useLabels = true);
    CSVParser(const Reader& characterStream, int delimiterChar = ',', bool useLabels = true);

    Node& nextNode();
    CSVParser& copy() { return *(new CSVParser(*this)); }

  private:
    typedef std::vector<DOMString> StringVector;
    Pointer<StringVector> labels_;
    Pointer<Node> node_;
    Pointer<Reader> reader_;
    Pointer<TokenStream> tis_;
    DOMString line_;
    EndNode endNode_;
    int delimiterChar_, lineNo_;
    bool isStarted_, useLabels_, hasLabels_, nodeSent_;
    DOMString delimiterString_;

    void parseLine(DOMString& rowName);
    bool setLine();
    void setLabels();
    void setColumn(Node& entryElement, DOMString& value, size_t index);
    DOMString getColumnLabel(size_t index);
    void configure();
  };
}

#endif
