/***************************************************************************
                          csvwriter.cpp  -  description
                             -------------------
    begin                : Tue Apr 11 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "csvwriter.h"
#include "domstring.h"
#include "liststream.h"

namespace limpid {

CSVWriter::CSVWriter(int separator):
  writer_(new Writer), separator_(separator) {}
CSVWriter::CSVWriter(const DOMString& destName, int separator):
    writer_(new Writer(destName)), separator_(separator) {}
CSVWriter::CSVWriter(CharacterWriter& characterWriter, int separator):
    writer_(&(characterWriter.copy())), separator_(separator) {}

void CSVWriter::update(const DOMString& destString) {
  DOMString outputString = destString;
  outputString += ".csv";
  writer_ = new Writer(outputString); }

Node& CSVWriter::process(Node& node) {
  if (node.getNodeType() != Node::ELEMENT_NODE)
    return node;  // no action

  DOMString nodeName = node.getNodeName();  
  if (nodeName == "ROW" || nodeName == "LABELS") {
    ListStream stream(node);

    while (true) {
      Node& childNode = stream.nextNode();
      if (!childNode.isValid()) break;

      *writer_ << childNode.getValue().resolveEntities();

      if (childNode.getNextSibling().isValid())
        *writer_ << separator_;
      else
        *writer_ << wrt::endl;
    }
  }

  return node;
}


}
