/***************************************************************************
                          cutter.cpp  -  description
                             -------------------
    begin                : Thu Jul 8 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cutter.h"
#include "node.h"
#include "endnode.h"
#include "nodestream.h"
#include "liststream.h"

namespace limpid {

Cutter::Cutter(const DOMString& matchString, bool discardChildren):
  matcher_(matchString), discardChildren_(discardChildren)
{}

Document& Cutter::processDocument(Document& document) {
  NodeList hitList;
  NodeStream nodeStream(document, false, false); // exclude root, no endnodes
  while (true) {
    Node& node = nodeStream.nextNode();
    if (!node.isValid()) break;

    if (matcher_.match(node))
      hitList.addNode(node);
  }

  ListStream hitStream(hitList);
  hitStream.setForward(false);  // move in reverse order
  while (true) {
    Node& node = hitStream.nextNode();
    if (!node.isValid()) break;

    if (!discardChildren_) copyChildren(node);

    node.getParentNode().removeChild(node);
  }

  return document;
}

void Cutter::copyChildren(Node& targetNode) {
  Node& parentNode = targetNode.getParentNode();
  if (!parentNode.isValid()) return;

  while (true) {
    Node& childNode = targetNode.getFirstChild();
    if (!childNode.isValid()) break;	// no more children

    parentNode.insertBefore(childNode, targetNode);
  }
}

};
