/***************************************************************************
                          decimalformat.cpp  -  description
                             -------------------
    begin                : Fri Apr 29 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "decimalformat.h"
#include "attributestream.h"

namespace limpid {

DecimalFormat::DecimalFormat(const DOMString& decimalSeparatorx,
        const DOMString& groupingSeparatorx,
        const DOMString& infinityx, const DOMString& minusSignx,
        const DOMString& nanx, const DOMString& percentx,
        const DOMString& permillex , const DOMString& zeroDigitx,
        const DOMString& digitx, const DOMString& patternSeparatorx):
          decimalSeparator(decimalSeparatorx), groupingSeparator(groupingSeparatorx),
          infinity(infinityx), minusSign(minusSignx), nan(nanx), percent(percentx),
          permille(permillex), zeroDigit(zeroDigitx), digit(digitx),
          patternSeparator(patternSeparatorx) {
}

void DecimalFormat::update(Node& parentNode) {
  AttributeStream attributeStream(parentNode);
  while (true) {
    Node& node = attributeStream.nextNode();
    if (!node.isValid()) break;

    DOMString name = node.getNodeName();
    DOMString value = node.getValue();
    doUpdate(name, value);
  }
}

void DecimalFormat::doUpdate(DOMString& name, DOMString& value) {
  if (name == "decimal-separator") decimalSeparator = value;
  else if (name == "grouping-separator") groupingSeparator = value;
  else if (name == "decimal-separator") decimalSeparator = value;
  else if (name == "infinity") infinity = value;
  else if (name == "minusSign") minusSign = value;
  else if (name == "NaN") nan = value;
  else if (name == "percent") percent = value;
  else if (name == "per-mille") permille = value;
  else if (name == "zero-digit") zeroDigit = value;
  else if (name == "digit") digit = value;
  else if (name == "pattern-separator") patternSeparator = value;
}

}

