/***************************************************************************
                          defaulttemplate.cpp  -  description
                             -------------------
    begin                : Sun Sep 18 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "defaulttemplate.h"
#include "xslcontext.h"
#include "node.h"
#include "document.h"
#include "cdatasection.h"
#include "liststream.h"

namespace limpid {

DefaultTemplate::DefaultTemplate() {}

bool DefaultTemplate::process(XSLContext& parentContext) {
  Node& root = parentContext.getNode();
  int nodeType = root.getNodeType();

  if (nodeType == Node::TEXT_NODE || nodeType == Node::CDATA_SECTION_NODE) {
    DOMString nodeValue = root.getValue();
    if (!parentContext.getEscapingText())
      nodeValue.resolveEntities();

    if (resources_.hasCDATASection(root.getParentNode())) {
      CDATASection cds(nodeValue);
      parentContext.getWriter() << cds;
    }
    else
      parentContext.getWriter() << nodeValue.trim();

    return true; // no children to process
  }

  if (nodeType == Node::ATTRIBUTE_NODE) {
    DOMString nodeValue = root.getValue();
    if (!parentContext.getEscapingAttributes())
      nodeValue.resolveEntities();

    parentContext.getWriter() << nodeValue;

    return true;
  }

  ListStream stream(root);  // move through the direct children of the context node
  while (true) {
    Node& node = stream.nextNode();
    if (!node.isValid()) break;

    parentContext.setNode(node);
    process(parentContext);
  }

  return true;
}

bool DefaultTemplate::processNode(XSLContext& parentContext, Node& node) {
  Node& root = parentContext.getNode();
  int nodeType = root.getNodeType();

  if (nodeType == Node::TEXT_NODE || nodeType == Node::CDATA_SECTION_NODE) {
    DOMString nodeValue = root.getValue();
    if (!parentContext.getEscapingText())
      nodeValue.resolveEntities();

    if (resources_.hasCDATASection(root.getParentNode())) {
      CDATASection cds(nodeValue);
      node.appendChild(cds);
    }
    else {
      Text text(nodeValue.escapeBra());
      if (node.getNodeType() != Node::DOCUMENT_NODE)
        node.appendChild(text);
    }
    return true;
  }

  if (nodeType == Node::ATTRIBUTE_NODE) {
    DOMString nodeValue = root.getValue();
    if (!parentContext.getEscapingAttributes())
      nodeValue.resolveEntities();

    Text text(nodeValue.escapeDelimiters());
    if (node.getNodeType() != Node::DOCUMENT_NODE)
      node.appendChild(text);

    return true;
  }

  ListStream stream(root);  // move through the direct children of the context node
  while (true) {
    Node& childNode = stream.nextNode();
    if (!childNode.isValid()) break;

    parentContext.setNode(childNode);
    processNode(parentContext, node);
  }

  return true;
}

}
