/***************************************************************************
                          document.h  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DOCUMENT_H
#define DOCUMENT_H

#include <map>
#include "node.h"
#include "parentnode.h"
#include "documentcontent.h"
/**
  *@author root
  */

namespace limpid {
  class Node;
  class DocumentType;
  class Element;
  class Text;
  class Comment;
  class CDATASection;
  class ProcessingInstruction;
  class XMLDeclaration;
  class Attr;
  class NodeManager;
  class ValidationProcessor;

  class Document : public ParentNode  {
    friend class Text;
  public:
  	Document();
  	Document(const DOMString& namespaceURI, const DOMString& name);
    Document(const Document& refNode);
    Document& operator=(const Document& refNode);
    ~Document();

    static Document failDocument;

    //DOM:
    DOMString getNodeName() { return Node::getNodeName(); }
    DOMString getLocalName() const { return ""; }
    DOMString getPrefix() const { return ""; }
    DOMString getNamespaceURI() const;
    Document& getOwnerDocument() const;
    Element& getDocumentElement();
    Node& appendChild(Node& childNode);

    NodeList getElementsByTagName(const DOMString& name);
    NodeList getElementsByTagName(const DOMString& name, const Node& root);
    NodeList getElementsByTagNameNS(const DOMString& namespaceURI, const DOMString& name);
    NodeList getElementsByTagNameNS(const DOMString& namespaceURI,
          const DOMString& name, const Node& root);
    Element& getElementById(const DOMString& elementId);
    Element& getElementById(const DOMString& elementId, const Node& root);
    Node& importNode(Node& node, bool deep);

    //DOM factory functions:
    Element createElement(const DOMString &tagName);
    Element createElementNS(const DOMString &namespaceURI, const DOMString &tagName);
    Text createTextNode(const DOMString &data);
    Comment createComment(const DOMString &data);
    CDATASection createCDATASection(const DOMString &data);
    ProcessingInstruction createProcessingInstruction(const DOMString &target,
        const DOMString &data);
    Attr createAttribute(const DOMString &name);
    Attr createAttributeNS(const DOMString &namespaceURI, const DOMString &name);

    //non-DOM:
    bool isValid() const { return nodeContent_ != failDocument.nodeContent_; }
    bool operator!() const { return nodeContent_ == failDocument.nodeContent_; }

    DocumentType& getDoctype();
    XMLDeclaration& getXMLDeclaration();
    DOMString getIDAttrName(const DOMString& elementName);

    DOMString getDisplayString() const { return ""; }
    DOMString& resolveEntities(DOMString& string);
    DOMString getText() { return ""; }
    Node& cloneNode(bool deep = false) const;
    void cacheNode(Node& node);

  protected:
    typedef std::map<DOMString, char> Equates;

    Document *previousCopy, *nextCopy; // support linked list of bodies sharing the content
    Document& copy() { return *(new Document(*this)); }
    void connect();
    void linkAfterDocument(const Document& refNode);
    void unlinkThisDocument();
  };
}
#endif
