/***************************************************************************
                          documentbox.cpp  -  description
                             -------------------
    begin                : Wed Mar 8 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "documentbox.h"
#include "writer.h"

namespace limpid {

DocumentBox::DocumentBox(): documentSource_(0), processors_(new Processors) {}

DocumentBox::DocumentBox(DocumentSource& documentSource):
  documentSource_(&documentSource.copy()), processors_(new Processors) {}

void DocumentBox::update(const DOMString& destString) {
  if (destString.empty())
    return;

  Processors::iterator p = processors_->begin();
  Processors::iterator pend = processors_->end();

  while (p != pend)
    (*p++)->update(destString);
}

void DocumentBox::start() {
  Processors::iterator p = processors_->begin();
  Processors::iterator pend = processors_->end();

  while (p != pend)
    (*p++)->start();
}

void DocumentBox::finish() {
  Processors::iterator p = processors_->begin();
  Processors::iterator pend = processors_->end();

  while (p != pend)
    (*p++)->finish();
}

void DocumentBox::process() {
  if (documentSource_) {
    Document& document = documentSource_->getDocument();
    processDocument(document);
  }
  else
    wrt::Errors << "DocumentBox has no source!!!" << wrt::endl;
}

Document& DocumentBox::processDocument(Document& inputDocument) {
  Document& document = inputDocument;
  Processors::iterator p = processors_->begin();
  Processors::iterator pend = processors_->end();

  while (p != pend)
    document = (*p++)->processDocument(document);

  return inputDocument;
}

}
