/***************************************************************************
                          documentcollector.cpp  -  description
                             -------------------
    begin                : Fri Jul 29 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "documentcollector.h"
#include "xpathfactory.h"
#include "boolean.h"
#include "xpathstring.h"
#include "number.h"
#include "nodeset.h"
#include "xpathstring.h"
#include "document.h"
#include "element.h"
#include "dombuilder.h"

#include "stringreader.h"
#include "writer.h"
#include "tokenstream.h"

namespace limpid {

DocumentCollector::DocumentCollector(): components_(new ComponentList) {}

NodeSet DocumentCollector::getNodeSet(Context& context, bool) {
  NodeSet resultSet(true); // safe mode, responsible for created documents
  size_t childCount = components_->size();
  if (!childCount) return resultSet;

  DOMString basePath;
  if (childCount > 1) basePath = components_->at(1)->getString(context);

  XPathExpression& firstExpr = *(components_->at(0));
  if (!firstExpr.isNodeSet()) {
    XPathString sourceString = firstExpr.getString(context);
    collectSimple(sourceString, basePath, resultSet);
  }
  else
    collectFromNodeSet(context, firstExpr, basePath, resultSet);

  return resultSet;
}

void DocumentCollector::collectSimple(const DOMString& sourceString,
    const DOMString& basePath, NodeSet& resultSet) {
  DOMString pathString = basePath;
  if (!pathString.empty() && !pathString.endsWith("/"))
    pathString += "/";
  pathString += sourceString;
  
  DOMBuilder domBuilder(sourceString);
  Document document = domBuilder.build();

  document.getDocumentElement().setAttribute("source-path", sourceString);

  resultSet.addNode(document);
}

void DocumentCollector::collectFromNodeSet(Context& context, XPathExpression& firstExpr,
    const DOMString& basePath, NodeSet& resultSet) {
  NodeSet nodeSet = firstExpr.getNodeSet(context);
  size_t count = nodeSet.getLength();

  for (size_t index = 0; index < count; ++index) {
    DOMString value = nodeSet.getItem(index).getValue();
    collectSimple(value, basePath, resultSet);
  }
}

}
