/***************************************************************************
                          documentcollector.h  -  description
                             -------------------
    begin                : Fri Jul 29 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DOCUMENTCOLLECTOR_H
#define DOCUMENTCOLLECTOR_H

#include <vector>
#include "parentcomponent.h"
#include "context.h"
#include "xpathexpression.h"
#include "domstring.h"
#include "node.h"
#include "xpathstring.h"
#include "boolean.h"
#include "number.h"
#include "nodeset.h"
#include "pointer.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class DocumentCollector: public ParentComponent {
  public:
    DocumentCollector();

    void appendChild(XPathExpression& child) {
     components_->push_back(XPEPointer(&child.copy())); }

    NodeSet getNodeSet(Context& context, bool exact = true);

    Boolean getBoolean(Context& context) {
      return getNodeSet(context).getBoolean(context); }
    XPathString getString(Context& context) {
      return getNodeSet(context).getString(context); }
    Number getNumber(Context& context) {
      return getNodeSet(context).getNumber(context); }
    Expr eval(Context& context) { return getNodeSet(context); }

    DocumentCollector& copy() { return *(new DocumentCollector(*this)); }

  private:
    typedef Pointer<XPathExpression> XPEPointer;
    typedef std::vector<XPEPointer> ComponentList;

    Pointer<ComponentList> components_;

    void collectSimple(const DOMString& sourceString, const DOMString& basePath,
      NodeSet& resultSet);
    void collectFromNodeSet(Context& context, XPathExpression& firstExpr,
      const DOMString& basePath, NodeSet& resultSet);
  };
}

#endif
