/***************************************************************************
                          documentcontent.cpp  -  description
                             -------------------
    begin                : Fri Sep 12 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "document.h"
#include "documentcontent.h"
#include "element.h"
#include "documenttype.h"
#include "systemmanager.h"

namespace limpid {

DocumentContent::DocumentContent(ParentNode& parent):
    ParentContent(Node::DOCUMENT_NODE, "#document", parent),
    controlDocument_(0), xmlDeclaration_(0), documentElement_(0),
    cachedNode_(0), doctype_(0), namespaceURI_(0) {
  try {
    controlDocument_ = static_cast<Document *>(&parent);
  }
  catch (std::bad_alloc) {
    throw;
  }
}

DocumentContent::DocumentContent(const DOMString& uri, const DOMString& name,
      ParentNode& parent):
    ParentContent(Node::DOCUMENT_NODE, name, parent),
  controlDocument_(0), xmlDeclaration_(0), documentElement_(0),
  cachedNode_(0), doctype_(0), namespaceURI_(0) {
  try {
    controlDocument_ = static_cast<Document *>(&parent);
    namespaceURI_ = uri.copyChars();
  }
  catch (std::bad_alloc) {
    delete namespaceURI_;
  }
}

DocumentContent::~DocumentContent() {
  delete cachedNode_;
  delete namespaceURI_;
}

size_t DocumentContent::getSize() {
  size_t size = sizeof(*this);

  if (prefix_) size += (DOMString::strLength(prefix_) + 1) * sizeof(Char);
  if (nodeName_) size += (DOMString::strLength(nodeName_) + 1) * sizeof(Char);
  if (namespaceURI_) size += (DOMString::strLength(namespaceURI_) + 1) * sizeof(Char);
  if (childNodes_) size += childNodes_->getSize();

  return size;
}

}
