/***************************************************************************
                          documentfragment.cpp  -  description
                             -------------------
    begin                : Fri Oct 10 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "documentfragment.h"
#include "parentcontent.h"
#include "document.h"
#include "systemmanager.h"

namespace limpid {

DocumentFragment::DocumentFragment(): ParentNode() {
  try {
    nodeContent_ = new ParentContent(DOCUMENT_FRAGMENT_NODE, "", *this);
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

DocumentFragment::DocumentFragment(const DocumentFragment& refFragment):
      ParentNode(refFragment) {
}

DocumentFragment& DocumentFragment::operator=(const DocumentFragment& refFragment) {
  replaceContent(refFragment);
  return *this;
}

Node& DocumentFragment::appendChild(Node& childNode) {
  if (!systemManager.checkAppendPermission(*this, childNode))
    return failNode;

  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (!contentPtr->childNodes_)
    contentPtr->childNodes_ = new NodeList(); // no parent node for the list!
  return contentPtr->childNodes_->addNode(childNode);
}

Node& DocumentFragment::cloneNode(bool deep) const {
  DocumentFragment *clone = new DocumentFragment();

  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (deep && contentPtr->childNodes_) {
    size_t length = contentPtr->childNodes_->getLength();
    for (size_t i = 0; i < length; i++) {
      Node& child = contentPtr->childNodes_->item(i).cloneNode(deep);
      clone->appendChild(child);
    }
  }

  clone->nodeContent_->documentContent_ = contentPtr->documentContent_;
  getOwnerDocument().cacheNode(*clone);
  return *clone;
}

}
