/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef LIMPIDDOCUMENTSUMMARY_H
#define LIMPIDDOCUMENTSUMMARY_H

#include "nodesummary.h"
#include "document.h"
#include "element.h"
#include "domstring.h"
#include "nodelist.h"
#include "nodestream.h"
#include <vector>
#include <map>
#include <algorithm>
#include "domutils.h"

namespace limpid {

/**
	@author root <john_redmond@optusnet.com.au>
*/
class DocumentSummary: public DOMUtils {

public:
    DocumentSummary() {}
    void setup(Node& rootNode);
    size_t getPathCount() { return stringList_.size(); }
    const DOMString& getPath(size_t index);
    NodeSummary& getSummary(const DOMString& path);
    NodeSummary& getSummary(size_t index);
    bool hasSummary(const DOMString& pathString);
    Document& getDocument() { buildDocument(); return document_; } 

private:
  typedef std::map<DOMString, NodeSummary> SummaryMap;
  typedef std::vector<NodeSummary*> SummaryList;

  Document document_;
  SummaryMap summaryMap_;
  StringList stringList_;
  StringSet stringSet_;

  void setupSummary(Node& node);
  bool haveSummary(const DOMString& key);
  void updateSummary(Node& node, const DOMString& key);
  void enterSummary(Node& node, const DOMString& key);
  void reportSummary(DOMString& key);
  void reportStructure(DOMString& path, StringList& childList);

  bool isLeaf(const Node& node);
  bool hasText(const Node& node);
  bool isFirstSibling(Node& node);
  int countCluster(Node& node);
  int countLeaves(Node& node);
  int countDescendants(Node& node);
  int countSiblings(Node& refNode);

  void addChildPaths(Node& parentNode, StringList& childNames, StringList& childPaths, StringSet& childSet);
  void addSiblingNames(Node& node, const DOMString& nodePath, StringSet& siblingBefore, StringSet& siblingAfter);
  DOMString getMatchString(const Node& node);

  void buildDocument();
  void addElementContent(NodeSummary& nodeSummary, Element& summaryElement);
  void addChildren(NodeSummary& nodeSummary, Element& summaryElement);
  void addBoolean(const DOMString& name, bool state, Element& summaryElement);
  void addCount(const DOMString& name, int number, Element& summaryElement);
};

}

#endif
