/***************************************************************************
                          documenttype.h  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DOCUMENTTYPE_H
#define DOCUMENTTYPE_H

#include "documenttypecontent.h"
#include "namednodemap.h"
#include "url.h"
#include "dtdconverter.h"

class NamedNodeMap;

/**
  *@author John Redmond
  */

namespace limpid {
  class DocumentType : public Node  {

  public:
  	DocumentType(const DOMString &name, const DOMString &publicId,
                        const DOMString &systemId);
    DocumentType(const DocumentType& refNode);
    Node& operator=(const DocumentType& refNode);

  //DOM:
    DOMString getName() const;
    DOMString getNodeName() const;
    const NamedNodeMap& getEntities();
    const NamedNodeMap& getNotations();
    DOMString getPublicId() const;
    DOMString getSystemId() const;
    DOMString getInternalSubset() const;
    Node& cloneNode(bool deep = false ) const;

  //non-DOM:
    NamedNodeMap& getUnparsedEntities();
    const StringMap& getIDMap();
    const StringMap& getEntityMap();
    DOMString& resolveEntities(DOMString& string);
    void setInternalSubset(DOMString&);
    void setSourceURL(URL& url); // set the source URL of the document itself
    void configure();
    DOMString getDisplayString() const;
    bool isValid() { return this != &getDefaultDocumentType(); }
    static DocumentType& getDefaultDocumentType();

  private:
    void applyExternalSource(URL& externalURL);
    void processConstraints(); // process the DTD or Schema
    void applySchema(Document& doc);
    void setEntities(Document& sd);
    void replaceReference(DOMString& string, const DOMString& key, const DOMString& value);
    void setIDs(Document& sd);
    DocumentType& copy() { return *(new DocumentType(*this)); }
    bool isXSDElement(Node& node);
  };
}
#endif
