/***************************************************************************
                          dombuilder.cpp  -  description
                             -------------------
    begin                : Wed Jan 21 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dombuilder.h"
#include "xmlparser.h"
#include "builder.h"

namespace limpid {

DOMBuilder::DOMBuilder(const URL& url, bool checkBalance):
    reader_(new Reader(url)), verbose_(false), processingXSLT_(false), trimmingText_(false),
    normalizingWhitespace_(false), checkingBalance_(checkBalance) {}

DOMBuilder::DOMBuilder(const CharacterReader& reader, bool checkBalance):
    reader_(new CharacterReader(reader)), verbose_(false), processingXSLT_(false), trimmingText_(false),
    normalizingWhitespace_(false), checkingBalance_(checkBalance) {}

DOMBuilder::DOMBuilder(const DOMString& source, bool checkBalance):
    reader_(new Reader(source)), verbose_(false), processingXSLT_(false), trimmingText_(false),
    normalizingWhitespace_(false), checkingBalance_(checkBalance) {}

Document DOMBuilder::build() {
  XMLParser parser(*reader_);
  parser.setProcessingXSLT(processingXSLT_);
  Builder builder(checkingBalance_);

  while (true) {
    Node& node = parser.nextNode();
    if (!node.isValid()) break;

    builder.process(node);
  }

  return builder.getDocument();
}

}
