/***************************************************************************
                          domstring.h  -  description
                             -------------------
    begin                : Wed Jul 2 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DOMSTRING_H
#define DOMSTRING_H

#include "definitions.h"
#include <string>

/**
  *@author John Redmond
  */

namespace limpid {
  class DOMString : public std::basic_string<Char> {
    typedef std::basic_string<Char> CharString;

  public:
    enum {ConversionError = -1};

  	DOMString(): CharString() {}
    DOMString(size_t len, Char c): CharString(len, c) {}
    #ifndef _NARROW_
    DOMString(const Char *str) { if (str) append(str); }
#endif
    DOMString(const char c) { push_back(Char(c)); };
    DOMString(const char *str) { if (str) appendASCII(str); }

    DOMString(const CharString& s): CharString(s) {}
    DOMString(const DOMString& refString): CharString(refString) {}
    DOMString& operator=(const DOMString& refString) {
      if (this != &refString) assign(refString); return *this; }

    DOMString substring(size_t start, size_t len) const;

    #ifndef _NARROW_
    DOMString& operator+=(Char c) { push_back(c); return *this; }
    #endif
    DOMString& operator+=(char c) { push_back(static_cast<Char>(c)); return *this; }

    DOMString& operator+=(const DOMString& str) { append(str); return *this; }
    #ifndef _NARROW_
    DOMString& operator+=(const Char* str) { append(str); return *this; }
    #endif
    DOMString& operator+=(const char* str) { return appendASCII(str); }

    bool operator==(const DOMString& rhs) const { return !compare(rhs); }
    bool operator!=(const DOMString& rhs) const { return compare(rhs); }
    bool operator<(const DOMString& rhs) const { return compare(rhs) < 0; }
    bool operator==(const char* s) const;
    bool operator!=(const char* s) const;

    static DOMString failString;

  //array functions:
    Char *copyChars() const;
    const char* getCString() const;

  //query functions:
    size_t getLength() const { return size(); }
    size_t indexOf(const DOMString& substring, size_t startPos = 0) const;
    size_t lastIndexOf(const DOMString& substring, size_t startPos = npos) const;
    bool contains(const DOMString& substring) const;
    bool startsWith(const DOMString& substring) const;
    bool endsWith(const DOMString& substring) const;

  //manipulation functions:
    DOMString& trim();
    DOMString& normalizeWhitespace();
    DOMString& change(const DOMString& oldString, const DOMString& newString);
    DOMString& change(size_t offset, size_t count, const DOMString& newString);
    DOMString& insertString(size_t pos, const DOMString& newString);
    DOMString& remove(size_t pos, size_t count);
    DOMString& toLowerCase();
    DOMString& toUpperCase();
    DOMString& capitalize();
    DOMString& reverse();
    DOMString& resolveEntities(bool includeLT = false);
    DOMString getDelimitedString(size_t startPos);
    int toInt(int base = 10) const;
    DOMString& resolveBra();
    DOMString& escapeBra();
    DOMString& escapeDelimiters();
    DOMString& escapeTex();
    size_t getSize() const { return sizeof(*this) + size() * sizeof(Char); }
    bool isValid() { return this != &failString; }

  //utility static functions:
    static size_t strLength(const Char* ptr) {size_t size = 0; while (*ptr++) ++size; return size; }
    static bool isWhitespace(int c) { return (c == ' ' || c == '\t' || c == '\n' || c == '\r'); }

  protected:
    DOMString &appendASCII(const char* str);
    DOMString& normalizeGaps();

  private:
    bool doResolve(DOMString& string, bool includeLT = false);
    bool doExtendedChar(DOMString& string);
    int getExtendedChar(DOMString& entityName);
};

  typedef DOMString String;
}
#endif
