/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDDOMUTILS_H
#define LIMPIDDOMUTILS_H

#include "element.h"
#include "domstring.h"
#include "nodelist.h"

namespace limpid {

/**
	@author root john_redmond@optusnet.com.au>
*/

class DOMUtils{
public:
  DOMUtils() {}

static  Node& getChild(const DOMString& name, Node& parentNode);
static NodeList getChildren(const DOMString& name, Node& parentNode);
static  DOMString getChildValue(const DOMString& name, Node& parentNode) {
    return getChild(name, parentNode).getValue(); }
static int getParentPosition(const Node& node);

static  Element& addElement(Element& parentElement, const DOMString& elementName);
static  Element& addElement(const DOMString& elementName, Element& parentElement) {return addElement(parentElement, elementName); }

static  Element& addLeaf(const DOMString& name, const DOMString& value, Node& parentNode);
static  void addText(Element& parentElement, const DOMString& textString);
static  void addText(const DOMString& textString, Element& parentElement) { addText(parentElement, textString); }
static  void addComment(Element& parentElement, const DOMString& commentString);

static  void moveContent(Element& sourceElement, Element& destElement);
static  void spillContent(Node& node);
static  bool isLeaf(const Node& node);
static  bool hasText(const Node& node);
static  bool hasMixedContent(Node& parentNode);
static int getDepth(Node& node);
static int getChildCount(Node& node) { return node.getChildNodes().getLength(); }
static int getSiblingCount(Node& node) { return getChildCount(node.getParentNode()) - 1; }
static int getPosition(Node& node);
static bool isXHTMLNode(Node& node);
static bool isXHTMLName(const DOMString& nodeName);
static Node& getLastSibling(Node& node) { return node.getParentNode().getLastChild(); }

static bool hasTextNeighbor(const Node& node) { return hasText(node.getPreviousSibling()) || hasText(node.getNextSibling()); }
static bool isSingle(const Node& node) { return getChildCount(node.getParentNode()) == 1 ; }
static bool hasTextBefore(const Node& node) { return node.getPreviousSibling().getNodeType() == Node::TEXT_NODE;}
static bool hasNextSibling(const Node& node) { return node.getNextSibling().isValid(); }
static bool hasTextAfter(const Node& node) { return node.getNextSibling().getNodeType() == Node::TEXT_NODE; }
};

}

#endif
