/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDDOTPARSER_H
#define LIMPIDDOTPARSER_H


#include <deque>
#include "parser.h"
#include "pointer.h"
#include "reader.h"
#include "node.h"
#include "element.h"
#include "text.h"
#include "endnode.h"
#include "domstring.h"

namespace limpid {


/**
  @author root <john_redmond@optusnet.com.au>
*/

class DotParser : public Parser
{
public:
  DotParser(const DOMString& fileName);
  DotParser(const Reader& reader);

  Node& nextNode();
  DotParser& copy() { return *(new DotParser(*this)); }

private:

    typedef std::deque<DOMString> NameStack;
    typedef std::deque<int> IntegerStack;
    NameStack tagStack_;
    IntegerStack nestingStack_;

    Pointer<Node> node_;
    Pointer<EndNode> endNode_;
    Pointer<Reader> reader_;
    Element* lastElement_;
    Node* root_;
    DOMString rootName_, activeString_, tagName_, blockName_;
    int blockDepth_, currentDepth_, state_, lineNo_;

    DOMString getDotString();
    DOMString getUSString();
    DOMString getWord();
    DOMString getLine();
    DOMString getTextString();
    DOMString getAttributeString();
    DOMString getHashString();
    DOMString getDelimitedString(const DOMString& delimiters);

    void setAny();
    void setXMLDeclaration();
    void setDocumentType();
    void setBlock();
    void setContent();
    void setTagStart();
    void setAttribute();
    void pushElement (const DOMString& name, bool isParent);
    void pushElement (const DOMString& name, const DOMString& attrName, const DOMString& attrValue);
    void popStacks();
    void skipWhitespace();
    Char peekChar();
};

}

#endif
