/***************************************************************************
                          sxfwriter.h  -  description
                             -------------------
    begin                : Wed Oct 24 2007
    copyright            : (C) 2007 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LIMPIDDOTWRITER_H
#define LIMPIDDOTWRITER_H

#include "dumper.h"
#include "domutils.h"
#include "writer.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class DOMString;
  class Element;
  class EndNode;

  class DotWriter : public Dumper, public DOMUtils  {
  public:
    DotWriter();
    DotWriter(const DOMString& destString);
    DotWriter(CharacterWriter& characterWriter);

    Node& process(Node& node);
    void flush() { if (writer_) writer_->flush(); }
    void update(const DOMString& destString);
    DotWriter& copy() { return *(new DotWriter(*this)); }

  private:
    Pointer<CharacterWriter> writer_;
    int depth_;
    Node* blockedNode_;
    DOMString rootName_;
    bool started_;

    void processElement(Node& node);
    void processEndNode(Node& node);
    void processText(Node& node);
    void processCDATASection(Node& node);
    void processXMLDeclaration(Node& node);
    void processProcessingInstruction(Node& node);
    void processComment(Node& node);
    void processDocumentType(Node& node);
    void processAttributes(Node& node);
    void unblockProcessing(Node& node);
    DOMString& escapeString(DOMString& valueString);
    DOMString& escapePreString(DOMString& valueString);
    DOMString getMixedString(Node& parentNode);
  };
}

#endif
