/***************************************************************************
                          dtdconverter.h  -  description
                             -------------------
    begin                : Wed Sep 24 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DTDCONVERTER_H
#define DTDCONVERTER_H

#include <map>
#include <deque>
#include <set>
#include <list>
#include "domstring.h"
#include "iobase.h"
#include "document.h"
#include "element.h"
#include "tokenstream.h"
#include "xmlwriter.h"
#include "domutils.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class URL;
  class CharacterReader;

  class DTDConverter: public DOMUtils {
  public:
  	DTDConverter(const URL& url);
    DTDConverter(CharacterReader& cstream);

    Document& process();
    Document& getDocument();

  private:
    typedef std::deque<TokenStream> InputStack;
    typedef std::set<DOMString> NameSet;
    typedef std::map<DOMString, Element> ElementMap;
    typedef std::pair<DOMString, Element> ElementPair;
    typedef std::map<DOMString, DOMString> ParameterMap;
    typedef std::list<DOMString> StringList;
    typedef std::map<DOMString, StringList> StringMap;
    typedef std::set<DOMString> StringSet;
    typedef std::list<StringList> SuperStringList;

    TokenStream input;
    InputStack inputStack;
    Document document;
//    ElementMap attributeMap, elementMap;
    ParameterMap parameterMap;
    StringMap elementMap, attributeMap, entityMap;
    StringList elementList;
    StringSet elementSet, attributeSet, childSet, leafSet;
    SuperStringList inputList;

    DOMString& nextToken();
    DOMString& peekToken();
    DOMString& nextTokenSimple();
    DOMString& peekTokenSimple();
    DOMString getDelimitedString();
    void bumpInput() { input.bump(); }
    void redirectInput();
    void getStringInput();

    void processTokens();
//    void createElement();
    void createEntity();
    void createNotation();
    void createDocumentation(DOMString s);

//    Element getElementEntry(DOMString& elementName);
//    Element getAttrEntry(DOMString& attrType);
    Element getClosure(DOMString& elementName, int depth = 1);

    void copyAttributes(Element& source, Element& dest);
    void copyChildren(Element& source, Element& dest);
    void setElementFrequency(Element& element);

    void createElementAttrs();
    void addAttrContent(DOMString& elementName, Element& parent);//, DOMString attrName);
    void setAttrNodeDefault(Element& attrElement);
    void setAttrNodeStatus(Element& attrElement);

    void processAttrClosure(Element& childAttrElement);
    void setAttributeNodeType(Element& attributeElement);
    void setAttributeNodeStatus(Element& attributeElement);

    void setParameterEntity();
    DOMString getParameter(DOMString& key);
    void addIDs(Element& entityElement);
    void addNDATA(Element& entityElement);

    Node& getAppendPoint(Node& entry);
    void removeEmptyTypes();
    void replaceReferences();
    void derefElementAttributes(Node& elementRoot);
    void derefElementElements(Node& elementRoot);
    DOMString findOrigin();
    void promoteOrigins();
    void promoteOrigin();
    void swallow();

    void initialize();
    void buildRows();
    void collectInputLists();
    void mapLists();
    void buildElements();
    void addEntryElement();
    bool addRow(StringList& tokenList, DOMString& typeString, Element& parent);
    int addClosure(StringList& tokenList, Element& parent, Element& rootElement);
    bool addAttributes(StringList& tokenList, Element& parent);
    bool addItem(DOMString& itemName, Element& parent, StringList& tokenList);
    void setEntities();
    void updateReferences();

    void getTokenList(StringList& resultList);
    void displayList(StringList& resultList);
    DOMString peekNext(StringList& tokenList);
    DOMString getNext(StringList& tokenList);
    DOMString getTypeName(const DOMString& name);
    void addCount(Element& element, StringList& tokenList);
  };
}
#endif
