/***************************************************************************
                          element.h  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ELEMENT_H
#define ELEMENT_H

#include "parentnode.h"

/**
  *@author root
  */

namespace limpid {
  class NamedNodeMap;
  class Attr;

  class Element : public ParentNode  {
  public:
    friend class Attr;
    friend class ProcessBox;

    //DOM:
    Element();
    Element(const DOMString& name);
    Element(const DOMString& namespaceURI, const DOMString& name);
    Element(const Element& refNode);
    Element& operator=(const Element& refNode);

    static Element failElement;

    DOMString getTagName() const { return getNodeName(); }
    DOMString getNamespaceURI() const;

    void setAttribute(const DOMString& name, const DOMString& value);

    void removeAttribute(const DOMString& name);

    Attr& getAttributeNode(const DOMString& name) const;
    Attr& getAttributeNodeNS(const DOMString& namespaceURI, const DOMString& name) const;

    Attr& setAttributeNode(Attr& attr);
    Attr& setAttributeNodeNS(Attr& attr);

    Attr& removeAttributeNode(Attr& oldAttr);

    DOMString getAttribute(const DOMString& name) const;
    DOMString getAttributeNS(const DOMString& namespaceURI, const DOMString& name) const;
    void setAttributeNS(const DOMString& namespaceURI,
            const DOMString& name, const DOMString& value);
    void removeAttributeNS(const DOMString& namespaceURI, const DOMString& name);

    Node& cloneNode(bool deep = false) const;
    NodeList getElementsByTagName(const DOMString& name) const;
    NodeList getElementsByTagNameNS(const DOMString& namespaceURI,
            const DOMString& localName) const;

    //non-DOM:
    bool isValid() const { return nodeContent_ != failElement.nodeContent_; }
    bool operator!() const { return nodeContent_ == failElement.nodeContent_; }
    void setIsParent(bool flag) { nodeContent_->isParent_ = flag; }

    DOMString getDisplayString() const;
    DOMString getRawValue() const;
    DOMString getValue() const; // support for XSLT
    NamedNodeMap& getAttributes() const;
    void setDocumentContent(DocumentContent& documentContent);

  private:
    void setNamespaceURI(const DOMString& namespaceURI);
    DOMString findNamespaceURI(DOMString& prefix) const;
    DOMString getAttributeString() const;
    Element& copy() { return *(new Element(*this)); }
    void clearLinks();
  };
}
#endif
