/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "elementdropper.h"
#include "element.h"
#include "nodelist.h"
#include "nodestream.h"
#include "liststream.h"

namespace limpid {

Document& ElementDropper::processDocument(Document& inputDocument) {
  while (process(inputDocument)) {}
  return inputDocument;
}

int ElementDropper::process(Document& inputDocument) {
  NodeList emptyList;
  NodeStream nodeStream(inputDocument.getDocumentElement());

  // first collect references to all the empty elements
  while (true) {
    Node& node = nodeStream.nextNode();
    if (!node.isValid()) break;

    if (node.getNodeType() != Node::ELEMENT_NODE) continue;

    Element& element = static_cast<Element &>(node);
    if (element.getChildNodes().getLength() == 0) {
      emptyList.addNode(node);
    }
  }

  if (!emptyList.getLength())
    return 0;

  ListStream emptyStream(emptyList);

  // now remove all the empty elements from the input document
  while (true) {
    Node& node = emptyStream.nextNode();
    if (!node.isValid()) break;

    node.getParentNode().removeChild(node);
  }

  return emptyList.getLength();

}

}
