/***************************************************************************
                          elementtemplate.h  -  description
                             -------------------
    begin                : Fri Jul 1 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ELEMENTTEMPLATE_H
#define ELEMENTTEMPLATE_H

#include "compoundtemplate.h"
#include "template.h"
#include "domstring.h"
#include "expr.h"
#include "xpathfactory.h"
#include "configurationset.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class XSLContext;
  class Node;
  class Element;
  class CompoundTemplate;

  class ElementTemplate : public Template  {
    public:
    	ElementTemplate(const DOMString& name, Expr& nameExpr,
        bool hasChildren, bool isHTML = false);

      bool process(XSLContext& parentContext);
      bool processNode(XSLContext& parentContext, Node& node);

      void addTemplate(Template& childTemplate);
      void setNamespaces(ConfigurationSet& configurationSet) {
        namespaceMap_ = configurationSet.namespaceMap; }
      ElementTemplate& copy() { return *(new ElementTemplate(*this)); }

    private:
      typedef std::map<DOMString, DOMString> StringMap;

      Expr nameExpr_;
      CompoundTemplate attributes_;
      CompoundTemplate templates_;
      bool hasChildren_, isHTML_;
      StringMap namespaceMap_;


      Element getElement(XSLContext& parentContext);
      bool isElement() { return true; }
      bool isEmptyTag(DOMString tagName);
      void outputAliases(CharacterWriter& writer);
      void setAliases(Element& element);
      bool processEmptyElement(DOMString& elementName, XSLContext& parentContext);
      bool processParentElement(DOMString& elementName, XSLContext& parentContext);
 };
}

#endif
