/***************************************************************************
                          endnode.cpp  -  description
                             -------------------
    begin                : Mon Oct 6 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "endnode.h"
#include "endcontent.h"

namespace limpid {

EndNode::EndNode(): ParentNode() {
  try {
    nodeContent_ = new EndContent("default-end");
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

EndNode::EndNode(const DOMString& name): ParentNode() {
  try {
    nodeContent_ = new EndContent(name);
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

EndNode::EndNode(const EndNode& refNode): ParentNode(refNode) {
}

EndNode& EndNode::operator=(const EndNode& refNode) {
  replaceContent(refNode);
  return *this;
}

void EndNode::update(Node& refNode) {
  EndContent *contentPtr = static_cast<EndContent *>(nodeContent_);
  if (refNode.getNodeType() == DOCUMENT_NODE)
    contentPtr->nodeName_ = 0;
  else
    contentPtr->setName(refNode.getNodeName());

  contentPtr->hasNonTextChildren_ = refNode.hasNonTextChildren();
  contentPtr->startNode_ = &refNode;
}

Node& EndNode::getStartNode() {
  EndContent *contentPtr = static_cast<EndContent *>(nodeContent_);
  if (contentPtr->startNode_)
    return *contentPtr->startNode_;
  else
    return Node::failNode;
}

bool EndNode::hasNonTextChildren() const {
  EndContent *contentPtr = static_cast<EndContent *>(nodeContent_);
  return contentPtr->hasNonTextChildren_;
}

DOMString EndNode::getDisplayString() const {
  EndContent *contentPtr = static_cast<EndContent *>(nodeContent_);
  if (!contentPtr->nodeName_) return "";  // this is an end node for a document

  DOMString value = "</";
  value += getNodeName();
  value += ">";
  return value;
}

}
