/***************************************************************************
                          entitycontent.cpp  -  description
                             -------------------
    begin                : Wed Sep 24 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "entitycontent.h"
#include "entity.h"

namespace limpid {

EntityContent::EntityContent(const DOMString& name, const DOMString& pubId,
      const DOMString& sysId, const DOMString& notName, ParentNode& parent):
      ParentContent(Node::ENTITY_NODE, name, parent),
      publicId_(0), systemId_(0), notationName_(0) {
  try {
    publicId_ = pubId.copyChars();
    systemId_ = sysId.copyChars();
    notationName_ = notName.copyChars();
  }
  catch (std::bad_alloc) {
    deleteContent();
    throw;
  }
}

EntityContent::~EntityContent() {
  deleteContent();
}

void EntityContent::deleteContent() {
  delete [] publicId_;
  delete [] systemId_;
  delete [] notationName_;
}

size_t EntityContent::getSize() {
  size_t size = sizeof(*this);

  if (prefix_) size += (DOMString::strLength(prefix_) + 1) * sizeof(Char);
  if (nodeName_) size += (DOMString::strLength(nodeName_) + 1) * sizeof(Char);
  if (childNodes_) size += childNodes_->getSize();
  if (publicId_) size += (DOMString::strLength(publicId_) + 1) * sizeof(Char);
  if (systemId_) size += (DOMString::strLength(systemId_) + 1) * sizeof(Char);
  if (notationName_) size += (DOMString::strLength(notationName_) + 1) * sizeof(Char);

  return size;
}

}
