/***************************************************************************
                          entityreader.cpp  -  description
                             -------------------
    begin                : Mon Sep 6 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "entityreader.h"
#include "node.h"
#include "xmldeclaration.h"
#include "domstring.h"
#include "url.h"
#include "xmlparser.h"
#include "writer.h"

namespace limpid {

EntityReader::EntityReader(): Reader(), hasRead(false) {}

EntityReader::EntityReader(const URL& url):
  Reader(url), hasRead(false) {}

EntityReader::EntityReader(const DOMString& source):
  Reader(source), hasRead(false) {}

int EntityReader::nextChar() {
  if (!hasRead) {
    int firstChar = Reader::nextChar();
    input_->seekg(0);
    hasRead = true;

    if (firstChar == '<') {
      XMLParser parser(*this);
      Node& node = parser.nextNode();
      if (node.getNodeType() == Node::PROCESSING_INSTRUCTION_NODE &&
        node.getNodeName() == "xml") {
        DOMString encodingString = (static_cast<XMLDeclaration &>(node)).getEncoding();
        setEncoding(IOBase::getEncoding(encodingString));
      }
      else input_->seekg(0); // reset the input
    }
  }
  return Reader::nextChar();
}

}

