/***************************************************************************
                          expressionsorter.cpp  -  description
                             -------------------
    begin                : Sat May 28 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "expressionsorter.h"
#include <map>
#include "domstring.h"
#include "xslcontext.h"
#include "nodeset.h"
#include "xpathfactory.h"
#include "converter.h"

namespace limpid {

ExpressionSorter::ExpressionSorter(const DOMString& selectString,
      const DOMString& typeString, const DOMString& orderString):
      isNumeric_(true), isAscending_(true),
      expr_(&XPathFactory::getExpr(selectString).copy()) {
  if (typeString != "number") isNumeric_ = false;
  if (orderString == "descending") isAscending_ = false;
  expr_ = &XPathFactory::getExpr(selectString).copy();
}

void ExpressionSorter::process(XSLContext& parentContext) {
  if (isNumeric_) processNumeric(parentContext);
  else processString(parentContext);
}

void ExpressionSorter::processNumeric(XSLContext& parentContext) {
  typedef std::multimap<double, Node*> DoubleMap;
  DoubleMap doubleMap;

  context_.setParent(parentContext);
  context_.setNode(parentContext.getNode());

  NodeSet& nodeSet = parentContext.getCurrentSet();
  size_t length = nodeSet.getLength();
  for (size_t index = 0; index < length; ++index) {
    Node& node = nodeSet.getItem(index);
    context_.setNode(node);

    double keyVal = Converter::toInt(expr_->getString(context_));
    doubleMap.insert(std::pair<double, Node*>(keyVal, &node));
  }
  // now empty the node set in the context and re-add the nodes in the required order
  nodeSet.clear();

  if (isAscending_) {
    DoubleMap::iterator p = doubleMap.begin();
    while (p != doubleMap.end()) {
      nodeSet.addNode(*(p->second));
      p++;
    }
  }
  else {
    DoubleMap::reverse_iterator p = doubleMap.rbegin();
    while (p != doubleMap.rend()) {
      nodeSet.addNode(*(p->second));
      p++;
    }
  }
}

void ExpressionSorter::processString(XSLContext& parentContext) {
  typedef std::multimap<DOMString, Node*> StringMap;
  StringMap stringMap;

  XSLContext context;
  context.setParent(parentContext);
  context.setNode(parentContext.getNode());

  NodeSet& nodeSet = parentContext.getCurrentSet();

  size_t length = nodeSet.getLength();
  for (size_t index = 0; index < length; ++index) {
    Node& node = nodeSet.getItem(index);
    context.setNode(node);

    DOMString keyString = expr_->getString(context);
    stringMap.insert(std::pair<DOMString, Node*>(keyString, &node));
  }

// now empty the node set in the context and re-add the nodes in the required order
  nodeSet.clear();

  if (isAscending_) {
    StringMap::iterator p = stringMap.begin();
    while (p != stringMap.end()) {
      nodeSet.addNode(*(p->second));
      p++;
    }
  }
  else {
    StringMap::reverse_iterator p = stringMap.rbegin();
    while (p != stringMap.rend()) {
      nodeSet.addNode(*(p->second));
      p++;
    }
  }
}

}
