/***************************************************************************
                          expressionsorter.h  -  description
                             -------------------
    begin                : Sat May 28 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************

ExpressionSorter is designed specifically to support <xsl:sort> in the XSLT
specification. Note that it does not use the sort or stable-sort algorithm of
the STL because of the heavy overhead of repeated calculation of either the
double or string values corresponding to the nodes being sorted.

Instead, these values are used as keys for inserting the nodes into a multimap.
After all nodes have been added to the map. the node set is cleared and the nodes
re-added in the desired sequence, using either a forward or reverse iterator to
traverse the map. This has the advantage that the keys are calculated only once
for each node. Moreover, the sort is stable.
*/

#ifndef EXPRESSIONSORTER_H
#define EXPRESSIONSORTER_H

#include "pointer.h"
#include "expr.h"
#include "xslcontext.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class DOMString;
  class Expr;

  class ExpressionSorter {
    public:
    	ExpressionSorter(const DOMString& selectString, const DOMString& typeString,
          const DOMString& orderString);

      void process(XSLContext& ctxt);
      ExpressionSorter& copy() { return *(new ExpressionSorter(*this)); }

    private:
      ExpressionSorter() {}

      bool isNumeric_, isAscending_;
      Pointer<Expr> expr_;
      XSLContext context_;

      void processNumeric(XSLContext& ctxt);
      void processString(XSLContext& ctxt);
  };
}


#endif
