/***************************************************************************
                          followingstream.cpp  -  description
                             -------------------
    begin                : Wed May 26 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "followingstream.h"
#include "node.h"
#include "writer.h"

namespace limpid {

FollowingStream::FollowingStream(bool fwd): forward_(fwd), includeChildren_(true),
    refNode_(&Node::failNode)   {}

FollowingStream::FollowingStream(const Node& root, bool forward): forward_(forward),
    includeChildren_(forward) {
  setRoot(root);
}

void FollowingStream::setRoot(const Node& root) {
  includeChildren_ = true;
  refNode_ = const_cast<Node *>(&root);
}

Node& FollowingStream::nextNode() {
  if (refNode_->isValid()) refNode_ = &getNext();
  includeChildren_ = true;

  return *refNode_;
}

Node& FollowingStream::getNext() {
  if (forward_) return getNodeFollowing(*refNode_);
  return getNodePreceding(*refNode_);
}

Node& FollowingStream::getNodePreceding(Node& refNode) {
  Node* previousSibling = &refNode.getPreviousSibling();

  if (previousSibling->isValid()) {
    Node* lastChild = &previousSibling->getLastChild();
    if (lastChild->isValid()) return *lastChild;
    else return *previousSibling;
  }

  return refNode.getParentNode();
}

Node& FollowingStream::getNodeFollowing(Node& refNode) {
  if (includeChildren_) {
    Node* firstChild = &refNode.getFirstChild();
    if (firstChild->isValid()) return *firstChild;
  }

  Node* nextSibling = &refNode.getNextSibling();
  if (nextSibling->isValid()) return *nextSibling;

  Node* parent = &refNode;
  while (true) {
    parent = &parent->getParentNode();
    if (!parent->isValid()) break;

    Node* parentSibling = &parent->getNextSibling();
    if (parentSibling->isValid()) return *parentSibling;
  }

  return Node::failNode;
}

}

