/***************************************************************************
                          graphnavigator.h  -  description
                             -------------------
    begin                : Sun Dec 7 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHNAVIGATOR_H
#define GRAPHNAVIGATOR_H

#include "copied.h"
#include "graphnodes.h"
#include "node.h"
#include "domstring.h"
#include "nodelist.h"
#include "liststream.h"
#include "writer.h"
#include "pointer.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class GraphNavigator: public Copied {
  public:
    GraphNavigator();
  	GraphNavigator(GraphNode& rootNode);

    static GraphNavigator failNavigator;

    void setMixed(bool flag) { mixed = flag; }
    void reset() { startWith(*root); }
    void startWith(GraphNode& graphNode) { position = &graphNode; }

    bool navigate(NameList& trail);
    bool navigate(Node& parent);

    void setWriter(Writer& wr) { writer = new Writer(wr); }
    NameList getOptions() { return position->getOptions(); }
    void reportError(Node& node);
    bool isValid() { return root; }
    size_t getSize();
    GraphNavigator& copy() { return *(new GraphNavigator(*this)); }

  private:
    Pointer<Writer> writer;
    Pointer<GraphNode> root;
    GraphNode *position; // this is volatile
    bool mixed;
    DOMString currentName;
    size_t errorLine;

    GraphNode& find(const DOMString& name);
    bool hasEnd() { return position->isEnd(); }
    DOMString getCurrentName() { return currentName; }
    DOMString getPosition() { return currentName; }
    void showOptions();
  };
}
#endif
