/***************************************************************************
                          htmldeclaration.cpp  -  description
                             -------------------
    begin                : Mon Aug 29 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "htmldeclaration.h"
#include "xmldeclarationcontent.h"
#include "document.h"
#include "xmldeclarationcontent.h"
#include "domstring.h"

namespace limpid {

HTMLDeclaration::HTMLDeclaration(const DOMString& version,
    const DOMString& encoding, const DOMString& standalone):
    ProcessingInstruction("", "") {
  try {
    nodeContent_ = new XMLDeclarationContent(version, encoding, standalone);
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

HTMLDeclaration::HTMLDeclaration(const HTMLDeclaration& refNode):
  ProcessingInstruction(refNode) {
}

HTMLDeclaration& HTMLDeclaration::operator=(const HTMLDeclaration& refNode) {
  replaceContent(refNode);
  return *this;
}

DOMString HTMLDeclaration::getData() const {
  XMLDeclarationContent *contentPtr =
    static_cast<XMLDeclarationContent *>(nodeContent_);
  DOMString data;
  if (contentPtr->version_) {
    data += " version=\"";
    data += contentPtr->version_;
    data += '\"';
  }
  if (contentPtr->encoding_) {
    data += " encoding=\"";
    data += contentPtr->encoding_;
    data += '\"';
  }
  if (contentPtr->standalone_) {
    data += " standalone=\"";
    data += contentPtr->standalone_;
    data += '\"';
  }

  return data;
}

DOMString HTMLDeclaration::getVersion() const {
  XMLDeclarationContent *contentPtr = static_cast<XMLDeclarationContent *>(nodeContent_);
  return contentPtr->version_;
}

DOMString HTMLDeclaration::getEncoding() const {
  XMLDeclarationContent *contentPtr = static_cast<XMLDeclarationContent *>(nodeContent_);
  return contentPtr->encoding_;
}

DOMString HTMLDeclaration::getStandAlone() const {
  XMLDeclarationContent *contentPtr = static_cast<XMLDeclarationContent *>(nodeContent_);
  return contentPtr->standalone_;
}

DOMString HTMLDeclaration::getDisplayString() const {
  DOMString s("<?html");
  DOMString data = getData();
  s += data;
  s += ">";
  return s;
}

Node& HTMLDeclaration::cloneNode(bool) const {
  DOMString version = getVersion();
  DOMString encoding = getEncoding();
  DOMString alone = getStandAlone();
  HTMLDeclaration *copyNode = new HTMLDeclaration(version, encoding, alone);

  copyNode->nodeContent_->documentContent_ = nodeContent_->documentContent_;
  getOwnerDocument().cacheNode(*copyNode);

  return *copyNode;
}

HTMLDeclaration HTMLDeclaration::failDeclaration = HTMLDeclaration("", "", "");

}
