/***************************************************************************
 *   Copyright (C) 2007 by John  Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDHTMLSTYLER_H
#define LIMPIDHTMLSTYLER_H

#include "documentprocessor.h"
#include "domutils.h"
#include "documentsummary.h"
#include "element.h"
#include "domstring.h"
#include "nodelist.h"
#include <vector>
#include <map>
#include <algorithm>

namespace limpid {

/**
	@author root <john_redmond@optusnet.com.au>
*/

class HTMLStyler : public DocumentProcessor, public DOMUtils {
public:
  HTMLStyler():
    cssScreenPath_("base.css"), cssPrintPath_(""), expandingStyles_(false) {}
  HTMLStyler(const DOMString& cssScreenPath, const DOMString& cssPrintPath):
    cssScreenPath_(cssScreenPath), cssPrintPath_(cssPrintPath), expandingStyles_(false) {}
  void setExpanding(bool expandingStyles) { expandingStyles_ = expandingStyles; }
  Document& processDocument(Document& document);
  HTMLStyler& copy() { return *(new HTMLStyler(*this)); }

private:
  DOMString cssScreenPath_, cssPrintPath_;
  bool expandingStyles_;
  Document outputDocument_;

  void buildDocument(Document& sourceDocument);
  void addDocumentTemplateElement(Element& parentElement);
  void addDefinedTemplate(Node& node, Element& documentElement);
  void addTemplateElement(const DOMString& matchString, const DOMString& tagString, const DOMString& nameString, const DOMString& depthString, Element& parentElement);
  Element getDocumentElement();
  void addLink(Element& parentElement, const DOMString& use,
    const DOMString& path, const DOMString& type);
  void expandStyle(Element& parentElement, const DOMString& use,
    const DOMString& path, const DOMString& type);
  void addTag(Element& parentElement, const DOMString& matchString, const DOMString& tagName);
};

}

#endif
