/***************************************************************************
                          idchecker.cpp  -  description
                             -------------------
    begin                : Sat Aug 6 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "idchecker.h"
#include "document.h"
#include "element.h"

namespace limpid {

IDChecker::IDChecker(const Expr& expr): expr_(expr) {}

NodeSet IDChecker::getNodeSet(Context& context, bool exact) {
  return process(context.getNode(), context, exact);
}

NodeSet IDChecker::processNodeSet(NodeSet& nodeSet, Context& context, bool exact) {
  XPathString refString = expr_.getString(context);
  size_t length = nodeSet.getLength();

  for (size_t index = 0; index < length; ++index) {
    Node& node = nodeSet.item(index);
    NodeSet resultSet = processNode(node, refString, exact);
    if (resultSet.getLength()) return resultSet;
  }

  return NodeSet(); // return empty set for failed match
}

NodeSet IDChecker::processNode(Node& node, XPathString& refString, bool exact) {
  NodeSet resultSet;

  if (node.getNodeType() != Node::ELEMENT_NODE) return resultSet;

  Element& element = static_cast<Element &>(node);
  Document& document = element.getOwnerDocument();
  DOMString nodeName = element.getNodeName();
  DOMString requiredAttrName = document.getIDAttrName(nodeName);

  if (requiredAttrName.empty()) return resultSet;

  if (!exact || element.getAttribute(requiredAttrName) == refString)
    resultSet.addNode(node);

  return resultSet;
}

}
