/***************************************************************************
                          inputstream.cpp  -  description
                             -------------------
    begin                : Thu Jun 24 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "inputstream.h"
#include "writer.h"

using namespace std;

namespace limpid {

InputStream::InputStream(bool useConsole): url_(useConsole) {
  setInput(url_);
  testGoodInput();
}

InputStream::InputStream(const URL& url): url_(url) {
  setInput(url_);
  testGoodInput();
}

InputStream::InputStream(ifstream* rawInput): input_(rawInput), fileInput_(rawInput) {
  testGoodInput();
}

int InputStream::nextByte() {
  if (!input_ || !input_->good()) {
    if (fileInput_) fileInput_->close();
    return -1;
  }

  char c;
  input_->get(c);
  return c;
}

void InputStream::testGoodInput() {
  if (fileInput_ && !fileInput_->good()) {
    wrt::Errors << "BAD INPUT FROM STREAM: terminating!" << wrt::endl;
    fileInput_->close();
    exit(1);
  }
}

void InputStream::setInput(URL& url) {
  if (!url.isValid()) {
    input_ = 0;
    wrt::Errors << url.getPath() << ": BAD INPUT FROM SUPPLIED STREAM: terminating!" << wrt::endl;
  }
  else {
    input_ = &url.getInput();
    if (!url.isDefault())
      fileInput_= static_cast<std::ifstream *>(input_);  // it must be a file stream;
  }
}

}
