/***************************************************************************
                          inputstream.h  -  description
                             -------------------
    begin                : Thu Jun 24 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INPUTSTREAM_H
#define INPUTSTREAM_H

#include <fstream>
#include "pointer.h"
#include "definitions.h"
#include "domstring.h"
#include "copied.h"
#include "iobase.h"
#include "url.h"

/**
  *@author John Redmond
  */

using std::istream;
using std::ifstream;

namespace limpid {
  class InputStream: public Copied, public IOBase {
    friend class Reader;
  public:
    InputStream(bool useConsole = true);
    InputStream(const URL& url);
    InputStream(std::ifstream* rawInput);

    int nextByte();
    void unget() { input_->unget(); }
    bool isValid() const { return *input_; }
    bool isDefault() const { return url_.isDefault(); }
    std::istream& getInput() { return *input_; }
    URL getURL() { return url_; }
    void close() { if (!url_.isDefault()) fileInput_->close(); }
    InputStream& copy() { return *(new InputStream(*this)); }

  protected:
    istream *input_;
    Pointer<ifstream> fileInput_; // this is a sentinel for input_
    URL url_;

    void setInput(URL& url);
    void testGoodInput();
  };
}
#endif
