/***************************************************************************
                          joiner.cpp  -  description
                             -------------------
    begin                : Mon May 8 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "joiner.h"

namespace limpid {

Joiner::Joiner(const DOMString& destPath) :
    destPath_(destPath), rootName_("ROOT"), domWriter_(0) {
 setTitleStrings("");
 titleString_.change("_", " ");
}

Joiner::Joiner(const DOMString& destPath, const DOMString& rootName, const DOMString& titleString) :  destPath_(destPath), rootName_(rootName) {
  if (rootName_.empty())
    rootName_ = "ROOT";

 setTitleStrings(titleString);
 titleString_.change("_", " ");
}

void Joiner::setTitleStrings(const DOMString& titleString) {
  if (titleString.empty())  {
    titleName_ = "TITLE";
    titleString_ = "UNTITLED";
    return;
  }

  size_t equalIndex = titleString.indexOf("=");

  if (equalIndex != titleString.npos) { // the titleString includes "="
    titleName_ = titleString.substr(0, equalIndex);
    titleString_ = titleString.substr(equalIndex + 1);

    if (titleName_.empty()) // nothing before "="
      titleName_ = "TITLE";
    if (titleString_.empty()) // nothing after "="
      titleString_ = "UNTITLED";
  }
  else {  // consider that the whole string is the actual title
    titleName_ = "TITLE";
    titleString_ = titleString;
  }
}

void Joiner::start() {
  if (destPath_.empty())
    writer_ = new Writer;
  else
    writer_ = new Writer(destPath_);

  domWriter_ = new DOMWriter(*writer_);

  *writer_ << "<" << rootName_ << ">" <<  wrt::endl;
  *writer_ << "<" << titleName_ << ">" << titleString_ << "</" << titleName_ << ">" << wrt::endl;
}

Document& Joiner::processDocument(Document& document) {
  domWriter_->write(document.getDocumentElement());
  return document;
}

void Joiner::finish() {
// fake an end node for the document element
  *writer_ << wrt::endl << "</" << rootName_ << ">" << wrt::endl;
}

}
