/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDLATEXDOCUMENTCONFIGURATION_H
#define LIMPIDLATEXDOCUMENTCONFIGURATION_H

#include "domstring.h"
#include "document.h"
#include "domutils.h"
#include "stringutils.h"
#include <map>
#include <set>

namespace limpid {

/**
	@author John Redmond <john_redmond@optusnet.com.au>
*/
class LatexDocumentConfiguration: public DOMUtils, public StringUtils {
public:
    LatexDocumentConfiguration() {}

    void configure(Document& configDocument);
    void setDocumentSpecs(Element& rootElement);

    // these public interfaces return constant references, while the internal ones return values
    DOMString getPreambleString(Document& dataDocument, const DOMString& titleString);
    DOMString getPackageString();
    DOMString getLengthsString();
    DOMString getDefinitionsString();
    DOMString getSectionsString();
    DOMString getDocumentStartString();
    DOMString getTOCString();
    DOMString getTailString();
    DOMString getDefaultFont();
    bool isVerbose();
    bool isXHTML();

private:
    typedef std::map<DOMString, DOMString> StringMap;
    typedef std::set<DOMString> StringSet;

    StringMap documentMap_, packageMap_, lengthMap_;
    StringSet packageSet_;

    void configureDocument(DOMString& tagString, Node& tagNode);
///    void configureGeometry(Node& tagNode);
    void configurePackage(Node& node);
    void configureLengths(Node& node);

///    bool isDocumentSpec(const DOMString& specString);

    DOMString getHeadersString(bool isBody);
    DOMString getFancyMappedString(bool isBody);

    void configureMapDefaults();
    void setMaps(Document& configDocument);
    void setSpecs(Node& refNode, StringMap& map);
};

}

#endif
