/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDLATEXMAPPER_H
#define LIMPIDLATEXMAPPER_H

#include "documentprocessor.h"
#include "domutils.h"
#include "getter.h"

namespace limpid {

/**
	@author John Redmond <john_redmond@optusnet.com.au>
*/
class LatexMapper : public DocumentProcessor, public DOMUtils {

public:
  LatexMapper(const DOMString& specPath);
  Document& processDocument(Document& sourceDocument);
  LatexMapper& copy() { return *(new LatexMapper(*this)); }

private:

  typedef std::map<DOMString, DOMString> StringMap;
  typedef std::vector<DOMString> StringList;

  Document configDocument_, resultDocument_;
  DOMString specPath_;
  DOMString documentString_, tocString_, sectionsString_, documentStartString_, documentBodyString_, tailString_;

  StringMap documentMap_, packageMap_, tocHeadersMap_, bodyHeadersMap_, styleMap_, preStringMap_, postStringMap_;
  StringList documentList_, packageList_;

  void setDefaultValues(Document& sourceDocument);
  void updateSpecs();
  void setDefaultDefinitions();
  void setTopStrings(Document& sourceDocument);

  void buildDocument(Document& sourceDocument);
  Element getDocumentElement();
  void addRootElement(Element& parentElement);
  void addTopElements(Element& parentElement);
  void addTopElement(const DOMString& nameString, const DOMString& contentString, Element& parentElement);
  void addDefinedTemplate(Node& node, Element& documentElement);
  void addNewlineElement(Element& parentElement);
  void addTextTemplate(Element& parentElement);
  void addTemplateElement(const DOMString& matchString, const DOMString& preString, const DOMString& postString, Element& parentElement, bool addTerminator);
  void addDefaultElement(Element& parentElement);
  void setDefinition(const DOMString& key, const DOMString& details, const DOMString& postString = "}");
  void addCallElement(const DOMString& matchString, Element& parentElement);
  void addTextElement(Element& parentElement);
  void addXSLText(Element& parentElement, const DOMString& value);

  DOMString getTitlePath(Node& rootNode);
  DOMString getPackageString(StringMap& sourceMap);
  DOMString getMappedString(StringMap& sourceMap, const DOMString& patternString);
  DOMString getFancyMappedString(StringMap& sourceMap);

  void setMap(Node& parentNode, StringMap& map);
  void setMap(Node& parentNode, StringMap& map, StringList& list);
  void mapItem(const DOMString& key, const DOMString& value, StringMap& map);
  void configure(Getter& getter);
};

}

#endif
