/***************************************************************************
                          lexxiaapp.h  -  description
                             -------------------
    begin                : Mon Apr 24 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LEXXIAAPP_H
#define LEXXIAAPP_H

#include <vector>
#include "domstring.h"
#include "getter.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class DocumentBox;
  class ProcessBox;

  class LexxiaApp {
  public:
    LexxiaApp();
    int exec(int argc, char *argv[]);

  private:
    typedef std::pair<DOMString, DOMString> StringPair;
    typedef std::vector<StringPair> PairList;
    typedef std::vector<DOMString> GetterPaths;

    DOMString mode_, inputType_;
    DOMString stepperString_;
    PairList getters_;
    GetterPaths getterPaths_;
    Pointer<Getter> documentSource_;
    int getterCount_, processorCount_;
    bool helpful_, verbose_;
    Writer& verboseWriter_;

    int execDocuments(DocumentBox& documentBox);
    int execStandardIn(DocumentBox& documentBox);

    bool configForDocBox(DocumentBox& documentBox, int argc, char *argv[]);
    bool setGetter(DocumentBox& documentBox, const DOMString& sourceString);
    bool setDefaultInGetter(DocumentBox& documentBox, const DOMString& sourceString);
    bool addPutter(DocumentBox& documentBox, const DOMString& argString, bool toConsole);
    bool addTexConverter(DocumentBox& documentBox, const DOMString& argString);
    bool addHTMLStyler(DocumentBox& documentBox, const DOMString& argString, bool expandStyle);
    bool addTransformer(DocumentBox& documentBox, const DOMString& argString);
    bool addMutator(DocumentBox& documentBox, const DOMString& argString);
    bool addXMLWrapper(DocumentBox& documentBox, const DOMString& argString, bool expandStyle);
    bool addNewNamer(DocumentBox& documentBox, const DOMString& argString, bool discardChildren);
    bool addJoiner(DocumentBox& documentBox, const DOMString& argString);
    bool addReporter(DocumentBox& documentBox, const DOMString& argString, bool simple);

    bool isDocumentProcess(int argc, char *argv[]);
    bool isVerbose(int argc, char *argv[]);
    bool configForStepper(ProcessBox& documentBox, int argc, char *argv[]);
    bool setParser(ProcessBox& processBox, const DOMString& sourceString);
    bool addStepper(ProcessBox& processBox, const DOMString& argString);

    DOMString getArgString(const char* arg);
  };
}

#endif
