/***************************************************************************
                          liststream.cpp  -  description
                             -------------------
    begin                : Tue Oct 7 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "liststream.h"
#include "writer.h"

namespace limpid {

ListStream::ListStream():
      index_(0), length_(0), direction_(true) {
}

ListStream::ListStream(NodeList& nodeList):
      nodeList_(&nodeList), index_(0), length_(nodeList.getLength()), direction_(true) {
}

ListStream::ListStream(Node& parent):
      nodeList_(&parent.getChildNodes()), index_(0),
      length_(parent.getChildNodes().getLength()), direction_(true) {
}

void ListStream::setForward(bool direction) {
  if (!length_ || direction == direction_) return; // nothing to do

  direction_ = direction;

  if (!direction_ )
    index_ = length_ - 1;  // index to last node in the list
  else
    index_ = 0; // index the first node in the list
}

Node& ListStream::nextNode() {
  if (!length_) return Node::failNode; // no nodes in the list

  if (direction_ && index_ < length_) // not yet hit the end
    return nodeList_->item(index_++);
  else if (!direction_ && index_ >= 0) // not yet hit the start
    return nodeList_->item(index_--);
  else return Node::failNode;
}

void ListStream::setRoot(const Node& root) {
  nodeList_ = &root.getChildNodes();
  length_ = nodeList_->getLength();

  if (direction_)
    index_ = 0;
  else
    index_ = length_ - 1;
}

}
