/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "matchpoint.h"
#include "element.h"

namespace limpid {

MatchPoint::MatchPoint(const DOMString& name, bool valid): type_(DEFAULT), name_(name), valid_(valid), mixed_(false), minHits_(1), maxHits_(1), hits_(0), activePoint_(0), parentPoint_(0), nextPoint_(0) {}

bool MatchPoint::isXSDElement(Node& node) {
  if (node.getNodeType() != Node::ELEMENT_NODE) return false;
  Element& element = static_cast<Element &>(node);
  return element.getNamespaceURI() == "http://www.w3c.org/2001/XMLSchema";
}

void MatchPoint::setFrequency(const Element& element) {
  minHits_ = getIntAttribute(element, "minOccurs");
  maxHits_ = getIntAttribute(element, "maxOccurs");
  unbounded_ = (maxHits_ == INT_MAX);
}

int MatchPoint::getIntAttribute(const Element& element, const char* attrName) {
  DOMString attrValue = element.getAttribute(attrName);
  if (!attrValue.size()) return 1;
  else if (attrValue == "unbounded")
    return INT_MAX;
  else return attrValue.toInt();
}

MatchPoint MatchPoint::failPoint("fail", false);
}
