/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDMATCHPOINT_H
#define LIMPIDMATCHPOINT_H

#include "node.h"
#include "pointer.h"
#include "writer.h"
#include <vector>
#include <list>

namespace limpid {

/**
	@author John Redmond <john_redmond@optusnet.com.au>
*/

enum MatchResult {FAILURE = 0, SUCCESS = 1, PASS = 2};
enum PointType {DEFAULT = 0, ELEMENT, SEQUENCE, CHOICE};

class MatchPoint {

public:
  static MatchPoint failPoint;

  virtual ~MatchPoint() {}

  virtual void traverse(const DOMString& = "") {}
  virtual bool test(Node& node) { return false; }

  void setNextPoint(MatchPoint& nextPoint) { nextPoint_ = &nextPoint; }
  bool hasNextPoint() { return nextPoint_; }
  MatchPoint& getNextPoint() {
    if (nextPoint_) return *nextPoint_;
    if (parentPoint_) return parentPoint_->getNextPoint(); return failPoint; }

  void setParentPoint(MatchPoint& parentPoint) { parentPoint_ = &parentPoint; }
  bool hasParentPoint() { return parentPoint_; }
  MatchPoint& getParentPoint() { return *parentPoint_; }
  virtual MatchPoint& getFirstPoint() { return *this; }

  PointType getType() { return type_; }
  DOMString getName() { return name_; }
  virtual MatchPoint& getActivePoint() { return *this; }
  virtual int getLength() { return 0; }

  virtual void addPoint(MatchPoint& point) {}
  virtual bool isAvailable() { return unbounded_ || hits_ < maxHits_; }
  virtual bool isRequired() { return hits_ < minHits_; }
  virtual MatchPoint& copy() { return *(new MatchPoint(*this)); }

  virtual void clearHits() { hits_ = 0;}
  void addHit() { ++hits_; }
  virtual int getSize() { return 1; }

  bool isValid() { return valid_; }
  bool isMixed() { return mixed_; }

protected:
  MatchPoint(const DOMString& name, bool valid = false);

  DOMString name_;
  PointType type_;
  bool valid_, unbounded_, mixed_;
  int minHits_, maxHits_, hits_;
  MatchPoint* activePoint_;
  Pointer<MatchPoint> parentPoint_, nextPoint_;

  void setFrequency(const Element& element);
  virtual void doTraversal() {}
  int getIntAttribute(const Element& element, const char* attrName);
  bool isXSDElement(Node& node);

};

typedef std::vector<MatchPoint *> MatchList;
typedef Pointer<MatchList> SearchPointer;
typedef std::vector<SearchPointer> SearchList;

}

#endif
