/***************************************************************************
                          namecontent.cpp  -  description
                             -------------------
    begin                : Mon Aug 11 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "namedcontent.h"

namespace limpid {

NamedContent::NamedContent(short type):
  NodeContent(type), nodeName_(0), prefix_(0) {
}

NamedContent::NamedContent(short type, const DOMString& name):
    NodeContent(type), nodeName_(0), prefix_(0) {
  setName(name);
}

NamedContent::~NamedContent() {
  delete [] prefix_;
  delete [] nodeName_;
}

void NamedContent::setName(const DOMString& s) {
  try {
    if (prefix_) delete [] prefix_;    // important for EndNode::update()
    if (nodeName_) delete [] nodeName_;

    prefix_ = nodeName_ = 0;

    size_t length = s.length();
    size_t prefixSize = s.find(':');
    if (prefixSize != DOMString::npos) {
      DOMString prefixString = const_cast<DOMString &>(s).substring((size_t)0, prefixSize);
      DOMString nameString = const_cast<DOMString &>(s).substring(prefixSize + 1, length - prefixSize - 1);
      prefix_ = prefixString.copyChars();
      nodeName_ = nameString.copyChars();
    }
    else {
      prefix_ = 0;
      nodeName_ = s.copyChars();
    }
  }
  catch (std::bad_alloc) {
    delete [] nodeName_;
    delete [] prefix_;
    throw;
  }

}

size_t NamedContent::getSize() {
  size_t size = sizeof(*this);

  if (prefix_) size += (DOMString::strLength(prefix_) + 1) * sizeof(Char);
  size += (DOMString::strLength(nodeName_) + 1) * sizeof(Char);

  return size;
}

}
