/***************************************************************************
                          namednodemap.h  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NAMEDNODEMAP_H
#define NAMEDNODEMAP_H

#include <map>
#include <vector>
#include <string>
#include <functional>
#include "node.h"
#include "attr.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class ElementContent;
  class Element;

  class NamedNodeMap {
    friend class Attr;
    friend class Element;
    friend class DocumentType;

  public:
  	NamedNodeMap();
  	NamedNodeMap(Element& parentNode);
    NamedNodeMap(const NamedNodeMap& refMap);
  	~NamedNodeMap();
    NamedNodeMap& operator=(const NamedNodeMap&);

    static NamedNodeMap defaultAttributes;

//DOM:
    Node& getNamedItem(const DOMString& name) const;
    Node& setNamedItem(Node& node);
    Node& removeNamedItem(const DOMString& name);
    Node& item(size_t index) const;
    size_t getLength() const;
    Node& getNamedItemNS(const DOMString& namespaceURI, const DOMString& localName) const;
    Node& setNamedItemNS(Node& node);
    Node& removeNamedItemNS(const DOMString& namespaceURI, const DOMString& localName);

  //non-DOM:
    bool isValid() { return this != &defaultAttributes; }
    bool operator!() { return !nodes_->size(); }
    size_t getSize();

  private:
    typedef std::vector<Node*> NodeVector;

    size_t *refCount_;
    ElementContent *elementContent_;
    NodeVector *nodes_;

    void setItem(Node& node);
    Node& removeSpecifiedItem(Node& node);
    bool getSetPermission(Node& parent, Node& child);
    bool getRemovePermission(Node& parent);
    Node& remove(const DOMString& namespaceURI);
    Node& remove(const DOMString& namespaceURI, const DOMString& localName);
    void updateLinks(Node& node);
    void deleteContent();
    void copyContent(const NamedNodeMap& refMap);
  };
}
#endif
