/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "newnamer.h"
#include "element.h"
#include "nodestream.h"
#include "liststream.h"
#include "attributestream.h"

namespace limpid {

NewNamer::NewNamer(const DOMString& oldPath, const DOMString& newName, bool discardChildren):
  matcher_(oldPath), newName_(newName), discardChildren_(discardChildren) {
}

Document& NewNamer::processDocument(Document& document) {
  NodeList hitList;
  NodeStream nodeStream(document.getDocumentElement(), false, false); // exclude root, no endnodes
  while (true) {
    Node& node = nodeStream.nextNode();
    if (!node.isValid()) break;
    if (matcher_.match(node))
      hitList.addNode(node);
  }

  ListStream hitStream(hitList);
  hitStream.setForward(false);  // move in reverse order
  while (true) {
    Node& node = hitStream.nextNode();
    if (!node.isValid()) break;

    Node& parentNode = node.getParentNode();
    if (!parentNode.isValid()) continue; // impossible?

    Element newElement(newName_);
    parentNode.insertBefore(newElement, node);
    if (!discardChildren_)
      transferContent(static_cast<Element &>(node), newElement);
    parentNode.removeChild(node);
  }

  return document;
}

void NewNamer::transferContent(Element& oldElement, Element& newElement) {
  while (true) {
    Node& childNode = oldElement.getFirstChild();
    if (!childNode.isValid()) break;	// no more children
    newElement.appendChild(childNode);
  }

  AttributeStream attributeStream(oldElement);
  while (true) {
    Node& node = attributeStream.nextNode();
    if (!node.isValid()) break;

    Attr& attributeClone = static_cast<Attr &>(node.cloneNode(true));
    newElement.setAttributeNode(attributeClone);
  }
}

}
