/***************************************************************************
                          Node.h  -  description
                             -------------------
    begin                : Wed Feb 19 2003
    copyright            : (C) 2003 by root
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This is the primitive, ancestral node of all functional DOM nodes.                                   *
 *                                                                         *
 ***************************************************************************
*/
#ifndef NODE_H
#define NODE_H

#include <vector>
#include <list>
#include "definitions.h"
#include "copied.h"
#include "domstring.h"
#include "systemmanager.h"
#include "nodecontent.h"
#include "domexception.h"
/*
#include "parentnode.h"
#include "element.h"
#include "document.h"
#include "nodelist.h"
#include "nodeset.h"
#include "namednodemap.h"
#include "builder.h"
#include "contentvalidator.h"
*/
namespace limpid {
  class Document;
  class Element;
  class Text;
  class Comment;
  class NameContent;
  
  class Node: public Copied  {
    friend class ParentNode;
    friend class Element;
    friend class Document;
    friend class NodeList;
    friend class NodeSet;
    friend class NamedNodeMap;
    friend class SystemManager;
    friend class Builder;
    friend class ContentValidator;
    friend class XMLParser;
    friend std::wostream& operator << (std::wostream& out, Node& node);

    public:
      enum NodeType {BASE_NODE = 0, ELEMENT_NODE = 1, ATTRIBUTE_NODE = 2, TEXT_NODE = 3,
            CDATA_SECTION_NODE = 4, ENTITY_REFERENCE_NODE = 5, ENTITY_NODE = 6,
            PROCESSING_INSTRUCTION_NODE = 7, COMMENT_NODE = 8, DOCUMENT_NODE = 9,
            DOCUMENT_TYPE_NODE = 10, DOCUMENT_FRAGMENT_NODE = 11,
            NOTATION_NODE = 12, XML_DECLARATION_NODE = 13, END_NODE = 14,
            SYSTEM_NODE = 15, FAIL_NODE = 16};

      virtual ~Node();

      virtual Node& operator=(const Node& refNode);
      bool operator==(const Node& refNode) const { return nodeContent_ == refNode.nodeContent_; }
      bool operator!=(const Node& refNode) const { return nodeContent_ != refNode.nodeContent_; }

  //system statics:
      static limpid::SystemManager& systemManager;
      static Node failNode, ignoreNode;
      static char nodeNames[][20];
      static bool verbose;

  //DOM:
      virtual DOMString getNodeName() const;
      virtual DOMString getRawValue() const { return ""; }
      virtual DOMString getNodeValue() const { return ""; }
      virtual void setNodeValue(const DOMString& ) { }
      int getNodeType() const;
      virtual Node& getParentNode() const;
      virtual NodeList& getChildNodes() const;
      virtual Node& getFirstChild() const { return failNode; }
      virtual Node& getLastChild() const { return failNode; }
      Node& getPreviousSibling() const;
      Node& getNextSibling() const;
      virtual NamedNodeMap& getAttributes() const;
      virtual Document& getOwnerDocument() const;

      //DOM defaults, to be overridden
      virtual Node& insertBefore(Node&, Node& refChild) { return refChild; }
      virtual Node& replaceChild(Node&, Node& oldChild) { return oldChild; }
      virtual Node& removeChild(Node& child) { return child; }
      virtual Node& appendChild(Node& childNode) { return childNode; }
      virtual bool hasChildNodes() const { return false; }
      virtual Node& cloneNode(bool deep = false) const;
      virtual void normalize() {}
      virtual bool isSupported(const DOMString&, const DOMString&) { return true; }
      virtual void setIsParent(bool) {}
      bool isParent() { return nodeContent_->isParent_; }
      virtual DOMString getNamespaceURI() const { return ""; }
      virtual DOMString getPrefix() const { return ""; }
      virtual void setPrefix(DOMString&) { }
      virtual DOMString getLocalName() const { return ""; }
      virtual bool hasAttributes() const { return false; }

  //non-DOM:
      virtual bool isValid() const { return nodeContent_ != failNode.nodeContent_; }
      virtual bool operator!() const { return nodeContent_ == failNode.nodeContent_; }
      virtual bool addChildNodes() { return false; }
      int getPosition() { return nodeContent_->position_; }
      void setPosition(int pos) { nodeContent_->position_ = pos; }

      Node& appendElement(Element child);
      Node& appendText(Text child);
      Node& appendComment(Comment child);
      virtual DOMString getDisplayString() const;
      std::list<DOMString> getPathList() const;
      DOMString getPath() const;
      virtual DOMString getValue() const { return ""; }

      size_t getRefCount() const { return nodeContent_->refCount_; }
      virtual size_t getSize();

      bool isReadOnly() const { return nodeContent_->readOnly_; }
      void setReadOnly(bool flag) { nodeContent_->readOnly_ = flag; }
      bool hasParent() { return nodeContent_->parentContent_; }
      virtual bool hasNonTextChildren() const { return false; }
      void reorder(size_t interval = 1);
      
    protected:
      Node();
      Node(NodeType nodeType);
      Node(const Node& refNode);
      NodeContent *nodeContent_;

      virtual void setNamespaceURI(const DOMString&) { }
      virtual void setDocumentContent(DocumentContent& documentContent) {
        nodeContent_->documentContent_ = &documentContent;
      }

      virtual void clearLinks();
      void detach();
      void copyContent(const Node& refNode);
      void replaceContent(const Node& refNode);
      virtual void connect() {}
      virtual Node& copy() { return *(new Node(*this)); }
  };
}
#endif
