/***************************************************************************
                          nodelist.h  -  description
                             -------------------
    begin                : Wed Mar 19 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NODELIST_H
#define NODELIST_H

#include <vector>
#include "node.h"
#include "pointer.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class ParentContent;
  class ParentNode;
  class Element;

  class NodeList {
    friend class Node;
    friend class ParentNode;
    friend class Element;
    friend class Sorter;

  public:
    static NodeList defaultChildNodes;

    NodeList();
  	NodeList(ParentNode& parentNode);
    NodeList(const NodeList& refList);
  	virtual ~NodeList();
    NodeList& operator=(const NodeList&);

  //DOM:
    Node& item(size_t index);
    size_t getLength() { return nodes_->size(); }

  //non-DOM:
    Node& addNode(Node& childNode);
    Node& removeNode(Node& node);
    Node& insertNode(Node& newChild, Node& refChild);
    Node& getFirstChild();
    Node& getLastChild();

    bool hasMember(Node& node) {
       return std::find(nodes_->begin(), nodes_->end(), &node) != nodes_->end();
    }
    bool isValid() { return this != &defaultChildNodes; }
    bool operator!() { return !nodes_->size(); }
    NodeList& clear();
    size_t getSize();

  private:
    typedef std::vector<Node*> NodeVector;

    NodeVector* nodes_;
    ParentContent *parentContent_;
    size_t *refCount_;

    NodeVector& getData() { return *nodes_; }

    void updateLinks(Node& node);
    bool getAppendPermission(const Node& parent, const Node& child);
    bool getRemovePermission(const Node& parent);
    void deleteContent();
    void copyContent(const NodeList& refList);
  };
}
#endif
