/***************************************************************************
                          nodeset.cpp  -  description
                             -------------------
    begin                : Mon Feb 16 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "nodeset.h"
#include "expr.h"
#include "node.h"

namespace limpid {

NodeSet::NodeSet(bool safe): safe_(safe), //nodes_(new NodeVector) /*,
    nodes_(new NodeVector), nodePointers_(safe ? new NodePointerVector : 0) {}

void NodeSet::addNode(Node& node) {
  if (hasMember(node)) return; // make sure that there is only one copy of node
  if (!safe_) {
    nodes_->push_back(&node);
    return;
  }

  Node* cpy = &node.copy();
  nodes_->push_back(cpy);
  nodePointers_->push_back(cpy);
}

NodeSet NodeSet::getNodeSet(Context&, bool) {
  return *this;
}

Expr NodeSet::eval(Context& context) {
  return getNodeSet(context);
}

Node& NodeSet::getItem(size_t index) const {
  if (index < nodes_->size())
    return *nodes_->at(index);
  else return Node::failNode;
}

int NodeSet::getPosition(size_t index) {
  if (index < nodes_->size())
    return nodes_->at(index)->getPosition();
  else return -1;
}

XPathString NodeSet::getLocalName() {
  if (nodes_->size()) return getItem(0).getLocalName();
  else return "";
}

XPathString NodeSet::getNamespaceUri() {
  if (nodes_->size()) return getItem(0).getNamespaceURI();
  else return "";
}

XPathString NodeSet::getName() {
  if (nodes_->size()) return getItem(0).getNodeName();
  else return "";
}

NodeSet NodeSet::failSet;

}

