/***************************************************************************
                          nodeset.h  -  description
                             -------------------
    begin                : Mon Feb 16 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NODESET_H
#define NODESET_H

#include <algorithm>
#include "number.h"  // this includes other xpath expressions
#include "pointer.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class Context;
  class Node;
  class Expr;

  class NodeSet: public XPathExpression {
  public:
    typedef std::vector<Node *> NodeVector;
    typedef Pointer<Node> NodePointer;
    typedef std::vector<NodePointer> NodePointerVector;

    NodeSet(bool safe = false);

    static NodeSet failSet;

    void addNode(Node& node);
    size_t getLength() const { return nodes_->size(); }
    Node& getItem(size_t index) const; // Node::failNode == out of range
    void clear() { nodes_->clear(); }
    Node& item(size_t index) const { return getItem(index); } // same as for NodeList
    int getPosition(size_t index); // -1 == out of range
    bool hasMember(Node& node) const {
       return std::find(nodes_->begin(), nodes_->end(), &node) != nodes_->end();
    }
    bool isNodeSet() { return true; }
    bool isValid() { return this != &failSet; }

    NodeSet getNodeSet(Context& context, bool exact = true);
    XPathString getLocalName();
    XPathString getNamespaceUri();
    XPathString getName();

    Boolean getBoolean(Context&) {
      return static_cast<int>(nodes_->size()); }
    XPathString getString(Context&) {
      if (nodes_->size()) return getItem(0).getValue(); else return ""; }
    Number getNumber(Context& context) {
      if (nodes_->size()) return getString(context).getNumber(context); else return 0; }
    Expr eval(Context& context);


    NodeVector& getData() { return *nodes_; }
    NodeSet& copy() { return *(new NodeSet(*this)); }

  private:
    bool safe_;
    Pointer<NodeVector> nodes_;
    Pointer<NodePointerVector> nodePointers_;
  };
}
#endif
