/***************************************************************************
                          nodestream.cpp  -  description
                             -------------------
    begin                : Sun Oct 5 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "nodestream.h"
#include "writer.h"

namespace limpid {

NodeStream::NodeStream(bool inclEnds, bool inclRoot):
    includeEnds(inclEnds), includeRoot(inclRoot) {
  reset();
}

NodeStream::NodeStream(const Node& rt, bool inclEnds, bool inclRoot):
    includeEnds(inclEnds), includeRoot(inclRoot) {
  reset();
  root = const_cast<Node *>(&rt);
  currentNode = root;
}

void NodeStream::setRoot(const Node& rt) {
  reset();
  root = const_cast<Node *>(&rt);
}

Node& NodeStream::nextNode() {
  if (!root) return Node::failNode;  // needs setRoot()

  if (finished) return Node::failNode;
  if (!refNode) {
    if (includeRoot) refNode = root;
    else {
      refNode = &root->getFirstChild();
      if (!refNode->isValid()) {
        finished = true;
        refNode = &Node::failNode;
      }
    }
    return *refNode;
  }
  else {
    Node& node = getNext();
    includeChildren = true;
    return node;
  }
}

Node& NodeStream::getNext() {
  if (includeChildren && !ascending) {
    currentNode = &refNode->getFirstChild();
    if (currentNode->isValid()) {
      refNode = currentNode;
      return *currentNode;
    }
  }

  if (*refNode != *root) {
    currentNode = &refNode->getNextSibling();
    if (currentNode->isValid()) {
      ascending = false;
      refNode = currentNode;
      return *currentNode;
    }
  }

  if (*refNode != *root)
    return getNextAbove();
  else {
    finished = true;
    endNode.update(*refNode);
    return endNode; // return an end node for the parent
  }
}

Node& NodeStream::getNextAbove() {
  ascending = true;
  while (true) {
    refNode = &refNode->getParentNode();
    if (!refNode->isValid() || refNode->getNodeType() == Node::DOCUMENT_NODE ||
        (!includeRoot && *refNode == *root)) {
      finished = true;
      return Node::failNode;
    }
    else if (*refNode == *root) {
      finished = true;
      if (includeEnds) {
        endNode.update(*refNode);
        return endNode;
      }
      else return Node::failNode;
    }
    if (includeEnds) {
      endNode.update(*refNode);
      return endNode; // return an end node for the parent
    }
    else {
      Node* sibling = &refNode->getNextSibling();
      if (sibling->isValid()) {
        ascending = false;  // stop ascending and move across
        refNode = sibling;
        return *refNode;
      }
    }
  }
}

void NodeStream::reset() {
  root = 0;
  currentNode = 0;
  refNode = 0;
  finished = false;
  ascending = false;
  includeChildren = true;
}

}
