/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "normalizer.h"
#include "nodestream.h"
#include "liststream.h"
#include "text.h"

namespace limpid {

Normalizer::Normalizer() {}

Document& Normalizer::processDocument(Document& document) {
  NodeList hitList;
  NodeStream nodeStream(document, false, false); // exclude root, no endnodes
  while (true) {
    Node& node = nodeStream.nextNode();
    if (!node.isValid()) break;
    if (node.getNodeType() != Node::TEXT_NODE)
      continue;
    if (node.getValue().contains("\n"))
      hitList.addNode(node);
  }

  ListStream hitStream(hitList);
  hitStream.setForward(false);  // move in reverse order
  while (true) {
    Node& node = hitStream.nextNode();
    if (!node.isValid()) break;

    Node& parentNode = node.getParentNode();
    if (!parentNode.isValid()) continue; // impossible?

    Text newText(node.getValue().normalizeWhitespace());
    parentNode.replaceChild(newText, node);
  }

  return document;
}

}
