/***************************************************************************
                          notation.cpp  -  description
                             -------------------
    begin                : Wed Sep 24 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "notation.h"
#include "notationcontent.h"
#include "writer.h"

namespace limpid {

Notation::Notation(const DOMString& name, const DOMString& pubId,
      const DOMString& sysId): Node(NOTATION_NODE) {
  try {
    nodeContent_ = new NotationContent(name, pubId, sysId);
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

DOMString Notation::getNodeName() const {
  const NotationContent *contentPtr = static_cast<const NotationContent *>(nodeContent_);
  if (contentPtr->nodeName_) return contentPtr->nodeName_;
  return "unnamed notation";  // impossible?
}

DOMString Notation::getPublicId() const {
  NotationContent *contentPtr = static_cast<NotationContent *>(nodeContent_);
  if (contentPtr->publicId_) return contentPtr->publicId_;
  else return "";
}

DOMString Notation::getSystemId() const {
  NotationContent *contentPtr = static_cast<NotationContent *>(nodeContent_);
  if (contentPtr->systemId_) return contentPtr->systemId_;
  else return "";
}

}
