/***************************************************************************
                          number.h  -  description
                             -------------------
    begin                : Mon Feb 16 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NUMBER_H
#define NUMBER_H

#include <cmath>
#include "xpathexpression.h"
#include "domstring.h"
#include "xpathstring.h"
#include "number.h"
#include "boolean.h"
#include "converter.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class Expr;

  class Number : public XPathExpression {
    public:
      Number();
      Number(double value);
      Number(const DOMString& valString);

      Number getFloor() { return floor(value_); }
      Number getCeiling() { return round(value_ + 0.5); }
      Number getRound() { return round(value_); }

      Boolean getBoolean(Context&) { return value_; }
      XPathString getString(Context& context);
      Number getNumber(Context&) { return *this; }
      Expr eval(Context& context);

      void addNumber(Number& number) { value_ += number.value_; }
      double getValue() { return value_; }
      int getInt() { return static_cast<int>(value_); }

      bool operator==(const Number& refNumber) { return value_ == refNumber.value_; }
      bool operator<(const Number& refNumber) { return value_ < refNumber.value_; }
      bool operator>(const Number& refNumber) { return value_ > refNumber.value_; }

      Number& copy() { return *(new Number(*this)); }

    private:
      double value_;
  };
}

#endif
