/***************************************************************************
                          numbercomparison.cpp  -  description
                             -------------------
    begin                : Mon May 17 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "numbercomparison.h"
#include "converter.h"
#include "writer.h"

namespace limpid {

NumberComparison::NumberComparison(Type type, bool logic):
      ParentComponent(), type_(type), logic_(logic) {
}

NumberComparison::NumberComparison(const DOMString& typeString):
      ParentComponent(), type_(EQ), logic_(true) {
  setMode(typeString);
}

Boolean NumberComparison::getBoolean(Context& context) {
  if (getLength() != 2) return false;  // should be impossible

  bool firstIsSet = components->at(0)->isNodeSet();
  bool secondIsSet = components->at(1)->isNodeSet();

  if (!firstIsSet && !secondIsSet)
    return comparePair(*components->at(0), *components->at(1), context);
  if (firstIsSet && !secondIsSet)
    return compareFirstNodeSet(*components->at(0), *components->at(1), context);
  else //if (!firstIsSet && secondIsSet)
    return compareSecondNodeSet(*components->at(0), *components->at(1), context);
}

Number NumberComparison::getNumber(Context& context) {
  return getBoolean(context).getNumber(context);
}

Expr NumberComparison::eval(Context& context) {
  return getBoolean(context);
}

bool NumberComparison::comparePair(XPathExpression& first, XPathExpression& second,
      Context& context) {
  return compareNumbers(first.getNumber(context).getValue(),
    second.getNumber(context).getValue());
}

bool NumberComparison::compareFirstNodeSet(XPathExpression& first,
      XPathExpression& second, Context& context) {
  NodeSet firstSet = first.getNodeSet(context);
  size_t length = firstSet.getLength();
  if (!length) return false;

  double secondValue = second.getNumber(context).getValue();

  for (size_t index = 0; index < length; ++index) {
    double firstValue = Converter::toDouble(firstSet.getItem(index).getValue());
    if (compareNumbers(firstValue, secondValue)) return true;
  }

  return false;
}

bool NumberComparison::compareSecondNodeSet(XPathExpression& first,
      XPathExpression& second, Context& context) {
  NodeSet secondSet = second.getNodeSet(context);
  size_t length = secondSet.getLength();
  if (!length) return false;

  double firstValue = first.getNumber(context).getValue();

  for (size_t index = 0; index < length; ++index) {
    double secondValue = Converter::toDouble(secondSet.getItem(index).getValue());
    if (compareNumbers(firstValue, secondValue)) return true;
  }

  return false;
}

inline bool NumberComparison::compareNumbers(double firstValue, double secondValue) {
  if (type_ == EQ) return (firstValue == secondValue) == logic_;
  if (type_ == LT) return (firstValue < secondValue) == logic_;
  return (firstValue > secondValue) == logic_;
}

void NumberComparison::setMode(const DOMString& typeString) {
  if (typeString == "!=") logic_ = false;
  else if (typeString == "false") logic_ = false;
  else if (typeString == ">") type_ = GT;
  else if (typeString == "<") type_ = LT;
  else if (typeString == ">=") {
    type_ = LT;
    logic_ = false;
  }
  else if (typeString == "<=") {
    type_ = GT;
    logic_ = false;
  }
}

}
