/***************************************************************************
                          numberformatter.h  -  description
                             -------------------
    begin                : Sun Jan 23 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NUMBERFORMATTER_H
#define NUMBERFORMATTER_H

#include <locale>
#include <vector>
#include "domstring.h"
#include "pointer.h"

/**
  *@author John Redmond

  NumberFormatter is specifically a helper class for NumberTemplate, which is
  used in XSLT. It formats only UNSIGNED numbers and is NOT a general-purpose
  number formatter. Instead, look at DecimalFormat.
*/

namespace limpid {
  class NumberFormatter {
  public:
  	NumberFormatter();
    NumberFormatter(const DOMString& formatString, const DOMString& langString,
      const DOMString& letterString, const DOMString& separatorString,
      const DOMString& sizeString);

    DOMString format(size_t number, size_t fieldNumber);

  private:
  DOMString separatorString_, letterString_, sizeString_;
    struct FieldSpecification  {
      DOMString preString, postString;
      size_t numberBase, width;
      bool isRoman, isUpperCase;
    };

    std::vector<FieldSpecification> fieldSpecifications_;
    Pointer<std::locale> outputLocale_;

    // string generation
    DOMString getNumberString(size_t number, const FieldSpecification& fieldSpecification);
    DOMString fillString(DOMString& numberString, size_t index);

    // setup utilities
    void parseFormat(const DOMString& formatString);
    void setLocale(const DOMString& langString);
    void setFieldSpecification(DOMString& numberString, DOMString& postString);
  };
}

#endif
