/***************************************************************************
                          numbergenerator.cpp  -  description
                             -------------------
    begin                : Sat May 29 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "numbergenerator.h"
#include "writer.h"
#include "converter.h"

namespace limpid {

NumberGenerator::NumberGenerator(ContextType t): ParentComponent(), type_(t) {
}

NumberGenerator::NumberGenerator(const DOMString& typeString): ParentComponent(), type_(COUNT) {
  setType(typeString);
}

Number NumberGenerator::getNumber(Context& context) {
  if (type_ == COUNT || type_ == LAST)
    return context.getCount();

  else if (type_ == POSITION)
    return context.getPosition();
  else if (type_ == STRING_LENGTH) return getStringLength(context);
  else if (type_ == NUMBER) return getChildNumber(context);
  else if (type_ == SUM) return getSum(context);
  else return getApproximate(context);
}

Number NumberGenerator::getStringLength(Context& context) {
  int length = getLength();
  if (!length) return 0;
  else return (int)components->at(0)->getString(context).size();
}

Number NumberGenerator::getChildNumber(Context& context) {
  int length = getLength();
  if (!length) return 0;
  else return components->at(0)->getNumber(context);
}

Number NumberGenerator::getApproximate(Context& context) {
  int length = getLength();
  if (!length) return 0;

  Number number = components->at(0)->getNumber(context);
  if (type_ == FLOOR) return number.getFloor();
  else if (type_ == CEILING) return number.getCeiling();
  else return number.getRound();
}

Number NumberGenerator::getSum(Context& context) {
  int length = getLength();
  if (!length) return 0;

  NodeSet nodeSet = components->at(0)->getNodeSet(context, true);
  length = nodeSet.getLength();

  Number sum;
  for (int index = 0; index < length; ++index) {
    Number number = Converter::toDouble(nodeSet.getItem(index).getValue());
    sum.addNumber(number);
  }

  return sum;
}

void NumberGenerator::setType(DOMString typeString) {
  if (typeString == "count")
    type_ = COUNT;
  else if (typeString == "last")
    type_ = LAST;
  else if (typeString == "position")
    type_ = POSITION;
  else if (typeString == "string-length")
    type_ = STRING_LENGTH;
  else if (typeString == "floor")
    type_ = FLOOR;
  else if (typeString == "ceiling")
    type_ = CEILING;
  else if (typeString == "round")
    type_ = ROUND;
  else if (typeString == "sum")
    type_ = SUM;
  else if (typeString == "number")
    type_ = NUMBER;
}

}
