/***************************************************************************
                          numbertemplate.h  -  description
                             -------------------
    begin                : Thu Oct 21 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NUMBERTEMPLATE_H
#define NUMBERTEMPLATE_H

/**
  @author John Redmond
*/

#include <deque>
#include "nodesource.h"
#include "template.h"
#include "number.h"
#include "test.h"
#include "numberformatter.h"

namespace limpid {
  class Test;
  class NodeSource;
  class CharacterWriter;

  class NumberTemplate : public Template  {
  public:
  	NumberTemplate(const DOMString& countString, const DOMString& valueString,
       const DOMString& levelString, const DOMString& fromString);

   bool process(XSLContext& context);
   bool processNode(XSLContext& parentContext, Node&);
   void setNumberFormatter(NumberFormatter& numberFormatter) {
     numberFormatter_ = numberFormatter;}
   NumberTemplate& copy() { return *(new NumberTemplate(*this)); }

  private:
    enum Level { SINGLE, MULTIPLE, ANY };
    enum Case { UPPER_CASE, LOWER_CASE };

    Expr valueExpr_, fromExpr_;
    Pointer<Test> test_;
    Pointer<NodeSource> stream_, ancestorStream_;
    Level level_;
    bool defined_;
    DOMString separator_;
    NumberFormatter numberFormatter_;

    typedef std::deque<int> NumberStack;
    Pointer<NumberStack> numberStack_;

    void processStream(XSLContext& parentContext);
    void addCounts(XSLContext& parentContext);
    int getCount(Node& rootNode, Pointer<NodeSource> stream);
    void output(CharacterWriter& writer);
    void configure(const DOMString& countString, const DOMString& valueString,
        const DOMString& levelString, const DOMString& fromString);
  };
}


#endif
