/***************************************************************************
                          outputmanager.cpp  -  description
                             -------------------
    begin                : Fri Apr 1 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "outputmanager.h"
#include "domstring.h"
#include "configurationset.h"
#include "xmldeclaration.h"
#include "htmldeclaration.h"
#include "domwriter.h"
#include "xslresources.h"
#include "xslcontext.h"

namespace limpid {

OutputManager::OutputManager() {
}

void OutputManager::setWriter(XSLContext& context,
    const DOMString& pathString, ConfigurationSet& configurationSet) {
  Writer writer(pathString);
  setupWriters(writer, context, configurationSet);
}

void OutputManager::setWriter(XSLContext& context, ConfigurationSet& configurationSet) {
  Writer writer;
  setupWriters(writer, context, configurationSet);
}

void OutputManager::setWriter(XSLContext& context, CharacterWriter& writer,
    ConfigurationSet& configurationSet) {
  setupWriters(writer, context, configurationSet);
}

void OutputManager::initializeDocument(Document& document, ConfigurationSet& configurationSet) {
  DOMString method = configurationSet.method;
  if (method == "html" || method == "xhtml") {
    DocumentType docType = getDoctype("xml", configurationSet);
    if (!docType.getPublicId().empty() || !docType.getSystemId().empty())
      document.insertBefore(docType, document.getFirstChild());

    if (!configurationSet.omit_xml_declaration) {
      XMLDeclaration decl(configurationSet.version, configurationSet.encoding,
        (configurationSet.standalone ? "yes" : "no"));
        document.insertBefore(decl, document.getFirstChild());
    }

  }
  else if (method == "xml") {
    DocumentType docType = getDoctype(method, configurationSet);
    if (!docType.getPublicId().empty() || !docType.getSystemId().empty())
      document.insertBefore(docType, document.getFirstChild());

    if (!configurationSet.omit_xml_declaration) {
      XMLDeclaration decl(configurationSet.version, configurationSet.encoding,
        (configurationSet.standalone ? "yes" : "no"));
        document.insertBefore(decl, document.getFirstChild());
    }
  }
  // else treat output as straight text with no declaration
}

void OutputManager::setupWriters(CharacterWriter& writer, XSLContext& context,
    ConfigurationSet& configurationSet) {
  context.setWriter(writer);
  context.setDOMWriter(writer, configurationSet.method == "html");

  if (!configurationSet.indent) {
    context.getFormat().setFormat("\n", "", "");
    context.getDOMWriter().setFormat("\n", "", "");
  }

  writer.setEncoding(configurationSet.encoding);

  setDeclaration(writer, configurationSet);
}  

void OutputManager::setDeclaration(CharacterWriter& writer,
    ConfigurationSet& configurationSet) {
  DOMString method = configurationSet.method;
  DOMString startString;

  if (method == "html" || method == "xhtml") {
    if (!configurationSet.omit_xml_declaration) {
      XMLDeclaration decl(configurationSet.version, configurationSet.encoding,
        (configurationSet.standalone ? "yes" : "no"));

      startString += decl.getDisplayString();
      startString += "\n";
    }
 
    DocumentType docType = getDoctype("html", configurationSet);
    if (!docType.getPublicId().empty() || !docType.getSystemId().empty()){
      startString += docType.getDisplayString();
      startString += "\n";
    }
  }

  else if (method == "xml") {
    if (!configurationSet.omit_xml_declaration) {
      XMLDeclaration decl(configurationSet.version, configurationSet.encoding,
        (configurationSet.standalone ? "yes" : "no"));

      startString += decl.getDisplayString();
      startString += "\n";
    }
    
    DocumentType docType = getDoctype(method, configurationSet);
    if (!docType.getPublicId().empty() || !docType.getSystemId().empty()){
      startString += docType.getDisplayString();
      startString += "\n";
    }
  }

  writer << startString;

  // else treat output as straight text with no declaration or doctype
}

DocumentType OutputManager::getDoctype(const DOMString& type,
    const ConfigurationSet& configurationSet) {
  DOMString publicID = configurationSet.doctype_public;
  DOMString systemID = configurationSet.doctype_system;

  return DocumentType(type, publicID, systemID);
}

void OutputManager::setCDATASectionNames(XSLResources& resources,
    ConfigurationSet& configurationSet) {
  if (configurationSet.cdata_section_elements.size())
    resources.setCDATASectionNames(configurationSet.cdata_section_elements);
}

}
