/***************************************************************************
                          parentnode.cpp  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "parentnode.h"
#include "text.h"
#include "node.h"
#include "nodelist.h"
#include "parentcontent.h"
#include "domstring.h"
#include "systemmanager.h"
#include "document.h"
#include "writer.h"

namespace limpid {

ParentNode::ParentNode(): Node(BASE_NODE) {
}
/*
ParentNode::~ParentNode() {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (this == contentPtr->ownerNode_)

    contentPtr->ownerNode_ = 0;
}
*/
DOMString ParentNode::getNodeName() const {
  DOMString name;
  const NamedContent *contentPtr = static_cast<const ParentContent *>(nodeContent_);
  if (contentPtr->prefix_) {
    name += contentPtr->prefix_;
    name += ":";
  }
  if (contentPtr->nodeName_) name += contentPtr->nodeName_;
  return name;
}

DOMString ParentNode::getLocalName() const {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (contentPtr->prefix_) return contentPtr->nodeName_;
  else return DOMString::failString;
}

DOMString ParentNode::getPrefix() const {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (contentPtr->prefix_) return contentPtr->prefix_;
  else return DOMString::failString;
}

void ParentNode::setPrefix(const DOMString& prefix) {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  delete [] contentPtr->prefix_;
  contentPtr->prefix_ = prefix.copyChars();
}

NodeList& ParentNode::getChildNodes() const {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (contentPtr->childNodes_) return *contentPtr->childNodes_;
  else return NodeList::defaultChildNodes;
}

Node& ParentNode::getFirstChild() const {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (contentPtr->childNodes_)
    return contentPtr->childNodes_->getFirstChild();
  else return failNode;
}

Node& ParentNode::getLastChild() const {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (contentPtr->childNodes_)
    return contentPtr->childNodes_->getLastChild();
  else return failNode;
}

bool ParentNode::addChildNodes() {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (!contentPtr->ownerNode_) contentPtr->ownerNode_ = this;

  if (contentPtr->childNodes_) {
    delete contentPtr->childNodes_;
    contentPtr->childNodes_ = 0;
  }

  try {
     contentPtr->childNodes_ = new NodeList(*contentPtr->ownerNode_);
   }
   catch (std::bad_alloc) {
     delete contentPtr->childNodes_;
     return false;
   }

  return true;
}

bool ParentNode::hasChildNodes() const {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  return contentPtr->childNodes_;
}

Node& ParentNode::appendChild(Node& childNode) {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);

  if (!contentPtr->childNodes_ && !addChildNodes()) return Node::failNode;

  if (childNode.getNodeType() == DOCUMENT_FRAGMENT_NODE) {
    NodeList& children = childNode.getChildNodes();
    size_t length = children.getLength();
    for (size_t i = 0; i < length; i++) {
      Node& node = children.item(i);
      Node& resultNode = appendSingleNode(node);
      if (!resultNode.isValid()) return resultNode;
    }
    return childNode;
  }
  else return appendSingleNode(childNode);
}

Node& ParentNode::removeChild(Node& childNode) {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (!contentPtr->childNodes_) return childNode;  // parent has no children!

  if (childNode.nodeContent_->parentContent_ != nodeContent_)
    return childNode; // not a parent of this node, or the node has no parent

  Node& resultNode = contentPtr->childNodes_->removeNode(childNode);

  /* This is a potentially lethal economy: if a node is accidentally added
    twice to the same parent node, and there are no other children of the
    parent node, the node list is deleted mid-stream, and a crash results
    when an attempt is made to add to it.
  */
/*
  if (!contentPtr->childNodes_->getLength()) { // remove an empty list
    delete contentPtr->childNodes_;
    contentPtr->childNodes = 0;
  }
*/
  return resultNode;
}

Node& ParentNode::replaceChild(Node& newChild, Node& oldChild) {
  if (!oldChild.isValid()) return appendChild(newChild);

  insertBefore(newChild, oldChild);
  return removeChild(oldChild);
}

Node& ParentNode::insertBefore(Node& newChild, Node& refChild) {
  if (newChild == refChild) return newChild;  // silly possibility
  if (!refChild.isValid()) return appendChild(newChild);

  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (!contentPtr->childNodes_) return refChild; // refChild not a child

  if (newChild.getNodeType() == DOCUMENT_FRAGMENT_NODE) {
    NodeList& children = newChild.getChildNodes();
    size_t length = children.getLength();
    for (size_t i = 0; i < length; i++) {
      Node& node = children.item(i);
      Node& resultNode = insertSingleNode(node, refChild);
      if (!resultNode.isValid()) return resultNode;
    }
  }
  return insertSingleNode(newChild, refChild);
}

void ParentNode::normalize() {
  if (!systemManager.checkModificationPermission(*this)) return;

  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (contentPtr && contentPtr->childNodes_) {
    NodeList childNodes = *contentPtr->childNodes_;
    size_t length = childNodes.getLength();
    if (length >= 2) {
      for (size_t i = length - 1; i > 0; i--) {
        Node& thisNode = childNodes.item(i);
        int thisType = thisNode.getNodeType();
        if (thisType != TEXT_NODE) thisNode.normalize();
        else {
          Node& prevNode = childNodes.item(i - 1);
          int prevType = prevNode.getNodeType();
          if (prevType != TEXT_NODE) continue;

          Text& text = static_cast<Text &>(prevNode);
          DOMString value = thisNode.getNodeValue();
          text.appendData(value);
          removeChild(thisNode);
        }
      }
    }
  }
}

void ParentNode::setDocumentContent(DocumentContent& documentContent) {
  nodeContent_->documentContent_ = &documentContent;
  if (hasChildNodes()) {
    NodeList &childNodes = getChildNodes();
    size_t length = childNodes.getLength();
    for (size_t i = 0; i < length; i++)
      childNodes.item(i).setDocumentContent(documentContent);
  }
}

bool ParentNode::hasNonTextChildren() const {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  short nodeType = contentPtr->nodeType_;

  if (contentPtr->childNodes_) {
    if (nodeType != ELEMENT_NODE &&
        nodeType != DOCUMENT_NODE) return false;

    int childCount = contentPtr->childNodes_->getLength();
    for (int i = 0; i < childCount; i++) {
      Node& node = contentPtr->childNodes_->item(i);
      if (node.getNodeType() != TEXT_NODE) return true;
    }
  }
  return false;
}

inline Node& ParentNode::appendSingleNode(Node& childNode) {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (!contentPtr->ownerNode_)
    contentPtr->ownerNode_ = this;
  return contentPtr->childNodes_->addNode(childNode);
}

inline Node& ParentNode::insertSingleNode(Node& newChild, Node& refChild) {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (!contentPtr->ownerNode_)
    contentPtr->ownerNode_ = this;
  return contentPtr->childNodes_->insertNode(newChild, refChild);
}

void ParentNode::connect() {
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  contentPtr->ownerNode_ = this;
}

void ParentNode::clearLinks() {
  Node::clearLinks();
  ParentContent *contentPtr = static_cast<ParentContent *>(nodeContent_);
  if (this == contentPtr->ownerNode_)  // we have removed an owner node
    contentPtr->ownerNode_ = 0;
}

}
