/***************************************************************************
                          pointer.h  -  description
                             -------------------
    begin                : Tue Nov 23 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef POINTER_H
#define POINTER_H

/**
  *@author John Redmond
  */

// note that PHolder is a simple receptacle for data;
// it has no responsibility for managing it.
namespace limpid {
  template<class T>
  struct PHolder {
    public:
      PHolder<T>(): pointer_(0), refCount_(1) {}
      PHolder<T>(T* pointer): pointer_(pointer), refCount_(1) {}

      T *pointer_;
      unsigned int refCount_;
  };

  template<class T>
  class Pointer {
  public:
    Pointer(): holder_(new PHolder<T>) { }
    Pointer(T *pointer): holder_(new PHolder<T>(pointer)) {}
    Pointer(const Pointer<T>& refPointer) { copyContent(refPointer); }
    ~Pointer() { deleteContent(); }
    Pointer<T>& operator=(const Pointer<T>& refPointer) {
      if (holder_ != refPointer.holder_) {
        deleteContent();
        copyContent(refPointer);
      }
      return *this;
    }

    T* operator->() const { return holder_->pointer_; }
    T& operator*() const { return *(holder_->pointer_); }
    T* getPointer() const { return holder_->pointer_; }
    operator bool() const { return holder_->pointer_; }

  private:
    PHolder<T> *holder_;

    void copyContent(const Pointer<T>& refPointer) {
      holder_ = refPointer.holder_;
      ++holder_->refCount_;
    }

    void deleteContent() {
      if (!--holder_->refCount_) {
        delete holder_->pointer_;
        delete holder_;
      }
    }
  };
}
/*
// this is the inefficient prototype version:

namespace limpid {
  template<class T>
  class Pointer {
  public:
  	Pointer<T>(): refCount_(new unsigned int(1)), classPointer_(0) { }
    Pointer<T>(T *pointer):
        refCount_(new unsigned int(1)), classPointer_(pointer) {}

    Pointer<T>(const Pointer<T>& refPointer)  { copyContent(refPointer); }
  	~Pointer<T>() { deleteContent(); }
    Pointer<T>& operator=(const Pointer<T>& refPointer) {
      if (this != &refPointer) {
        deleteContent();
        copyContent(refPointer);
      }
      return *this;
    }

    T* operator->() const { return classPointer_; }
    T& operator*() const { return *classPointer_; }
    T* getPointer() { return classPointer_; }
    operator bool() { return classPointer_; }

  private:
    unsigned int *refCount_;
    T *classPointer_;

    void copyContent(const Pointer<T>& refPointer) {
      refCount_ = refPointer.refCount_;
      classPointer_ = refPointer.classPointer_;
      ++*refCount_;
    }

    void deleteContent() {
      if (!--*refCount_) {
        delete refCount_;
        delete classPointer_;
      }
    }
  };
}
*/

#endif
