/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "pointtracer.h"
#include "liststream.h"
#include "writer.h"

namespace limpid {

PointTracer::PointTracer() {}

void PointTracer::traverse(MatchPoint& matchPoint) {
  matchPoint.traverse();
}
/*
int traverse(MatchPoint& matchPoint, NodeList& nodeList, int index) {
  return matchPoint.visit(nodeList, 0);
}
*/
bool PointTracer::trace(Node& element, MatchPoint& matchPoint, Writer& writer) {
  matchPoint.clearHits();
  NodeList& childList = element.getChildNodes(); 
  int index = 0;
  MatchList startList, resultList;

  if (matchPoint.getType() == SEQUENCE && matchPoint.getSize())
    addPoint(startList, matchPoint.getFirstPoint());
  else
    addPoint(startList, matchPoint);

  for (int index = 0; index < childList.getLength(); ++index) {
    if (childList.item(index).getNodeType() == Node::TEXT_NODE) {
      if (!matchPoint.isMixed()) // complain and continue
        writer << element << " SHOULD NOT HAVE TEXT CHILDREN" << wrt::endl;
      continue;
    }

    Node& activeNode = childList.item(index);
    resultList = searchNext(activeNode, startList);

    if (resultList.empty()) {
      writer << "L:" << activeNode.getPosition() << " " << activeNode << " UNEXPECTED UNDER " << activeNode.getParentNode() << wrt::endl;
      return false;
    }

    for (int index = 0; index < resultList.size(); ++index) {
      resultList[index]->addHit();
    }  

    startList = resultList;
  }

  return true;
}

void PointTracer::addPoint(MatchList& matchList, MatchPoint& matchPoint) {
  matchList.push_back(&matchPoint);
}

MatchList PointTracer::searchNext(Node& nextNode, MatchList& stateList) {
  MatchList resultList;

  for (int index = 0; index < stateList.size(); ++index) {
    getOptions(nextNode, *stateList[index], resultList);
  }

  return resultList;
}

void PointTracer::getOptions(Node& node, MatchPoint& startPoint, MatchList& resultList) {
  // test the start point first
  if (startPoint.isAvailable() && startPoint.test(node)) addPoint(resultList, startPoint);

  if (startPoint.isRequired()) {
    return;  // cannot proceed
  }

  // now look further ahead
  if (startPoint.hasNextPoint()) {
    MatchPoint& nextPoint = startPoint.getNextPoint();
    getOptions(node, nextPoint, resultList);
    return;
  }

/*// this code remains problematic
  if (startPoint.hasParentPoint()) { // the parentPoint is a sequence or choice
    MatchPoint& parentPoint = startPoint.getParentPoint();

    MatchPoint& nextPoint = parentPoint.getNextPoint();
    if (nextPoint.isValid() && nextPoint.isAvailable() && nextPoint.test(node))
      addPoint(resultList, nextPoint);
  }
*/
}

}
