/***************************************************************************
                          processbox.cpp  -  description
                             -------------------
    begin                : Tue May 6 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include "element.h"
#include "endnode.h"
#include "processbox.h"
#include "xmlparser.h"
#include "reader.h"

namespace limpid {

ProcessBox::ProcessBox(): parser_(0), processors_(new ProcessorList) {}

ProcessBox::ProcessBox(CharacterReader& reader):
    parser_(new XMLParser(reader)), processors_(new ProcessorList) {}

ProcessBox::ProcessBox(const URL& url):
    parser_(0), processors_(new ProcessorList) {
  Reader reader(url);
  parser_ = new XMLParser(reader);
}

ProcessBox::ProcessBox(Parser& parser):
  parser_(&parser.copy()), processors_(new ProcessorList) {}

void ProcessBox::setSource(Parser& parser) {
  parser_ = &(parser.copy());
}

void ProcessBox::process() {
  if (!parser_) return;

  start();

  Node *nextNode = &Node::ignoreNode;
  while (true) {
    nextNode = &(parser_->nextNode());
    if (!nextNode->isValid()) break;

    process(*nextNode);
  }

  finish();
}

Node& ProcessBox::process(Node& node) {
//  Node& currentNode = node;
//if (true) return node;
  for (ProcessorList::iterator p = processors_->begin(); p != processors_->end(); p++) {
    if (!node.isValid()) return node;  // return fail if result is fail

    Processor& processor = **p;
    processor.process(node);
  }

  return node;
}

void ProcessBox::start() {
  // initialise all processors
  for (ProcessorList::iterator p = processors_->begin(); p != processors_->end(); p++)
    (*p)->start();
}

void ProcessBox::finish() {
  // finalise all processors
  for (ProcessorList::iterator p = processors_->begin(); p != processors_->end(); p++)
    (*p)->finish();
}

}
