/***************************************************************************
                          processinginstruction.cpp  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "processinginstruction.h"
#include "processinginstructioncontent.h"
#include "document.h"

namespace limpid {

ProcessingInstruction::ProcessingInstruction(const DOMString& target, const DOMString& data):
  Node(BASE_NODE) {
  try {
    nodeContent_ = new ProcessingInstructionContent(target, data);
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

ProcessingInstruction::ProcessingInstruction(const ProcessingInstruction& refNode):
  Node(refNode) {
}

ProcessingInstruction& ProcessingInstruction::operator=(const ProcessingInstruction& refNode) {
  replaceContent(refNode);
  return *this;
}

DOMString ProcessingInstruction::getTarget() const {
  ProcessingInstructionContent *contentPtr =
    static_cast<ProcessingInstructionContent *>(nodeContent_);
  return contentPtr->target_;
}

DOMString ProcessingInstruction::getData() const {
  ProcessingInstructionContent *contentPtr =
    static_cast<ProcessingInstructionContent *>(nodeContent_);
  return contentPtr->data_;
}

void ProcessingInstruction::setData(const DOMString& data) {
  ProcessingInstructionContent *contentPtr =
    static_cast<ProcessingInstructionContent *>(nodeContent_);
  delete [] contentPtr->data_;
  contentPtr->data_ = data.copyChars();
}

DOMString ProcessingInstruction::getNodeName() const {
  ProcessingInstructionContent *contentPtr =
    static_cast<ProcessingInstructionContent *>(nodeContent_);
  return contentPtr->target_;
}

DOMString ProcessingInstruction::getDisplayString() const {
  ProcessingInstructionContent *contentPtr =
    static_cast<ProcessingInstructionContent *>(nodeContent_);
  DOMString s("<?");
  s += contentPtr->target_;
  s += " ";
  s += contentPtr->data_;
  s += "?>";
  return s;
}

Node& ProcessingInstruction::cloneNode(bool) const {
  DOMString nodeName = getNodeName();
  DOMString data = getData();
  ProcessingInstruction *copyNode =
    new ProcessingInstruction(nodeName, data);

  copyNode->nodeContent_->documentContent_ = nodeContent_->documentContent_;
  getOwnerDocument().cacheNode(*copyNode);

  return *copyNode;
}

}
