/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDPROTOTYPER_H
#define LIMPIDPROTOTYPER_H

#include "documentprocessor.h"
#include "domstring.h"
#include "domutils.h"
#include <vector>
#include <set>
#include <map>


namespace limpid {

/**
 Prototyper provides a default (empty) stylesheet for a source XML document. In so doing, it identifies all
the unique element paths in the document.

 @author root <john_redmond@optusnet.com.au>
*/
class Prototyper : public DocumentProcessor, public DOMUtils {
public:
    Prototyper() {}
    Document& processDocument(Document&);
    DocumentProcessor& copy() { return *(new Prototyper(*this)); }

private:
  typedef std::vector<DOMString> StringList;
  typedef std::set<DOMString> StringSet;
  typedef std::map<DOMString, bool> LeafMap;
  StringList pathList_;
  StringSet pathSet_;
  LeafMap leafMap_;

  Document resultDocument_;

  void setPaths(Document& sourceDocument);
  void addDocumentElement(Document& resultDocument);
  void addProlog(Document& resultDocument);
  void addMatchElements(Element& documentElement);
  void addRootElement(Element& documentElement);
  void addMatchElement(Element& documentElement, const DOMString& pathString);
  void addTextElement(Element& documentElement);

};

}

#endif
