/***************************************************************************
                          putter.cpp  -  description
                             -------------------
    begin                : Tue Apr 11 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "putter.h"
#include "element.h"
#include "liststream.h"
#include "nodestream.h"
#include "traverser.h"
#include "writer.h"
#include "xmlwriter.h"
#include "csvwriter.h"
#include "sxfwriter.h"
#include "dotwriter.h"
#include "csswriter.h"

namespace limpid {

Putter::Putter(Dumper& dumper):
  toConsole_(false), typeString_("XML"), dumper_(&dumper.copy()) {}

Putter::Putter(const DOMString& destString, bool toConsole): toConsole_(toConsole), typeString_("XML"), dumper_(0) {
  setSpecs(destString);
}

Document& Putter::processDocument(Document& inputDocument) {
  dumper_->setProcessingHTML(inputDocument.getDocumentElement().getNodeName().toUpperCase() == "BODY");
  Traverser traverser(*dumper_);
  traverser.traverse(inputDocument);
  dumper_->flush();

  return inputDocument;
}

void Putter::setSpecs(const DOMString& destString) {
  destPath_ = destString;
  destPath_.trim();
  destSpec_ = destPath_;

  // try to determine a file type
  if (!destPath_.empty()) {
    size_t dotPos = destPath_.find('.');
    if (dotPos != destPath_.npos) {
      typeString_ = destPath_.substr(dotPos + 1);
      typeString_.toUpperCase();
    }
    else {
      typeString_ = destString;
      typeString_.toUpperCase();
    }
  }

  if (toConsole_)
    setConsoleDumper();
}

void Putter::setDumper(bool update) {
  if (toConsole_ && update) return;

  if (typeString_ == "XML" || typeString_ == "XSL" || typeString_ == "XSD")
    dumper_ = new XMLWriter(destPath_);
  else if (typeString_ == "HTML")
    dumper_ = new XMLWriter(destPath_, true); 
  else if (typeString_ == "CSV")
    dumper_ = new CSVWriter(destPath_);
  else if (typeString_ == "SXF")
    dumper_ = new SXFWriter(destPath_);
  else if (typeString_ == "TXT")
    dumper_ = new DotWriter(destPath_);
  else if (typeString_ == "CSS")
    dumper_ = new CSSWriter(destPath_);
  else
    dumper_ = new XMLWriter(destPath_); // xml output is the default
}

void Putter::setConsoleDumper() {
  if (typeString_ == "XML" || typeString_ == "XSL" || typeString_ == "XSD")
    dumper_ = new XMLWriter();
  else if (typeString_ == "HTML")
    dumper_ = new XMLWriter(true); 
  else if (typeString_ == "CSV")
    dumper_ = new CSVWriter();
  else if (typeString_ == "SXF")
    dumper_ = new SXFWriter();
  else if (typeString_ == "TXT")
    dumper_ = new DotWriter();
  else if (typeString_ == "CSS")
    dumper_ = new CSSWriter();
  else
    dumper_ = new XMLWriter(); // xml output is the default
}

void Putter::update(const DOMString& destName) {
  // replace wildcard, if there is one
  destPath_ = destSpec_;
  destPath_.change("*", destName);
  setDumper(true);
}

}
