/***************************************************************************
                          reader.cpp  -  description
                             -------------------
    begin                : Sat Jan 24 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <sstream>
#include <iostream>
#include "reader.h"
#include "writer.h"
#include "utf16converter.h"
#include "utf8converter.h"

using namespace std;

namespace limpid {

Reader::Reader(bool isConsole):
    inputStream_(new InputStream(isConsole)), converter_(new UTF8Converter),
    input_(0), isDefault_(true) {
  setEncoding(UTF8);
  input_ = inputStream_->input_;
  ungot_ = false;
}

Reader::Reader(const URL& url):
    inputStream_(new InputStream(url)), converter_(new UTF8Converter),
    input_(0), isDefault_(false) {
  input_ = inputStream_->input_;
  skipBOM();
  ungot_ = false;
}

void Reader::setEncoding(Encoding enc) {
  if (isDefault_) return;  // cannot alter console encoding

  if (enc == UTF8)
    converter_ = new UTF8Converter;
  else if (enc == UTF16LE)
    converter_ = new UTF16Converter(false); // little endian
  else
    converter_ = new UTF16Converter(true); // big-endian
}

void Reader::skipBOM() {
  if (!input_) return;

  char c;
  input_->get(c);
  unsigned char uc = c;
  if (uc == 0xFE || uc == 0xFF) {
    input_->get(c);
    if (uc == 0xFE) setEncoding(UTF16);
    else setEncoding(UTF16LE);
  }
  else input_->unget();
}

Reader Reader::KeyBd;

}
