/***************************************************************************
                          reporter.cpp  -  description
                             -------------------
    begin                : Fri Jun 2 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "reporter.h"
#include "element.h"
#include "liststream.h"
#include "nodestream.h"
#include "traverser.h"
#include "writer.h"
#include "xmlwriter.h"
#include "csvwriter.h"
#include "sxfwriter.h"
#include "traverser.h"
#include "documenttype.h"
#include "sizer.h"
#include "dombuilder.h"
#include "dtdgetter.h"

namespace limpid {

Reporter::Reporter(): dtdConverter_(0), validationProcessor_(new ValidationProcessor),
    defined_(false) {}

Reporter::Reporter(const DOMString& validatorString): defined_(false)
{
  if (!validatorString.empty()) {
    Reader reader(validatorString);
    dtdConverter_ = new DTDConverter(reader);
    defined_ = true;
  }

  DOMString validatorCopy = validatorString;
  validatorCopy.toUpperCase();

  if (validatorCopy.endsWith(".DTD")) {
    DTDGetter dtdGetter(validatorString);
    validationProcessor_->applySchema(dtdGetter.getDocument());
    defined_ = true;
  }
  else if (validatorCopy.endsWith("XSD")) {
    DOMBuilder domBuilder(validatorString);
    Document schemaDocument = domBuilder.build();
    validationProcessor_->applySchema(schemaDocument);
    defined_ = true;
  }

  // other file formats are presently ignored
}

Document& Reporter::processDocument(Document& document) {
  Traverser validationTraverser(*validationProcessor_);
  validationTraverser.traverse(document);

  if (!defined_) {
    Sizer sizer(false);
    Traverser sizeTraverser(sizer, false, true); // ignore endnodes, use pre/post
    sizeTraverser.traverse(document);

    wrt::Console << "Total size: " << sizer.getTotalSize() << " bytes for " << sizer.getTotalNodes();
    wrt::Console << " nodes and " << sizer.getTotalAttrs() << " attributes" << wrt::endl;
    wrt::Console << "There are " << sizer.getMultiples() << " nodes with multiple copies" << wrt::endl;
    wrt::Console.flush();
  }

  return document;
}

}
