/***************************************************************************
                          rjxwriter.h  -  description
                             -------------------
    begin                : Tue Mar 21 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RJXWRITER_H
#define RJXWRITER_H

#include "dumper.h"
#include "writer.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class DOMString;
  class Element;
  class EndNode;

  class RJXWriter : public Dumper  {
  public:
    RJXWriter();
    RJXWriter(const DOMString& destString);
    RJXWriter(CharacterWriter& characterWriter);

    Node& process(Node& node);
    void flush() { if (writer_) writer_->flush(); }
    void update(const DOMString& destString);
    RJXWriter& copy() { return *(new RJXWriter(*this)); }

  private:
    Pointer<CharacterWriter> writer_;
    int depth_;

    void processElement(Node& node);
    void processEndNode(Node& node);
    void processText(Node& node);
    void processCDATASection(Node& node);
    void processXMLDeclaration(Node& node);
    void processProcessingInstruction(Node& node);
    void processComment(Node& node);
    void processDocumentType(Node& node);
    void processAttributes(Node& node);
    DOMString& escapeString(DOMString& valueString);
    void nest() { ++depth_; }
    void denest() { --depth_; }
    void indent(Node& node);
  };
}

#endif
