/***************************************************************************
                          schemabuilder.h  -  description
                             -------------------
    begin                : Sat Feb 7 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCHEMABUILDER_H
#define SCHEMABUILDER_H

/**
  *@author John Redmond
  */

#include <set>
#include "list"
#include <map>
#include "documentprocessor.h"
#include "domutils.h"
#include "domstring.h"
#include "stringwriter.h"
#include "dtdconverter.h"

namespace limpid {

class SchemaBuilder: public DocumentProcessor, public DOMUtils {
  public:
    SchemaBuilder();
    SchemaBuilder(const DOMString& pathString);

    Document& processDocument(Document& inputDocument);
    SchemaBuilder& copy() { return *(new SchemaBuilder(*this)); }

    int getMaxCount(const DOMString& mapString);

  private:
    DOMString outputPath_;
    Writer writer_;
    StringWriter stringWriter_;
    Pointer<DTDConverter> dtdConverter_;
    bool convert_;

    struct ElementSummary {
      DOMString name;
      int first, last;
      bool hasText;
    };


    typedef std::vector<DOMString> StringList;

    typedef std::pair<int, int> Limits;
    typedef std::map<DOMString, Limits> ChildMap;
    typedef std::set<DOMString> StringSet;
    typedef std::map<DOMString, StringList> ListMap;

    struct ChildDetails {
      DOMString parentName;
      StringSet childSet;
      StringList childList;
    };

    typedef std::map<DOMString, ChildDetails> ParentMap;
    typedef std::map<DOMString, StringSet> SiblingMap;
    typedef std::map<DOMString, StringSet> AttributeMap;
    typedef std::map<DOMString, Limits> CountMap;


    ParentMap masterMap_;
    SiblingMap siblingMap_;
    CountMap countMap_;
    StringList masterList_, attributeList_;
    AttributeMap attributeMap_;
    StringSet attributeSet_;

    void mapElements(Element& rootElement);
    void setChildren(const DOMString& elementName, Element& element);
    void setAttributes(const DOMString& elementName, Element& element);
    void setChild(const DOMString& parentName, Node& childNode);
    void buildDTD();
    void buildAttributeDTD(const DOMString& parentName);
    void buildElementDTD(const DOMString& parentName, StringList& childList);//, ChildMap& childMap);
    void buildDTDChoice(const DOMString& parentName, StringList& hitList);
    void collectChoices(const DOMString& parentName, StringList& siblingList, StringList& sourceList, StringList& resultList, StringSet& usedSet);

    void addToMaps(const DOMString& elementName);
    int countSame(Node& node);
    DOMString getMapName(const DOMString& parentName, const DOMString& childName);
    DOMString getMapName(Node& node);

    void printList(StringList& elementList);
  };
}

#endif
