/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDSCHEMAINSPECTOR_H
#define LIMPIDSCHEMAINSPECTOR_H

#include <map>
#include <set>
#include <list>
#include "converter.h"
#include "element.h"
#include "pointer.h"
#include "documentprocessor.h"
#include "domutils.h"
#include "schemabuilder.h"

namespace limpid {

/**
	@author John Redmond <john_redmond@optusnet.com.au>
*/
  class SchemaInspector : public DocumentProcessor, public DOMUtils
{
public:
  SchemaInspector(const DOMString& pathLength): pathLength_(pathLength.size() ? Converter::toInt(pathLength) : 99) {}
  Document& processDocument(Document&);
  SchemaInspector& copy() { return *(new SchemaInspector(*this)); }

private:
  typedef std::map<DOMString, Element*> ElementMap;
  typedef std::set<DOMString> StringSet;
  typedef std::list<DOMString> StringList;
  typedef std::list<Element*> ElementList;

  Document document_;
  int pathLength_;
  ElementMap complexMap_;
  StringList complexList_;
  StringSet mixedContent_, outputPaths_;
  ElementList topList_;

  void clearStructures();
  void registerElements(Element& rootElement);
  void expandElements(Element& rootElement);
  bool expandElement(Element& rootElement, bool includeRoot);
  void setPathDetails(Element& sourceElement, Element& resultElement);
  void getPathList(Element& leafElement, StringList& resultList);
  DOMString getPathString(StringList& pathList);

  void registerComplexType(Element& complexElement);
  void registerInstance(Element& instanceElement);

  void appendEntry(const DOMString& pathString, const DOMString& typeString, Element& parentElement);
  DOMString getStyleString(Element& element);
  DOMString getType(Element& element);
  int getAttributeCount(Node& node, const DOMString& attrName, bool useParent = true);

  bool isFirst(Node& node) { return !node.getPreviousSibling().isValid(); }
  bool isSecond(Node& node) { return node.getPreviousSibling().isValid() && !node.getPreviousSibling().getPreviousSibling().isValid(); }
  bool isLast(Node& node) { return !node.getNextSibling().isValid(); }
  bool inSequence(Node& node) { return node.getParentNode().getLocalName() == "sequence"; }
  bool inChoice(Node& node) { return node.getParentNode().getLocalName() == "choice"; }
  bool isSingle(Element& element) { DOMString attrString = element.getAttribute("maxOccurs"); return attrString.empty() || attrString == "1"; }
  bool isDouble(Element& element) { DOMString attrString = element.getAttribute("maxOccurs"); return attrString == "2"; }

  bool hasText(Element& element) { return element.getAttribute("mixed") == "true"; }
  bool isEmpty(Element& element);
  bool isMixed(const DOMString& elementName) { return mixedContent_.find(elementName) != mixedContent_.end(); }
  bool isParentMixed(Element& element);

  bool isRecursion(Element& element);
  void updateContent(Element& element);
};

}

#endif
