/***************************************************************************
 *   Copyright (C) 2007 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "sequencepoint.h"
#include "liststream.h"
#include "choicepoint.h"
#include "elementpoint.h"

namespace limpid {

SequencePoint::SequencePoint(): MatchPoint("sequence", true), childPointers_(new PointList), firstPoint_(0), activePoint_(0) {
  minHits_ = 1;
  maxHits_ = 1;
}

SequencePoint::SequencePoint(const Element& element, const DOMString& name): MatchPoint(element.getLocalName(), true), childPointers_(new PointList), firstPoint_(0), activePoint_(0) {
  setFrequency(element);
  mixed_ = static_cast<const Element &>(element.getParentNode()).getAttribute("mixed") == "true";
  addContent(element);
  type_ = SEQUENCE;
}

bool SequencePoint::test(Node& node) {
  for (int index = 0; index < childPointers_->size(); ++index) {
    MatchPoint& indexPoint = *childPointers_->at(index);

    if (indexPoint.test(node)) {
      activePoint_ = &indexPoint;
      return true;
    }
if (indexPoint.isRequired()) {
wrt::Console << "L: " << node.getPosition() << " FOUND " << node.getNodeName() << " INSTEAD OF " << indexPoint.getName() << wrt::endl;
      return false;
    }
  }

  activePoint_ = 0;
  return false;
}

void SequencePoint::traverse(const DOMString& indent) {
  DOMString newIndent = indent;
  newIndent += "  ";
  doTraversal(newIndent);
}

void SequencePoint::addPoint(MatchPoint& point) {
  MatchPoint& pointCopy = point.copy();
  pointCopy.setParentPoint(*this);

  if (!firstPoint_)  // set first point
    firstPoint_ = &pointCopy;

  else if (!childPointers_->empty())
    childPointers_->back()->setNextPoint(pointCopy);

  childPointers_->push_back(&pointCopy);
}

void SequencePoint::addContent(const Element& element) {
  ListStream childStream(const_cast<Element &>(element));

  while (true) {
    Node& node = childStream.nextNode();
    if (!node.isValid()) break;

    if (!isXSDElement(node)) continue;
    
    Element& element = static_cast<Element &>(node);
    DOMString nodeName = element.getLocalName();

    if (nodeName == "element") {
      ElementPoint point(element);
      addPoint(point);
      continue;
    }
    if (nodeName == "choice") {
      ChoicePoint point(element);
      addPoint(point);
      continue;
    }
    if (nodeName == "sequence") {
      SequencePoint point(element);
      addPoint(point);
      continue;
    }
  }
}

void SequencePoint::clearHits() {
  hits_ = 0;

  if (childPointers_->size())
    activePoint_ = (*childPointers_)[0];
  else
    activePoint_ = 0;

  for (int index = 0; index < childPointers_->size(); ++index)
    (*childPointers_)[index]->clearHits();
}

void SequencePoint::doTraversal(const DOMString& indent) {
  for (int index = 0; index < childPointers_->size(); ++index)
    childPointers_->at(index)->traverse(indent);
}

}
