/***************************************************************************
                          singlestream.cpp  -  description
                             -------------------
    begin                : Mon May 24 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "singlestream.h"
#include "document.h"

namespace limpid {

SingleStream::SingleStream(const DOMString& type): NodeSource(),
      type_(SELF), root_(0), lastNode_(0) {
  configure(type);
}

Node& SingleStream::nextNode() {
  if (lastNode_) return Node::failNode;
  else if (type_ == SELF)
    lastNode_ = root_;
  else if (type_ == DOCUMENT) {  // this becomes a flag for further calls
    lastNode_ = root_;
    return root_->getOwnerDocument();
  }
  else
    lastNode_ = &root_->getParentNode();
  return *lastNode_;
}

void SingleStream::setRoot(const Node& root) {
  root_ = &const_cast<Node &>(root);
  lastNode_ = 0;
}

void SingleStream::configure(DOMString typeString) {
  typeString.toUpperCase();
  if (typeString == "PARENT" || typeString == "TEST-PARENT")
    type_ = PARENT;
  else if (typeString == "ROOT")
    type_ = DOCUMENT;
}

}
