/***************************************************************************
                          specifiedattributestemplate.cpp  -  description
                             -------------------
    begin                : Tue Mar 22 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "specifiedattributestemplate.h"
#include "element.h"

namespace limpid {

SpecifiedAttributesTemplate::SpecifiedAttributesTemplate(): attributes_(new AttributeList) {
}

// We work through the attributes in reverse order and, as each attribute
// is processed, its key is added to a set. If an identically-named attribute
// is later encountered, and its key found in the set, it is not processed.
// Given the sequence in which the attributes are assembled in the attributes
// vector, this has the outcome of using only the last attribute with a particular
// name, which is what would happen if attributes were set on an element, and subsequently
// overwritten.
// Note, too, that the attribute names and values are determined only at the last
// minute, by evaluation of expressions. For this reason, it is only then that a key
// could be used to avoid duplicates.
bool SpecifiedAttributesTemplate::process(XSLContext& parentContext) {
  XSLContext context = parentContext;
  CharacterWriter & writer = parentContext.getWriter();
  KeySet keySet;

  AttributeList::reverse_iterator p = attributes_->rbegin();
  while (p != attributes_->rend()) {
    XPathString key = p->first.getString(context);//parentContext);
    if (keySet.find(key) == keySet.end()) {
      keySet.insert(key);

      CompoundTemplate& valueTemplate = p->second;
      writer << " " << key << "=" << "\"";  // write start of attribute
      writer << valueTemplate.getString(context); // write value of attribute
      writer << "\"";
    }
    ++p;
  }

  return true;
}

bool SpecifiedAttributesTemplate::processNode(XSLContext& parentContext, Node& node) {
  if (node.getNodeType() != Node::ELEMENT_NODE) return true;
  Element& element = static_cast<Element &>(node);

  AttributeList::iterator p = attributes_->begin();
  while (p != attributes_->end()) {
    XPathString key = p->first.getString(parentContext);
    XPathString value = p->second.getString(parentContext);
    element.setAttribute(key, value);
    ++p;
  }

  return true;
}

void SpecifiedAttributesTemplate::addAttribute(CompoundTemplate& keyTemplate, CompoundTemplate& valueTemplate) {
  attributes_->push_back(AttributePair(keyTemplate, valueTemplate));
}


}
