/***************************************************************************
                          step.cpp  -  description
                             -------------------
    begin                : Thu Jun 10 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "step.h"
#include "writer.h"
#include "test.h"

namespace limpid {

Step::Step(): predicates_(new Predicates) {
}

NodeSet Step::processNodeSet(NodeSet& nodeSet, Context& context, bool exact) {
  NodeSet resultSet;
  size_t length = nodeSet.getLength();

  for (size_t index = 0; index < length; ++index)
    processNode(nodeSet.getItem(index), resultSet, context);
  if (!resultSet.getLength() || !predicates_->size()) return resultSet;

  if (!exact) return resultSet;

  return applyFilters(resultSet, context_);
}

// Note that the context is not used here.
void Step::processNode(Node& root, NodeSet& resultSet, Context&) {
  if (!test_) {
    wrt::Console << "NULL TEST" << wrt::endl;
    return;
  }

  stream_->setRoot(root);

  while (true) {
    Node& node = stream_->nextNode();
    if (!node.isValid()) break;

    if (test_->test(node)) {
      resultSet.addNode(node);
    }
  }
}

// This option is factored out of the node collection so that it is easy to
// make it an option (exact parameter = false).
NodeSet Step::applyFilters(NodeSet& testSet, Context& context) {
  NodeSet resultSet = testSet;
  context.setCount(testSet.getLength());

  Predicates::iterator p = predicates_->begin();
  while (p != predicates_->end()) {
    PredicateFilter filter = *p;
    resultSet = filter->processNodeSet(resultSet, context, true);
    if (!resultSet.getLength()) break;
    ++p;
  }
  return resultSet;
}

}
