/***************************************************************************
                          stepper.cpp  -  description
                             -------------------
    begin                : Tue Apr 18 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stepper.h"
#include "document.h"

namespace limpid {

Stepper::Stepper(const DOMString& xslPath, bool useBuilding):
    writer_(new Writer), styleSheet_(new StyleSheet(xslPath, true)), nodeStack_(new NodeStack),
    beginNode_("_STEPPER_BEGIN_"), endNode_("_STEPPER_END_"),
    useBuilding_(useBuilding)
  {
    name_ = "stepper";
    styleSheet_->setWriter(*writer_);
  }

Stepper::Stepper(const DOMString& xslPath,
    const DOMString& outputPath, bool useBuilding):
    writer_(new Writer(outputPath)), styleSheet_(new StyleSheet(xslPath, true)), nodeStack_(new NodeStack),
    beginNode_("_STEPPER_BEGIN_"), endNode_("_STEPPER_END_"),
    useBuilding_(useBuilding)
  {
    styleSheet_->setWriter(*writer_);
    name_ = "stepper";
  }

Node& Stepper::process(Node& node) {
  // respond only to end nodes (== nodes complete with child nodes)
  if (node.getNodeType() == Node::ELEMENT_NODE &&
      (node.getChildNodes().getLength() || node.isParent())) {
    nodeStack_->push_back(&node);
  }

  if (node.getNodeType() == Node::END_NODE) {
    if (nodeStack_->empty()) return node; // should never happen

    Node* startNode = nodeStack_->back();
    Node* joinedNode = &static_cast<EndNode &>(node).getStartNode();
    nodeStack_->pop_back();

    if (styleSheet_->hasStep(node)) {
      styleSheet_->transformNode(*joinedNode, useBuilding_);
      // signal a noop to any following processors (should there be any?)
      return Node::ignoreNode;
    }
  }

  return node;
}

void Stepper::start() {
  // this is likely to use escaped tags disguised within text,
  // and Text cannot be appended to a document.
  // therefore, building is turned off within the transformation.
  styleSheet_->transformNode(beginNode_, false);
}

void Stepper::finish() {
  // this is likely to use escaped tags disguised within text,
  // and Text cannot be appended to a document.
  // therefore, building is turned off within the transformation.
  styleSheet_->transformNode(endNode_, false);
  writer_->flush();
}

}
