/***************************************************************************
                          stringcomparison.cpp  -  description
                             -------------------
    begin                : Sat Jun 18 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stringcomparison.h"
#include "xpathstring.h"
#include "boolean.h"
#include "number.h"
#include "context.h"

namespace limpid {

StringComparison::StringComparison():
    logic_(true) {}

StringComparison::StringComparison(const DOMString& typeString):
    logic_(typeString == "=") {}

Boolean StringComparison::getBoolean(Context& context) {
  if (getLength() != 2) return false;  // should be impossible

  bool firstIsSet = components->at(0)->isNodeSet();
  bool secondIsSet = components->at(1)->isNodeSet();

  if (!firstIsSet && !secondIsSet)
    return comparePair(*components->at(0), *components->at(1), context);
  if (firstIsSet && !secondIsSet)
    return compareFirstNodeSet(*components->at(0),
      components->at(1)->getString(context), context);
  if (!firstIsSet && secondIsSet)
    return compareSecondNodeSet(components->at(0)->getString(context),
      *components->at(1), context);
  return compareNodeSets(*components->at(0), *components->at(1), context);
}

bool StringComparison::comparePair(XPathExpression& first, XPathExpression& second,
      Context& context) {
  XPathString s0 = first.getString(context);
  XPathString s1 = second.getString(context);
  return compareStrings(s0, s1);
}

bool StringComparison::compareFirstNodeSet(XPathExpression& first,
      const XPathString& secondString, Context& context) {
  NodeSet firstSet = first.getNodeSet(context);
  size_t length = firstSet.getLength();
  if (!length) return false;

  for (size_t index = 0; index < length; ++index) {
      DOMString firstString = firstSet.getItem(index).getValue();
      if (compareStrings(firstString, secondString)) return true;
  }

  return false;
}

bool StringComparison::compareSecondNodeSet(const XPathString& firstString,
      XPathExpression& second, Context& context) {
  NodeSet secondSet = second.getNodeSet(context);
  size_t length = secondSet.getLength();
  if (!length) return false;

  for (size_t index = 0; index < length; ++index) {
      DOMString secondString = secondSet.getItem(index).getValue();
      if (compareStrings(firstString, secondString)) return true;
  }

  return false;
}

bool StringComparison::compareNodeSets(XPathExpression& first,
      XPathExpression& second, Context& context) {
  NodeSet secondSet = second.getNodeSet(context);
  size_t secondLength = secondSet.getLength();

  for (size_t index = 0; index < secondLength; ++index) {
    XPathString secondString = secondSet.getItem(index).getValue();
    if (compareFirstNodeSet(first, secondString, context)) return true;
  }
  return false;
}

}
