/***************************************************************************
                          stringgenerator.cpp  -  description
                             -------------------
    begin                : Fri May 28 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stringgenerator.h"
#include "writer.h"

namespace limpid {

StringGenerator::StringGenerator(const DOMString& typeString):
      ParentComponent(), type_(SUBSTRING_BEFORE) {
  setType(typeString);
}

XPathString StringGenerator::getString(Context& context) {
  DOMString childStr = childString(context);

  if (type_ == STRING)
    return childStr;
  if (type_ == NORMALIZE_SPACE)
    return childStr.normalizeWhitespace();
  if (type_ == ESCAPE_XML)
    return childStr.escapeBra();
  if (type_ == ESCAPE_TEX)
    return childStr;///.escapeTex();

  if (type_ == CONCAT)
    return concat(childStr, context);

  if (type_ == SUBSTRING) return substring(childStr, context);
  if (type_ == SUBSTRING_BEFORE) return substringBefore(childStr, context);
  if (type_ == SUBSTRING_AFTER) return substringAfter(childStr, context);

  return concat(childStr, context);
}

XPathString StringGenerator::childString(Context& context) {
  if (!getLength()) return "";
  return components->at(0)->getString(context);
}

XPathString StringGenerator::substring(DOMString& firstString,
    Context& context) {
    if (getLength() < 2)
      return "substring() NEEDS 2 PARAMS";

  size_t startIndex =
    static_cast<size_t>(components->at(1)->getNumber(context).getInt()) - 1;

  unsigned int sublength;

  if (getLength() > 2)
    sublength = static_cast<unsigned int>(components->at(2)->getNumber(context).getInt());
  else
    sublength = firstString.size() - startIndex;

  return firstString.substring(startIndex, sublength);
}

XPathString StringGenerator::substringBefore(DOMString& firstString,
    Context& context) {
  if (getLength() < 2)
    return "substring-before() NEEDS 2 PARAMS";

  XPathString refString = components->at(1)->getString(context);
  size_t offset = firstString.find(refString);

  if (offset == firstString.npos) return firstString;
  return firstString.substring(0, offset);
}

XPathString StringGenerator::substringAfter(DOMString& firstString,
    Context& context) {
  if (getLength() < 2)
    return "substring-after() NEEDS 2 PARAMS";

  size_t firstSize = firstString.size();
  XPathString secondString = components->at(1)->getString(context);

  unsigned int offset = firstString.find(secondString);
  if (offset == firstString.npos) return firstString;

  offset += secondString.size();

  return firstString.substring(offset, firstSize - offset);
}

XPathString StringGenerator::concat(DOMString& firstString,
    Context& context) {
  if (getLength() < 2)
    return "concat() NEEDS AT LEAST 2 PARAMS";

  XPathString returnString = firstString;

  size_t length = getLength();
  for (size_t index = 1; index < length; ++index) {
    XPathString s = components->at(index)->getString(context);
    returnString += s;
  }

  return returnString;
}

void StringGenerator::setType(const DOMString& typeString) {
  if (typeString == "string") type_ = ESCAPE_XML;
  else if (typeString == "normalize-space") type_ = NORMALIZE_SPACE;
  else if (typeString == "escape-xml") type_ = ESCAPE_XML;
  else if (typeString == "escape-tex") type_ = ESCAPE_TEX;
  else if (typeString == "concat") type_ = CONCAT;
  else if (typeString == "substring-before") type_ = SUBSTRING_BEFORE;
  else if (typeString == "substring-after") type_ = SUBSTRING_AFTER;
  else if (typeString == "substring") type_ = SUBSTRING;
}

}
