/***************************************************************************
                          stringtranslator.cpp  -  description
                             -------------------
    begin                : Sat Jun 25 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stringtranslator.h"
#include "boolean.h"
#include "xpathstring.h"
#include "number.h"
#include "context.h"
#include "xpathfactory.h"

namespace limpid {

StringTranslator::StringTranslator(Expr& expr,
    const DOMString& from, const DOMString& to):
    expr_(expr), charMap_(new CharMap) {
  setupMap(from, to);
}

XPathString StringTranslator::getString(Context& context) {
  XPathString string = expr_.getString(context);

  if (!charMap_->size())
    return string;  // no translations defined

  for (size_t index = 0; index < string.size(); ++index) {
    CharMap::iterator p = charMap_->find(string[index]);
    if (p != charMap_->end())
      string[index] = p->second;
  }
  
  return string;
}

void StringTranslator::setupMap(const DOMString& fromString, const DOMString& toString) {
  size_t fromLength = fromString.size();
  size_t toLength = toString.size();

  if (fromLength > toLength)
    fromLength = toLength;


  for (size_t index = 0; index < fromLength; ++index)
    charMap_->insert(std::pair<Char, Char>(fromString[index], toString[index]));

}

}
