/***************************************************************************
                          stripper.cpp  -  description
                             -------------------
    begin                : Fri Mar 31 2006
    copyright            : (C) 2006 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stripper.h"
#include "text.h"
#include "nodestream.h"

namespace limpid {

Stripper::Stripper(bool keepTags): keepTags_(keepTags), root_("root") {
  document_.appendChild(root_);
}

Document& Stripper::processDocument(Document& document) {
  NodeStream stream(document);

  while (true) {
    Node& node = stream.nextNode();
    if (!node.isValid()) break;

    if (node.getNodeType() != Node::TEXT_NODE) continue;

    DOMString textValue = node.getNodeValue().trim();
    if (!keepTags_)
      textValue += '\n';
      
    Text text(textValue);

    if (!keepTags_) {
      root_.appendChild(text);
      continue;
    }
    
    Node &parentNode = node.getParentNode();
    if (parentNode.getNodeType() != Node::ELEMENT_NODE) continue;

    DOMString parentName = static_cast<Element &>(parentNode).getAttribute("tag");
    if (parentName.empty())
      parentName = parentNode.getNodeName();
    Element newParent(parentName);

    newParent.appendChild(text);
    root_.appendChild(newParent);
  }

  return document_;
}

}


