/***************************************************************************
                          stylesheet.h  -  description
                             -------------------
    begin                : Mon Sep 27 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef STYLESHEET_H
#define STYLESHEET_H

#include "templatemanager.h"
#include "whitespacemanager.h"
#include "xslresources.h"
#include "xslcontext.h"
#include "domwriter.h"
#include "configurationset.h"
#include "selectionmanager.h"
#include "defaulttemplate.h"
#include "domstring.h"
#include "document.h"
#include "pointer.h"


/**
  *@author John Redmond
  */

namespace limpid {
  class NodeList;
  class Node;

  class StyleSheet {
  public:
    StyleSheet(bool verbose = false);
    StyleSheet(const DOMString& fileName, bool isBuilding = false, bool verbose = false);

    // highest-level interfaces, as used by Transformer
    void transform(const DOMString& sourcePath);
    void transform(const DOMString& sourcePath, const DOMString& destPath);

    // intermediate-level interfaces, as used by XSLOutputter
    void transformDocument(Document& sourceDocument);
    void transformDocument(Document& sourceDocument, const DOMString& destPath);

    // lowest-level interfaces, as used by Stepper
    void transformNode(Node& sourceNode, bool isBuilding);
    void transformNode(Node& sourceNode,
      const DOMString& destPath, bool isBuilding);
    bool hasStep(Node& node) { return xslResources_.isStep(node); }
    void setWriter(CharacterWriter& writer);

    // interface used by Mutator
    Document& buildDocument(Node& sourceNode);

    DOMString getMethod() { return configurationSet_->method; }

  private:
    bool isBuilding_, verbose_;
    Pointer<ConfigurationSet> configurationSet_;

    TemplateManager templateManager_;
    WhitespaceManager whitespaceManager_;
    XSLContext context_;
    XSLResources xslResources_;
    SelectionManager selectionManager_;
    DefaultTemplate defaultTemplate_;
    Element templateDocumentElement_;
    Document destinationDocument_;

    void doBuild(Document& sourceDocument, const DOMString& destPath);
    void writeDocument(Node& sourceNode);
    void writeStream(Node& sourceNode);
    void writeStream(Node& sourceNode, const DOMString& destString);

    void configure(const DOMString& fileName, bool isBuilding);
    void configureTemplates(Element& documentElement);
    void configureSimplifiedStyleSheet(Element& documentElement);

    NodeList getChildSheets(Node& anchor, bool includeAnchor);
    void visit(Node& root, NodeList& resultList, bool includeRoot);
    Document getSourceDocument(const DOMString& sourcePath);
  };
}


#endif
