/***************************************************************************
                          documentmanager.h  -  description
                             -------------------
    begin                : Thu Sep 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SYSTEMMANAGER_H
#define SYSTEMMANAGER_H

#include <list>
#include "domstring.h"
#include "converter.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class Node;
  class Attr;
  class Document;
  class NodeManager;

  class SystemManager: protected Converter {
  public:
    enum Size { PERMISSIONS_SIZE = 20 };
    enum Response {IGNORE, WARN, THROW };

  	~SystemManager();

    static char errorNames[][30];
    static SystemManager& getSystemManager();

    Response getResponse() { return response; }

    // the namespaces flag is not presently used, but is included here
    // as a globally-available hook for further development.
    bool isUsingNamespaces() { return namespaces; }
    void setUsingNamespaces(bool flag) { namespaces = flag; }

    // flags for handling of text content
    void setTrimmingText(bool flag) { trimmingText = flag; }
    bool isTrimmingText() { return trimmingText; }
    void setNormalizingText(bool flag) { normalizingText = flag; }
    bool isNormalizingText() { return normalizingText; }

    // the active flag determines whether there are any correctness checks
    // during appending child nodes and setting attributes. It defaults to true,
    // but should be true to monitor hand-coded dom operations.
    void setActive(bool flag) { active = flag; }
    bool isActive() { return active; }

    // the response flag controls how build errors are handled.
    // It defaults to WARN.
    void setResponse(Response resp) { response = resp; }
    void processResponse(int errorType, DOMString& spec);

    // check for any error conditions and act appropriately.
    bool checkCorrectDocument(const Node& parent, const Node& child);
    bool checkModificationPermission(const Node& parent);
    bool checkAttributeNotInUse(const Attr& attr);
    bool checkAppendPermission(const Node&, const Node&);

    // respond to known error conditions.
    void processIndexError(size_t index);
    void processNotFoundError(Node& node);
    void processInUseError(Node& node);

  private:
  	SystemManager();

    bool namespaces, active, trimmingText, normalizingText;
    int *permissions;
    Response response;

    void setPermission(int, int);
    void populatePermissions();
  };
}
#endif
