/***************************************************************************
                          template.h  -  description
                             -------------------
    begin                : Tue Sep 28 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TEMPLATE_H
#define TEMPLATE_H

#include "copied.h"
#include "writer.h"
#include "xslcontext.h"
#include "expr.h"

/**
  *@author John Redmond
  */
namespace limpid {
  class XSLResources;
  class ExpressionMap;

  class Template: public Copied {
  public:

    static Template failTemplate;

    virtual bool process(XSLContext&) { return true; }   // a noop
    virtual bool processNode(XSLContext&, Node&) { return true; }  // a noop
    virtual void addTemplate(Template&) { wrt::Console << "NO ADD";}  // a noop
    virtual void addSortTemplate(Template&) { wrt::Console << "NO ADD SORTER";}  // a noop
    virtual void setResources(XSLResources&) { wrt::Console << "!!!"; } // a noop
    virtual void setName(DOMString&) {} // a noop

    virtual void setParameter(const DOMString&, const Expr&) {}  // a noop
    virtual void passParameters(const ExpressionMap&, XSLContext&) {} // a noop

    virtual size_t getLength() { return 0; }
    virtual XPathString getString(XSLContext&) { return ""; }
    virtual bool isValid() { return valid_; }
    virtual bool matchNode(Node&, bool) { return false; }
    virtual DOMString getMode() { return "#none"; }
    virtual bool isAttribute() { return false; }
    Template& copy() { return *this; }

    DOMString getMatchValue() { return matchValue_; }
    double getPriority() const { return priority_; }
    void setPriority(double priority) { priority_ = priority; }
    void setHasText(bool hasTxt) { hasText_ = hasTxt; }

    void setPreviousTemplate(Template& previous) { previousTemplate_ = &previous; }
    void setNextTemplate(Template& next) {nextTemplate_ = &next; }
    Template& getPreviousTemplate() {
      if (previousTemplate_) return *previousTemplate_;
      else return failTemplate; }
    Template& getNextTemplate() {
      if (nextTemplate_) return *nextTemplate_;
      else return failTemplate; }
 
    void nest(XSLContext& context) { context.getFormat().nest(); }
    void deNest(XSLContext& context) { context.getFormat().deNest(); }
    void writeIndent(XSLContext& context) {
      context.getWriter() << context.getFormat().indentation; }
    void writeNewline(XSLContext& context) {
      context.getWriter() << context.getFormat().newline; }
    void writeFormat(XSLContext& context) {
      context.getWriter() << context.getFormat().newline << context.getFormat().indentation; }

  protected:
  	Template();
    Template(const DOMString& matchValue);

    DOMString matchValue_;
    double priority_;
    bool valid_, hasText_, isText_;
    Template *previousTemplate_, *nextTemplate_; // not managed
  };
}

#endif
