/***************************************************************************
                          templatelauncher.cpp  -  description
                             -------------------
    begin                : Mon Sep 19 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "templatelauncher.h"
#include "selectionmanager.h"
#include "defaulttemplate.h"
#include "xslcontext.h"
#include "node.h"
#include "liststream.h"
#include "expressionmap.h"

namespace limpid {

TemplateLauncher::TemplateLauncher(SelectionManager& selectionManager,
      DefaultTemplate& defaultTemplate):
  selectionManager_(&selectionManager), defaultTemplate_(&defaultTemplate),
  parameters_(0) {}

TemplateLauncher::TemplateLauncher(SelectionManager& selectionManager,
      DefaultTemplate& defaultTemplate, ExpressionMap& parameters):
  selectionManager_(&selectionManager), defaultTemplate_(&defaultTemplate),
  parameters_(&parameters) {}

bool TemplateLauncher::process(XSLContext& parentContext) {
  Node& activeNode = parentContext.getNode();
  int nodeType = activeNode.getNodeType();
  if (nodeType == Node::DOCUMENT_TYPE_NODE) return true; // no action here

  Template& matchedTemplate = selectionManager_->getMatchedTemplate(activeNode);
  if (matchedTemplate.isValid()) {
    matchedTemplate.passParameters(*parameters_, parentContext);
    return matchedTemplate.process(parentContext);
  }

  // fall through to here if there was no match

  // no action required here
  if (nodeType == Node::PROCESSING_INSTRUCTION_NODE ||
      nodeType == Node::COMMENT_NODE) return true;

  // now consider simple data types
  if (nodeType == Node::TEXT_NODE || nodeType == Node::CDATA_SECTION_NODE || nodeType == Node::ATTRIBUTE_NODE)
    return defaultTemplate_->process(parentContext);

  // otherwise work through the children of the unmatched node
  ListStream listStream(activeNode);
  while (true) {
    Node& childNode = listStream.nextNode();
    if (!childNode.isValid()) break;

    XSLContext context;
    context.setParent(parentContext);
    context.setNode(childNode);
    process(context); // recurse
  }

  return true;
}

bool TemplateLauncher::processNode(XSLContext& parentContext, Node& node) {
/// wrt::Console << "TL " << parentContext.getNode() << wrt::endl;
  Node& activeNode = parentContext.getNode();
  int nodeType = activeNode.getNodeType();
  if (nodeType == Node::DOCUMENT_TYPE_NODE) return true; // no action here

  Template& matchedTemplate = selectionManager_->getMatchedTemplate(activeNode);
  if (matchedTemplate.isValid()) {
    matchedTemplate.passParameters(*parameters_, parentContext);
    return matchedTemplate.processNode(parentContext, node);
  }
  // fall through to here if there was no match

  // no action required here
  if (nodeType == Node::PROCESSING_INSTRUCTION_NODE ||
      nodeType == Node::COMMENT_NODE) return true;

  // now consider simple data types
  if (nodeType == Node::TEXT_NODE || nodeType == Node::CDATA_SECTION_NODE || nodeType == Node::ATTRIBUTE_NODE)
    return defaultTemplate_->processNode(parentContext, node);

  // otherwise work through the children of the unmatched node
  ListStream listStream(activeNode);
  while (true) {
    Node& childNode = listStream.nextNode();
    if (!childNode.isValid()) break;

    XSLContext context;
    context.setParent(parentContext);
    context.setNode(childNode);
    processNode(context, node); // recurse
  }

  return true;
}

}
