/***************************************************************************
                          templatelist.cpp  -  description
                             -------------------
    begin                : Wed Sep 29 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "templatelist.h"
#include "xslcontext.h"
#include "node.h"
#include "writer.h"

namespace limpid {

TemplateList::TemplateList(): templates_(new Templates),
    templatePointers_(new TemplatePointers) {}

void TemplateList::addTemplate(Template& tmp) {
  Template *cpy = &tmp.copy();

  if (templates_->size()) {
    templates_->back()->setNextTemplate(*cpy);
    cpy->setPreviousTemplate(*templates_->back());
  }
  
  templates_->push_back(cpy);
  templatePointers_->push_back(cpy);
}

Template& TemplateList::getItem(size_t index) {
  if (index >= templates_->size())
    return *templates_->at(templates_->size() - 1);
  else return *templates_->at(index);
}

bool TemplateList::process(XSLContext& context) {
  Templates::iterator p = templates_->begin();
  Templates::iterator pend = templates_->end();
  while (p != pend) {
    if (!((*p)->process(context))) break;
    ++p;
  }
  return true;
}

bool TemplateList::processNode(XSLContext& context, Node& node) {
  Templates::iterator p = templates_->begin();
  Templates::iterator pend = templates_->end();

  while (p != pend) {
    if (!((*p)->processNode(context, node))) break;
    ++p;
  }
  return true;
}

void TemplateList::doSort(bool verbose) {
  if (verbose) {
    Templates::iterator p = templates_->begin();
    Templates::iterator pend = templates_->end();

    while (p != pend) {
      wrt::Errors << (*p)->getPriority() << " TMP MATCH VAL BEFORE " << (*p)->getMatchValue() << wrt::endl;
      ++p;
    }
  }

  static CompTemplates comparator;
  std::stable_sort(templates_->begin(), templates_->end(), comparator);

  if (verbose) {
    Templates::iterator p = templates_->begin();
    Templates::iterator pend = templates_->end();

    while (p != pend) {
      wrt::Errors << (*p)->getPriority() << "TMP MATCH VAL AFTER " << (*p)->getMatchValue() << wrt::endl;
      ++p;
    }
  }
}

void TemplateList::list() {
  size_t length = getLength();
  for (size_t index = 0; index < length; ++index)
wrt::Console << getItem(index).getMatchValue() << ": " << index << " has " <<  getItem(index).getPriority() << wrt::endl;
}

}
