/***************************************************************************
                          templatemanager.h  -  description
                             -------------------
    begin                : Wed Sep 29 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TEMPLATEMANAGER_H
#define TEMPLATEMANAGER_H

#include "xslutility.h"
#include "xslresources.h"
#include "xslcontext.h"
#include "configurationset.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class DOMString;
  class Element;
  class Template;
  class SpecifiedAttributesTemplate;
  class ElementTemplate;
  class CopyTemplate;
  class ParentComponent;

  class TemplateManager {
  public:
    TemplateManager() {} // used just as a space filler
    TemplateManager(XSLResources& resources, XSLContext& context, bool imported);

    void configure(Element& documentElement, const ConfigurationSet& configurationSet,
      XSLResources& resources, bool isImported);
    void setWriter(Writer& writer) { context_.setWriter(writer); }
    XSLResources& getResources() { return resources_; }
    XSLContext& getContext() { return context_; }

  private:
    XSLResources resources_;
    XSLContext context_;
    bool isImported_;
    ConfigurationSet configurationSet_;

    void addDecimalFormat(Element& formatElement);
    void buildTemplate(Element& templateElement);
    void processSheet(Element& sheetElement);
    void setAttributeSets(Element& sheetElement);
    void processTopLevelElement(Element& element);
    void addTemplateContent(Template& parentTemplate, Element& templateElement);
    void processXSLElement(Template& parentTemplate, Element& xslElement);

    void addDefinedElement(Template& parentTemplate, Element& elementElement);
    void addLiteralElement(Template& parentTemplate, Element& elementElement);
    void addAttribute(Template& parentTemplate, Element& attributeElement);
    void addCopyNode(Template& parentTemplate, Element& copyElement);
    void addCopyOfNode(Template& parentTemplate, Element& copyOfElement);
    void addComment(Template& tmp, Element& commentElement);
    void addProcessingInstruction(Template& tmp, Element& instructionElement);
    void addMessage(Template& tmp, Element& messageElement);
    void addNumber(Template& tmp, Element& numberElement);

    void addVariable(Element& variableElement, XSLContext& context);
    void addParameters(Template& parentTemplate, Element& templateElement,
          const DOMString& tagName);
    void addDefinedAttributes(Template& asTemplate, Element& templateElement);
    void addAttributesFromSets(Template& elementTemplate, Element& templateElement);
    void addNamespaceAttr(ElementTemplate& elementTemplate,
      DOMString namespaceName, DOMString elementName);
    void addVariableTemplate(Template& parentTemplate, Element& xslElement);
    void addLiteralAttribute(ElementTemplate& elementTemplate,
      const DOMString& name, const DOMString& value);
    void addSimpleString(DOMString& displayString, Template& parentTemplate);

    bool isXSLElement(Node& node) { return XSLUtility::isXSLElement(node); }
    bool hasChildren(Node& parentNode);
    bool hasAttributeValueTemplate(DOMString string);
    bool isAttributeValueTemplate(DOMString string);

    DOMString getNodeName(Element& element) { return XSLUtility::getNodeName(element); }
    DOMString getExprString(DOMString& string);
    Expr getVariableExpr(Element& variableElement);
    Expr getStringExpr(DOMString string);
    Expr getDefinedAttributeExpr(Element& attrElement);
    DOMString addDelimiters(DOMString& string);

    void setMultiplyMatchedTemplate(DOMString selectString,
      DOMString& modeString, DOMString& priorityString, Element& templateElement);
    MatchedTemplate getMatchedTemplate(DOMString& matchString,
      DOMString& modeString, DOMString& priorityString, Element& templateElement);
    DOMString getCompoundTemplateName(DOMString& pathString, DOMString& modeString);
  };
}


#endif
