/***************************************************************************
 *   Copyright (C) 2006 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDTEXCONVERTER_H
#define LIMPIDTEXCONVERTER_H

#include "documentprocessor.h"
#include <deque>
#include "domutils.h"
#include "getter.h"
#include "latexconfiguration.h"
#include "nodestream.h"

namespace limpid {

/**
  @author John Redmond <john_redmond@optusnet.com.au>
*/
class TexConverter : public DocumentProcessor, public DOMUtils {

public:
  TexConverter(const DOMString& specPath, const DOMString& destPath);
  Document& processDocument(Document& sourceDocument);
  TexConverter& copy() { return *(new TexConverter(*this)); }
  void update(const DOMString& fileName);

  typedef std::map<DOMString, DOMString> StringMap;
  typedef std::vector<DOMString> StringList;
  typedef std::deque<DOMString> StringStack;
  typedef std::deque<bool> BooleanStack;

private:
  bool verbose_;
  Writer output_;
  LatexConfiguration latexConfiguration_;

  Getter configGetter_;
  Document configDocument_, resultDocument_;
  DOMString specPath_, destPath_;
  DOMString documentString_, tocString_, sectionsString_, documentStartString_, documentBodyString_, tailString_;

  StringMap documentMap_, packageMap_, tocHeadersMap_, bodyHeadersMap_, styleMap_, preStringMap_, postStringMap_;
  StringList documentList_, packageList_;
  StringStack endStack_;
  BooleanStack sectionStack_;

  void traverse(Node& sourceRoot, CharacterWriter& writer, bool includeRoot);
  void writeOutput(Document& sourceDocument);
  void writeText(Node& node, CharacterWriter& writer);
  void writeList(Node& node, const DOMString& nodeName, CharacterWriter& writer);
  void writeDefinitionList(Node& node, const DOMString& nodeName, CharacterWriter& writer);
  void writeTable(Node& node, const DOMString& nodeName, CharacterWriter& writer);
  void writeRows(NodeList& rowList, bool highlighting, CharacterWriter& writer);
  void writeRow(Node& rowNode, bool highlighting, CharacterWriter& writer);
  int getMaxLength(NodeList& rowList);
  void writeLink(Node& node, const DOMString& nodeName, CharacterWriter& writer);
  void writeLiteral(Node& node, const DOMString& nodeName, CharacterWriter& writer);
  void writeEnd(Node& node, CharacterWriter& writer);

  bool isBlockNode(const Node& node) { return latexConfiguration_.isBlock(node.getNodeName()) || !hasText(node); }
  bool isTextNode(const Node& node) { return latexConfiguration_.isText(node.getNodeName()); }
  bool isUndefined(const DOMString& pathString) { return latexConfiguration_.isUndefined(pathString); }
  bool hasBlockBefore(const Node& node) { return isBlockNode(node.getPreviousSibling()); }
  bool hasBlockAfter(const Node& node) { return isBlockNode(node.getNextSibling()); }
  bool hasBlockNeighbor(const Node& node) { return hasBlockBefore(node) || hasBlockAfter(node); }

  DOMString getContentString(Node& node);
  DOMString getPrestring(const DOMString& key) { return latexConfiguration_.getPrestring(key); }
  DOMString getPoststring(const DOMString& key) { return latexConfiguration_.getPoststring(key); }
  void openBlockStyle(Node& node, NodeStream& stream, CharacterWriter& writer);
  void closeBlockStyle(Node& node, CharacterWriter& writer);
  void openTagStyle(Node& node, NodeStream& stream, CharacterWriter& writer);
  void closeTagStyle(Node& node, CharacterWriter& writer);

  bool getSectionStatus();
  DOMString getBackString();
  DOMString getLabel(Node& node);
};

}

#endif
