/***************************************************************************
 *   Copyright (C) 2006 by root   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDTEXSTYLER_H
#define LIMPIDTEXSTYLER_H

#include "documentprocessor.h"
#include "domutils.h"
#include "documentsummary.h"
#include "element.h"
#include "domstring.h"
#include "nodelist.h"
#include <vector>
#include <map>
#include <algorithm>

namespace limpid {

/**
    @author root <john_redmond@optusnet.com.au>
*/

class TexStyler : public DocumentProcessor, public DOMUtils {
public:
    TexStyler() {}
    Document& processDocument(Document&);
    TexStyler& copy() { return *(new TexStyler(*this)); }

private:
  typedef std::map<DOMString, DOMString> StringMap;
  typedef std::vector<DOMString> StringList;

  DOMString destPath_, contentCols_, bodyCols_;
  DOMString classString_, sectionsString_, documentStartString_, documentBodyString_, definesString_, tocString_, columnsStartString_, columnsEndString_;
  DOMString titleStyle_, sectionStyle_, subsectionStyle_, subsubsectionStyle_;
  DOMString titlePath_, sectionPath_, subsectionPath_, subsubsectionPath_;
  bool tocLeft_, tocCenter_, tocRight_, bodyLeft_, bodyCenter_, bodyRight_, fancyPage_, haveTOC_;

  StringMap definitionsMap_;
  StringMap pathList_;

  DocumentSummary documentSummary_;
  Document resultDocument_;

  void setDefaultStyles();
  void setDefaultDefinitions();
  void setHeaders(Document& sourceDocument);
  void setHeaders(const DOMString& whereString, Node& whereNode);
  void setNewDefinitions(Document& sourceDocument);
  void setDefinition(const DOMString& key, const DOMString& details);
  void setTitles(Document& sourceDocument);
  void addPackages(DOMString& classString, Node& latexNode);
  void setTopStrings(Document& sourceDocument);
  void buildDocument(Document& sourceDocument);
  Element getDocumentElement();
  void addTopElements(Element& parentElement);
  void addTopElement(const DOMString& nameString, const DOMString& contentString, Element& parentElement);
  void addRootElement(Element& parentElement);
  void addDefinedTemplate(Node& node, Element& documentElement);
  void addTemplateElement(const DOMString& matchString, const DOMString& preString, const DOMString& postString, Element& parentElement);
  void addDefaultElement(Element& parentElement);
  void addTextElement(Element& parentElement);
  void addXSLText(Element& parentElement, const DOMString& value);
  void addSection(Element& element, Element& documentElement, DOMString& pathString, DOMString& type);
  void addCallElement(const DOMString& matchString, Element& parentElement);
  };
}

#endif
