/***************************************************************************
                          text.cpp  -  description
                             -------------------
    begin                : Wed Sep 10 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "document.h"
#include "text.h"
#include "writer.h"

namespace limpid {

Text::Text(): CharacterData(Text::TEXT_NODE) {}

Text::Text(const DOMString& data):
      CharacterData(Node::TEXT_NODE, data) {}

Text::Text(NodeType nodeType, const DOMString& data):
      CharacterData(nodeType, data) {
}

Text::Text(const Text& refNode): CharacterData(refNode) {}

Text& Text::operator=(const Text& refNode) {
  replaceContent(refNode);
  return *this;
}

DOMString Text::getNodeValue() const {
  DOMString rawValue = getData();
  if (rawValue.empty()) return rawValue;

  rawValue.resolveEntities();
  return getOwnerDocument().resolveEntities(rawValue);
}

Text Text::splitText(size_t offset) {
  Text newText;
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);

  if (contentPtr->data_) {
    DOMString d = contentPtr->data_;

    size_t size = d.size();
    if (offset < d.size()) {
      DOMString s = d.substring(offset, size);
      newText = s;

      d.remove(offset, size);
      delete [] contentPtr->data_;
      contentPtr->data_ = d.copyChars();
    }
  }

  Node& nextSibling = getNextSibling();
  Node& parentNode = getParentNode();

  if (parentNode.isValid()) {
    if (nextSibling.isValid())
      parentNode.insertBefore(newText, nextSibling);
    else parentNode.appendChild(newText);
  }

  return newText;
}

Node& Text::cloneNode(bool) const {
  CharacterContent *contentPtr = static_cast<CharacterContent *>(nodeContent_);
  DOMString data = contentPtr->data_;
  Text *clone = new Text(data);

  clone->nodeContent_->documentContent_ = contentPtr->documentContent_;
  getOwnerDocument().cacheNode(*clone);
  return *clone;
}

DOMString Text::getDisplayString() const {
  return getData();
}

}
