/***************************************************************************
                          traverser.cpp  -  description
                             -------------------
    begin                : Fri Apr 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "traverser.h"
#include "writer.h"

namespace limpid {

Traverser::Traverser(Processor& processor, bool processEnds, bool processPrePost):
    processor_(processor), processEnds_(processEnds),
    processPrePost_(processPrePost) {
}

Node& Traverser::process(Node& root) {
  if (processPrePost_) processor_.start();
  Node& returnNode = processNode(root);
  if (processPrePost_) processor_.finish();

  return returnNode;
}

Node& Traverser::processNode(Node& root) {
  processor_.process(root);

  NodeList& childNodes = root.getChildNodes();
  if (!childNodes.isValid()) return root;

  int childCount = childNodes.getLength();
  if (childCount == 0) return root;

  for (int i = 0; i < childCount; i++) {
    Node& node = childNodes.item(i);
    processNode(node);
  }

  if (processEnds_ && root.getNodeType() == Node::ELEMENT_NODE) {
    endNode_.update(root);
    processor_.process(endNode_);
  }

  return root;
}

NodeList& Traverser::process(NodeList& nodes) {
  size_t length = nodes.getLength();
  for (size_t i = 1; i <= length; i++) process(nodes.item(i));

  return nodes;
}

}
