/***************************************************************************
                          utf16converter.cpp  -  description
                             -------------------
    begin                : Sat Jun 19 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "utf16converter.h"

namespace limpid {

UTF16Converter::UTF16Converter (bool bigEndian):
      CharacterConverter(), isBigEndian(bigEndian) {
}

int UTF16Converter::get(istream *input) {
  if (input->bad()) return -1;

  unsigned long word1 = getWord(input);
  if (word1 == 0xFFFF) return -1; // input error
  else if (word1 < 0xD800 || word1 > 0xDFFF) return word1;

  unsigned long word2 = getWord(input);
  if (word2 == 0xFFFF) return -1; // input error
  else if (word2 < 0xDC00 || word2 > 0xDFFF) return -1; // error

  unsigned long result = word1 & 0x3FF;
  result = result << 10;
  result |= (word2 & 0x3FF);

  return (result + 0x10000) & 0xFFFFF;
}

inline unsigned long UTF16Converter::getWord(istream *input) {
  if (input->bad()) return 0xFFFF;

  char c; // watch this: get a signed char then assign to unsigned chars
  unsigned char first, second;
  unsigned long wchar;

  if (!input->get(c)) return 0xFFFF;
  else first = c;
  if (!input->get(c)) return 0xFFFF;
  else second = c;

  if (isBigEndian) {
    wchar = first;
    return (wchar << 8) | second;
  }
  else {
    wchar = second;
    return (second << 8) | first;
  }
}


int UTF16Converter::put(int wchar, ostream *output) {
  if (wchar < 0x10000 && (wchar < 0xD800 || wchar > 0xDFFF)) {
    putWord(wchar, output);
    return 1;
  }
  else {
    wchar -= 0x10000;
    unsigned int word1 = 0xD800, word2 = 0xDC00;

    word1 |= ((wchar >> 10) & 0x3FF);
    word2 |= (wchar & 0x3FF);

    putWord(word1, output);
    putWord(word2, output);

    return 2;
  }
}

void UTF16Converter::putWord(unsigned int schar, ostream *output) {
  if (isBigEndian) {
    output->put(schar >> 8);
    output->put(schar & 0xFF);
  }
  else {
    output->put(schar & 0xFF);
    output->put(schar >> 8);
  }
}

}

