/***************************************************************************
                          utf8converter.cpp  -  description
                             -------------------
    begin                : Fri Jun 25 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "utf8converter.h"

#include "writer.h"

namespace limpid {
  using std::istream;

UTF8Converter::UTF8Converter() {
}

int UTF8Converter::get(std::basic_istream<char> *input) {
  if (input->bad()) return -1;

  char c;
  if (!input->get(c)) return -1;
  unsigned char uc = c;

  if (uc < 0x80) {
    return c;
  }
  if (uc < 0xE0) return getWCharFrom2(c, input);
  if (uc < 0xF0) return getWCharFrom3(c, input);
  if (uc < 0xF8) return getWCharFrom4(c, input);
  if (uc < 0xFC) return getWCharFrom5(c, input);
  if (uc < 0xFE) return getWCharFrom6(c, input);

  return -1;
}

inline wchar_t UTF8Converter::getWCharFrom2(unsigned char first,
      std::basic_istream<char> *input) {
  char c;
  wchar_t result = 0;
  result |= ((first & 0x1F) << 6);
  if (!input->get(c)) return -1;

  result |= (c & 0x7F);
  return result;
}

inline wchar_t UTF8Converter::getWCharFrom3(unsigned char first,
      istream *input) {
  char c;
  wchar_t result = 0;
  result |= ((first & 0x0F) << 12);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 6);

  if (!input->get(c)) return -1;
  result |= (c & 0x3F);

  return result;
}

inline wchar_t UTF8Converter::getWCharFrom4(unsigned char first,
      istream *input) {
  char c;
  wchar_t result = 0;
  result |= ((first & 0x07) << 18);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 12);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 6);

  if (!input->get(c)) return -1;
  result |= (c & 0x3F);

  return result;
}

inline wchar_t UTF8Converter::getWCharFrom5(unsigned char first,
      std::basic_istream<char> *input) {
  char c;
  wchar_t result = 0;
  result |= ((first & 0x03) << 24);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 18);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 12);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 6);

  if (!input->get(c)) return -1;
  result |= (c & 0x3F);


   return result;
}

inline wchar_t UTF8Converter::getWCharFrom6(unsigned char first,
      std::basic_istream<char> *input) {
  char c;
  wchar_t result = 0;
  result |= ((first & 0x01) << 30);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 24);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 18);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 12);

  if (!input->get(c)) return -1;
  result |= ((c & 0x3F) << 6);

  if (!input->get(c)) return -1;
  result |= (c& 0x3F);

  return result;
}

int UTF8Converter::put(int wchar, std::basic_ostream<char> *output) {
  if (wchar < 0x80) {
    output->put(wchar);
    return 1;
  }
  if (wchar < 0x800) {
    output->put(0xC0 | (wchar >> 6 & 0x1F));
    output->put(0x80 | (wchar & 0x3F));
    return 2;
  }
  if (wchar < 0x10000) {
    output->put(0xE0 | ((wchar >> 12) & 0x0F));
    output->put(0x80 | ((wchar >> 6) & 0x3F));
    output->put(0x80 | (wchar & 0x3f));
    return 3;
  }
  if (wchar < 0x200000) {
    output->put(0xF0 | ((wchar >> 18) & 0x07));
    output->put(0x80 | ((wchar >> 12) & 0x3F));
    output->put(0x80 | ((wchar >> 6) & 0x3F));
    output->put(0x80 | (wchar & 0x3f));
    return 4;
  }
  if (wchar < 0x4000000) {
    output->put(0xF8 | ((wchar >> 24) & 0x03));
    output->put(0x80 | ((wchar >> 18) & 0x3F));
    output->put(0x80 | ((wchar >> 12) & 0x3F));
    output->put(0x80 | ((wchar >> 6) & 0x3F));
    output->put(0x80 | (wchar & 0x3f));
    return 5;
  }
  if ((unsigned int)wchar < 0x80000000) {
    output->put(0xFC | ((wchar >> 30) & 0x01));
    output->put(0x80 | ((wchar >> 24) & 0x3F));
    output->put(0x80 | ((wchar >> 18) & 0x3F));
    output->put(0x80 | ((wchar >> 12) & 0x3F));
    output->put(0x80 | ((wchar >> 6) & 0x3F));
    output->put(0x80 | (wchar & 0x3f));
    return 6;
  }
  output->put('?');
  return -1;
}

}
