/***************************************************************************
                          validationprocessor.cpp  -  description
                             -------------------
    begin                : Sat Nov 22 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "validationprocessor.h"
#include "text.h"
#include "namednodemap.h"
#include "writer.h"
#include "domwriter.h"

namespace limpid {

ValidationProcessor::ValidationProcessor(): constrained_(false) {}

void ValidationProcessor::applySchema(Document& schemaDocument) {
  contentValidator_.setup(schemaDocument);
  attributeValidator_.setup(schemaDocument);
  constrained_ = true;
}

void ValidationProcessor::setWriter(Writer& writer) {
  writer_ = writer;
  characterValidator_.setWriter(writer);
  contentValidator_.setWriter(writer);
  attributeValidator_.setWriter(writer);
}

Node& ValidationProcessor::process(Node& node) {
  int type = node.getNodeType();
  if (type == Node::ELEMENT_NODE)
    validateElement(node);
  else if (type == Node::TEXT_NODE)
    validateText(node);
  return node;
}

void ValidationProcessor::validateElement(Node& node) {
  Element& element = static_cast<Element &>(node);

  if (constrained_ && simple_) {
    contentValidator_.validateChildren(element);
    return;
  }

  if (constrained_) { // complete validation if not simple_
    contentValidator_.validate(element);
    attributeValidator_.validateNames(element, characterValidator_);
    attributeValidator_.validateValues(element, characterValidator_);
    attributeValidator_.validateSpecified(element);
    attributeValidator_.validateComplete(element);
    return;
  }

  // do the best that we can
  DOMString name = element.getLocalName();
  size_t lineNo = element.getPosition();

  if (!name.empty()) { // it is a qualified name
    DOMString prefix = element.getPrefix();
    characterValidator_.checkValidName(prefix, lineNo);
    characterValidator_.checkValidElementName(name, lineNo);
  }
  else { // an unqualified name
    name = element.getNodeName();
    characterValidator_.checkValidElementName(name, lineNo);
  }

  attributeValidator_.validateNames(element, characterValidator_);
  attributeValidator_.validateValues(element, characterValidator_);
}

void ValidationProcessor::validateText(Node& node) {
  Text &txt = static_cast<Text &>(node);
  DOMString value = txt.getData();
  size_t lineNo = txt.getPosition();
  characterValidator_.checkValidText(value, lineNo);
}

size_t ValidationProcessor::getSize() {
  size_t size = sizeof(*this);
  return size;
}

}
