/***************************************************************************
                          value.h  -  description
                             -------------------
    begin                : Wed Feb 2 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VALUE_H
#define VALUE_H

#include "domstring.h"

/**
  *@author John Redmond
  */

// note that Holder is a simple receptacle for a value, which is managed
// automatically by the system.
namespace limpid {
  template<class T>
  struct VHolder {
    public:
      VHolder<T>() { refCount_ = 1; }
      VHolder<T>(const T& value) { value_ = value; refCount_ = 1; }

      T value_;
      unsigned int refCount_;
  };

  template<class T>
  class Value {
  public:
    Value(): holder_(new VHolder<T>) { }
    Value(const T& value): holder_(new VHolder<T>(value)) {}

    Value(const Value<T>& refValue) { copyContent(refValue); }
    ~Value() { deleteContent(); }

    Value<T>& operator=(const Value<T>& refValue) {
      if (this != &refValue) {
        deleteContent();
        copyContent(refValue);
      }
      return *this;
    }
    Value<T>& operator=(const T& refValue) {
      holder_->value_ = refValue;
      return *this;
    }

    T& operator()() const { return holder_->value_; }
    T& operator*() const { return holder_->value_; }
    T* operator->() const { return &(holder_->value_); }

  private:
    VHolder<T> *holder_;

    void copyContent(const Value<T>& refValue) {
      holder_ = refValue.holder_;
      ++holder_->refCount_;
    }

    void deleteContent() {
      if (!--holder_->refCount_)
        delete holder_;
    }
  };
}

#endif
