/***************************************************************************
                          valuetemplate.cpp  -  description
                             -------------------
    begin                : Fri Oct 1 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "valuetemplate.h"
#include "xpathfactory.h"
#include "xpathstring.h"
#include "cdatasection.h"
#include "text.h"
#include "document.h"
#include "nodeset.h"

namespace limpid {

ValueTemplate::ValueTemplate(const DOMString& matchString, bool disableOutputEscaping):
    Template(matchString), expr_(XPathFactory::getExpr(matchString)),
    disableOutputEscaping_(disableOutputEscaping) {
}

bool ValueTemplate::process(XSLContext& parentContext) {
  if (!expr_.isNodeSet()) {
    XPathString valueString = expr_.getString(parentContext);
    if (disableOutputEscaping_ || !parentContext.getEscapingText())
      valueString.resolveEntities();

    parentContext.getWriter() << valueString.escapeBra();

    return true;
  }

  NodeSet matchedSet = expr_.getNodeSet(parentContext);
  if (!matchedSet.getLength()) return true; // nothing to work with

  Node& firstNode = matchedSet.getItem(0);
  XPathString valueString = firstNode.getValue();

  bool isAttributeNode = firstNode.getNodeType() == Node::ATTRIBUTE_NODE;
  if (isAttributeNode) {
    if (disableOutputEscaping_ || !parentContext.getEscapingAttributes())
      valueString.resolveEntities();
  }

  else if (disableOutputEscaping_ || !parentContext.getEscapingText())
    valueString.resolveEntities();

  if (!isAttributeNode && resources_.hasCDATASection(parentContext.getNode()))
    parentContext.getWriter() << CDATASection(valueString).getDisplayString();

  else
    parentContext.getWriter() << valueString.escapeBra();

  return true;
}

bool ValueTemplate::processNode(XSLContext& parentContext, Node& node) {
  if (!expr_.isNodeSet()) {
    XPathString valueString = expr_.getString(parentContext);
    if (!parentContext.getEscapingText())
      valueString.resolveEntities();

    Text text(valueString.escapeBra());
    node.appendChild(text);

    return true;
  }

  NodeSet matchedSet = expr_.getNodeSet(parentContext);
  if (!matchedSet.getLength()) return true; // nothing to work with

  Node& firstNode = matchedSet.getItem(0);
  XPathString valueString = firstNode.getValue();

  bool isAttributeNode = firstNode.getNodeType() == Node::ATTRIBUTE_NODE;
  if (isAttributeNode && !parentContext.getEscapingAttributes())
    valueString.resolveEntities();
  else if (!parentContext.getEscapingText())
    valueString.resolveEntities();

  else if (disableOutputEscaping_ || !parentContext.getEscapingText())
    valueString.resolveEntities();

  if (!isAttributeNode && resources_.hasCDATASection(parentContext.getNode())) {
    CDATASection cds(valueString);
    node.appendChild(cds);
  }
  else {
    Text text(valueString.escapeBra());
    node.appendChild(text);  // the node may be an attribute
  }

  return true;
}

}
