/***************************************************************************
                          writer.cpp  -  description
                             -------------------
    begin                : Sat Jan 24 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *      NOTE THAT THIS CODE IS SPECIALIZED FOR WIN32                       *
 ***************************************************************************/

#include <sstream>
#include <iostream>
#include "outputstream.h"
#include "writer.h"
#include "utf16converter.h"
#include "utf8converter.h"
#include "domstring.h"

using namespace std;

namespace limpid {

Writer::Writer(bool toConsole):  path_(toConsole ? "Console" : "Errors"),
      outputStream_(new OutputStream(toConsole)),
      output_(0), converter_(0), toFile_(false), toErrors_(!toConsole), started_(false) {
  #ifndef _WINDOWS_
    setEncoding(UTF8);
  #endif
  output_ = outputStream_->output_;
}

Writer::Writer(const URL& url, Encoding encoding):
    path_(url.getPath()), outputStream_(new OutputStream(url)),
    output_(0), converter_(0), toFile_(true), toErrors_(false),
    started_(false) {
  setEncoding(encoding);
  output_ = outputStream_->output_;
  doWriteBOM(encoding);
}

#ifdef _WINDOWS_
int Writer::putChar(int c) {
  if (c == 13) return c; // refuse to output a CR

  if (toFile_)
    return converter_->put(c, output_);

  if (c == 10) {
    if (!toErrors_) ConsoleStream << Char(13) << Char(c);
    else ErrorStream << Char(13) << Char(c);
    return c;
  }

  if (!toErrors_) ConsoleStream << Char(c);
  else ErrorStream << Char(c);

  return c;
}

#else
// all is simpler in Linux
int Writer::putChar(int c) {
  if (c == 13) return c;

  return converter_->put(c, output_);
}
#endif


void Writer::setEncoding(Encoding enc) {
  if (enc == UTF16LE)
    converter_ = new UTF16Converter(false); // little endian
  else if (enc == UTF16 || enc == UTF16BE)
    converter_ = new UTF16Converter(true); // big-endian
  else // assume UTF8, even if there is another encoding attempted
    converter_ = new UTF8Converter;
}

void Writer::setEncoding(const DOMString& enc) {
  DOMString encodingString = enc;
  encodingString.toUpperCase();

  if (encodingString == "UTF-16LE")
    converter_ = new UTF16Converter(false); // little endian
  else if (encodingString == "UTF-16" || enc == "UTF-16BE")
    converter_ = new UTF16Converter(true); // big-endian
  else // assume UTF8, even if there is another encoding attempted
    converter_ = new UTF8Converter;
}

void Writer::doWriteBOM(Encoding encoding) {
  if (encoding == UTF16 || encoding == UTF16BE) {
    output_->put(0xFE);
    output_->put(0xFF);
  }
  else if (encoding == UTF16LE) {
    output_->put(0xFF);
    output_->put(0xFE);
  }
}

void Writer::flushScreen() {
  if (!toErrors_) ConsoleStream << wrt::endl;
  else ErrorStream << wrt::endl;
}

  Writer Writer::Console(true);
  Writer Writer::Errors(false);
  const char* Writer::endl = EndString;

}
