  /***************************************************************************
                          writer.h  -  description
                             -------------------
    begin                : Sat Jan 24 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef WRITER_H
#define WRITER_H

#include <string>
#include <iostream>
#include <fstream>
#include "node.h"
#include "domstring.h"
#include "characterwriter.h"
#include "outputstream.h"
#include "characterconverter.h"
#include "pointer.h"
#include "definitions.h"


/**
  *@author John Redmond
  */

namespace limpid {
  class Writer : public CharacterWriter {
  public:
    explicit Writer(bool toConsole = true);
    Writer(const URL& url, Encoding encoding = UTF8);

    static Writer Console, Errors;

    static const char* endl;

    void setEncoding(Encoding encoding);
    void setEncoding(const DOMString& encoding);
    int putChar(int c);
    URL getURL() { return outputStream_->url_; }
    void flush() { if (outputStream_) outputStream_->flush(); }
    void close() { if (outputStream_) outputStream_->close(); }
    bool isValid() { return !toFile_ || *output_; }
    Writer& copy() { return *(new Writer(*this)); }
    DOMString getPath() { return path_; }

  protected:
    void doWriteBOM(Encoding encoding);

  private:
    DOMString path_;
    Pointer<OutputStream> outputStream_;
    std::basic_ostream<char> *output_;
    Pointer<CharacterConverter> converter_;
    bool toFile_, toErrors_, started_;

    void flushScreen();
  };

  typedef Writer wrt;
}
#endif
