/***************************************************************************
                          xmldeclaration.cpp  -  description
                             -------------------
    begin                : Fri Mar 19 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xmldeclaration.h"
#include "document.h"
#include "xmldeclarationcontent.h"
#include "domstring.h"

namespace limpid {

XMLDeclaration::XMLDeclaration(const DOMString& version,
    const DOMString& encoding, const DOMString& standalone):
    ProcessingInstruction("", "") {
  try {
    nodeContent_ = new XMLDeclarationContent(version, encoding, standalone);
    nodeContent_->nodeType_ = XML_DECLARATION_NODE;
  }
  catch (std::bad_alloc) {
    delete nodeContent_;
    throw;
  }
}

XMLDeclaration::XMLDeclaration(const XMLDeclaration& refNode):
  ProcessingInstruction(refNode) {
}

XMLDeclaration& XMLDeclaration::operator=(const XMLDeclaration& refNode) {
  replaceContent(refNode);
  return *this;
}

DOMString XMLDeclaration::getData() const {
  XMLDeclarationContent *contentPtr =
    static_cast<XMLDeclarationContent *>(nodeContent_);
  DOMString data;
  if (contentPtr->version_) {
    data += " version=\"";
    data += contentPtr->version_;
    data += '\"';
  }
  if (contentPtr->encoding_) {
    data += " encoding=\"";
    data += contentPtr->encoding_;
    data += '\"';
  }
  if (contentPtr->standalone_) {
    data += " standalone=\"";
    data += contentPtr->standalone_;
    data += '\"';
  }

  return data;
}

DOMString XMLDeclaration::getVersion() const {
  XMLDeclarationContent *contentPtr = static_cast<XMLDeclarationContent *>(nodeContent_);
  return contentPtr->version_;
}

DOMString XMLDeclaration::getEncoding() const {
  XMLDeclarationContent *contentPtr = static_cast<XMLDeclarationContent *>(nodeContent_);
  return contentPtr->encoding_;
}

DOMString XMLDeclaration::getStandAlone() const {
  XMLDeclarationContent *contentPtr = static_cast<XMLDeclarationContent *>(nodeContent_);
  return contentPtr->standalone_;
}

DOMString XMLDeclaration::getDisplayString() const {
  DOMString s("<?xml");
  DOMString data = getData();
  s += data;
  s += "?>";
  return s;
}

Node& XMLDeclaration::cloneNode(bool) const {
  DOMString version = getVersion();
  DOMString encoding = getEncoding();
  DOMString alone = getStandAlone();
  XMLDeclaration *copyNode = new XMLDeclaration(version, encoding, alone);

  copyNode->nodeContent_->documentContent_ = nodeContent_->documentContent_;
  getOwnerDocument().cacheNode(*copyNode);

  return *copyNode;
}

XMLDeclaration XMLDeclaration::failDeclaration = XMLDeclaration("", "", "");

}
