/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIMPIDXMLMAPPER_H
#define LIMPIDXMLMAPPER_H

#include <map>
#include <set>
#include "documentprocessor.h"
#include "element.h"
#include "pointer.h"
#include "domutils.h"

namespace limpid {

/**
	@author John Redmond <john_redmond@optusnet.com.au>
*/
class XMLMapper : public DocumentProcessor, public DOMUtils {
public:
  XMLMapper();
  Document& processDocument(Document&);
  XMLMapper& copy() { return *(new XMLMapper(*this)); }

  typedef std::map<DOMString, DOMString> StringMap;
  typedef std::set<DOMString> StringSet;

  Document document_;
  Pointer<StringMap> stringMap_;
  Pointer<StringSet> stringSet_;

  void setDefaultMappings();
  void setDefaultGroup(const DOMString& keyStart, const DOMString& resultStart);
  void setDivs();
  void setInvariants(const DOMString& keyStart, const DOMString& result);
  void setMapping(const DOMString& key, const DOMString& value);
  Element getDocumentElement();
  void setTags(Document& document);
  void appendTemplates(Element& sourceElement, Element& documentElement);
  void appendTemplate(const DOMString& pathName, const DOMString& tagName, Element& parentElement);
  void appendTextTemplate(Element& parentElement);
  void addXSLText(Element& parentElement, const DOMString& value);
};

}

#endif
