/***************************************************************************
                          xmlparser.h  -  description
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef XMLPARSER_H
#define XMLPARSER_H

#include <istream>
#include <set>
#include <deque>
#include "parser.h"
#include "reader.h"
#include "tokenstream.h"
#include "iobase.h"
#include "pointer.h"
#include "nodestack.h"

#include "writer.h"

namespace limpid {
  class XMLParser: public Parser  {
  public:
    XMLParser(const DOMString& fileName, bool isHTML = false);
    XMLParser(CharacterReader& characterStream, bool isHTML = false);

    Node& nextNode();
    void setVerbose(bool flag) { tis_->setVerbose(flag); }
    void setProcessingXSLT(bool flag) { processingXSLT_ = flag; }
    void setWritingHTML(bool isHTML) { isHTML_ = isHTML; }

    void setSource(const DOMString& sourceName);
    XMLParser& copy() { return *(new XMLParser(*this)); }

  protected:
    void setNode(Node* node);

  private:
    typedef std::set<DOMString> StringSet;

    Pointer<CharacterReader> reader_;
    Pointer<TokenStream> tis_;
    Pointer<StringSet> emptySet_;
    Node* node_;
    bool processingXSLT_, trimText_, normalizeText_, isHTML_, hasContent_;
    char gtChar_, ltChar_;
    DOMString delimiterString_;

    void setConstants();
    void buildTag();
    void completeStartTag(DOMString& s);
    void completeEndTag();
    void buildNamedTag(DOMString& s);
    void addAttributes(Element& element, const DOMString& name);
    void buildTextNode(DOMString& s);
    void buildPI();
    void buildXMLDeclaration();
    void buildComment(DOMString& s);
    void buildCDATASection();
    void buildDoctype();
    void buildSpecial();
    void configureNode(int type, const DOMString& name);
    DOMString getUnquotedString();
    void reportError(int type, DOMString& desc);
    void alignOn(const DOMString& s) { tis_->alignOn(s);}
    void swallow(const DOMString& s) { alignOn(s); tis_->bump(); }
    DOMString getDelimitedString(const DOMString& str) {
      return tis_->getDelimitedString(str); }
    void setEmptySet();
    bool isEmpty(const DOMString& elementName) {
      return isHTML_ && emptySet_->find(elementName) != emptySet_->end(); }
  };
}
#endif
