/***************************************************************************
                          xmlwriter.h  -  description
                             -------------------
    begin                : Tue Oct 21 2003
    copyright            : (C) 2003 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef XMLWRITER_H
#define XMLWRITER_H

#include <iostream>
#include <set>
#include "pointer.h"
#include "node.h"
#include "element.h"
#include "dumper.h"
#include "writer.h"
#include "outputformat.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class XMLWriter : public Dumper  {
  public:
  	XMLWriter(bool isHTML = false);
    XMLWriter(const URL& url, bool isHTML = false);
    XMLWriter(CharacterWriter& output, bool isHTML = false);

    void update(const DOMString& destString);
    void setFormat(const DOMString& nl, const DOMString& i, const DOMString& ai);
    void setStartAndFinish(const DOMString& start, const DOMString& finish);
    void setFormatting(bool flag) { format_->isFormatting = flag; }
    void setEncoding(DOMString& encoding);
    void setWritingHTML(bool flag) { isHTML_ = flag; }
    void nest() { format_->isStarted = true; format_->nest(); }
    void deNest() { format_->deNest(); }
    Node& process(Node& node);
    void flush() { if (writer_) writer_->flush(); }
    XMLWriter& copy() { return *(new XMLWriter(*this)); }

    // Processor interfaces
    void start();
    void finish();

  protected:
    typedef std::set<DOMString> StringSet;

    Pointer<StringSet> emptySet_;
    Pointer<CharacterWriter> writer_;
    Pointer<OutputFormat> format_;
    bool isHTML_;

    Node& processDocumentStart(Node& node);
    Node& processXMLDeclaration(Node& node);
    Node& processDoctype(Node& node);
    Node& processStart(Node& node);
    Node& processEnd(Node& node);
    Node& processText(Node& node);
    Node& processSimple(Node& node);
    void setDefaults(bool isHTML);
    bool containsText(Node& node);
    void writeFormat();
    void setEmptySet();
    bool isHTMLNode(Node& node);
    void setProcessingHTML(bool isHTML) { isHTML_ &= isHTML; }
  };
}
#endif
