/***************************************************************************
                          xpathcorrector.cpp  -  description
                             -------------------
    begin                : Tue May 17 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xpathcorrector.h"
#include "document.h"
#include "element.h"
#include "nodelist.h"
#include "nodestream.h"
#include "liststream.h"
#include "writer.h"

namespace limpid {

XPathCorrector::XPathCorrector() {}

void XPathCorrector::correct(Document& document) {
  NodeList functionList;

  NodeStream nodeStream(document, false, false);
  while (true) {
    Node& node = nodeStream.nextNode();
    if (!node.isValid()) break;

    if (node.getNodeName() == "FunctionCall" && !isCollectorFunction(node))
      functionList.addNode(node);
  }

  ListStream functionStream(functionList);
  while (true) {
    Node& node = functionStream.nextNode();
    if (!node.isValid()) break;

    Node& parentNode = node.getParentNode();
    if (parentNode.getNodeName() == "PathExpr") {
      Node& grandParentNode = parentNode.getParentNode();
      if (!grandParentNode.isValid()) continue; // this should never happen
      grandParentNode.replaceChild(node, parentNode);
    }
  }
}

bool XPathCorrector::isCollectorFunction(Node& node) {
  Element& element = static_cast<Element &>(node);
  DOMString type = element.getAttribute("type");

  return type == "id" || type == "key" || type == "current";
}

}
