/***************************************************************************
                          xpathfactory.cpp  -  description
                             -------------------
    begin                : Sat Jun 12 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xpathfactory.h"
#include "writer.h"
#include "xpathbuilder.h"
#include "xpathcorrector.h"
#include "xpathorganizer.h"
#include "xpathreverser.h"
#include "domwriter.h"

namespace limpid {

XPathFactory::XPathFactory(){}

Expr XPathFactory::getExpr(const DOMString& pathStr, bool verbose) {
  if (pathStr.empty()) return Expr();

  Document xpathDocument = getDocument(pathStr, verbose);
  return XPathBuilder().buildExpr(xpathDocument, pathStr);
}

Expr XPathFactory::getReverseExpr(const DOMString& pathString, bool verbose) {
  if (pathString.empty()) return Expr();
  Document xpathDocument = XPathParser(pathString).getDocument();
  XPathCorrector().correct(xpathDocument);
  XPathOrganizer().organize(xpathDocument);

  reverseDocument(xpathDocument, verbose);
  return XPathBuilder().buildExpr(xpathDocument, pathString);
}

Document XPathFactory::getDocument(const DOMString& pathString, bool verbose) {
  if (verbose)
    wrt::Console << "XPATH from [" << pathString << "]\n";

  Document xpathDocument = XPathParser(pathString).getDocument();

  if (verbose) {
    DOMWriter outputter;
    wrt::Console << "RAW PATH DOCUMENT\n";
    outputter.write(xpathDocument);
  }


  XPathCorrector corrector;
  corrector.correct(xpathDocument);

  if (verbose) {
    DOMWriter outputter;
    wrt::Console << "CORRECTED PATH DOCUMENT\n";
    outputter.write(xpathDocument);
  }

  XPathOrganizer organizer;
  organizer.organize(xpathDocument);

  if (verbose) {
    DOMWriter outputter;
    wrt::Console << "ORGANIZED PATH DOCUMENT from " << pathString << "\n";
    outputter.write(xpathDocument);
  }

  return xpathDocument;
}

void XPathFactory::reverseDocument(Document& document, bool verbose) {
  XPathReverser reverser;
  reverser.reverse(document);
  if (verbose) {
    wrt::Console << "REVERSED PATH DOCUMENT\n";
    DOMWriter().write(document);
  }
}

};

