/***************************************************************************
                          xpathparser.h  -  description
                             -------------------
    begin                : Wed Apr 28 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef XPATHPARSER_H
#define XPATHPARSER_H


/**
  *@author John Redmond
  */
#include <vector>
#include "node.h"
#include "element.h"
#include "nodelist.h"
#include "document.h"
#include "domstring.h"
#include "xmlwriter.h"
#include "tokenhandler.h"

namespace limpid {
  class XPathParser {
  public:
    XPathParser(const DOMString& s);

    Document getDocument() { return document; }

  private:
    typedef std::vector<DOMString> TokenList;

    Document document;
    TokenHandler tokenHandler;

    DOMString nextToken() {return tokenHandler.nextToken(); }
    DOMString peekToken(int offset = 0) { return tokenHandler.peekToken(offset); }
    void ungetToken() { tokenHandler.ungetToken(); }
    void bumpToken(size_t step = 1) { tokenHandler.bumpToken(step); }
    void markPosition() { tokenHandler.markPosition(); }
    void resumePosition() { tokenHandler.resumePosition(); }
    bool processHyphens(DOMString token);
    void processOperators();

    void setup(const DOMString& sourceString);
    void setTokenList(const DOMString& source);
    DOMString getQName(TokenHandler& handler, DOMString& token);
    DOMString getHyphenatedName(TokenHandler& handler, DOMString& token);

    void populateUnionExpression(Element& unionExpression);
    Element getStep();
    Element getSteps();
    Element getNode();
    Element getLiteral();
    Element getVariable();
    Element getNumber();
    Element getConjunction();
    Element getRelation();
    Element getEquality();
    Element getAdditiveNode();
    Element getBoolean();
    Element getFunctionCall();
    Element getPredicate();
    Element getParenthesis();
    void setErrorReport(Element& parentNode);

    void addFurtherProcessors(Element& parentElement);
    void addPredicates(Element& parentElement);

    DOMString getAxisName();
    void setNodeTypeTest(Element& parentElement);
    void setNameTest(Element& parentElement);

    bool isDelimiter(const DOMString& token);
    bool isNumber(const DOMString& token);
    bool isConjunction(const DOMString& token);
    bool isOperator(const DOMString& token);

    bool numberFollows();
    bool relationFollows();
    bool equalityFollows();
    bool additiveOperatorFollows();
    bool conjunctionFollows();
    bool booleanFollows();
    bool delimiterFollows();
    bool variableFollows();
    bool functionCallFollows();
    bool defaultStepFollows();
    bool abbreviatedStepFollows();
    bool axisFollows();
    bool nodeTypeTestFollows();
    bool nameTestFollows();
    bool nodeTestFollows();
    bool stepFollows();
    bool idFollows();
    bool currentFollows();
    bool nonDefaultStepFollows();
    bool locationPathFollows();
    bool predicateFollows() { return peekToken() == "["; }
  };
}

#endif
