/***************************************************************************
                          xslcontext.cpp  -  description
                             -------------------
    begin                : Sun Sep 26 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xslcontext.h"

namespace limpid {

XSLContext::XSLContext():
    writer_(new Writer), format_(new OutputFormat),
    escapeText_(true), escapeAttributes_(true), elementWritten_(false) {}

void XSLContext::setParent(XSLContext& parentContext) {
  parentContext_ = &parentContext;
  node_ = parentContext.node_;
  current_ = parentContext.current_;
  count_ = parentContext.count_;
  position_ = parentContext.position_;
  writer_ = parentContext.writer_;
  format_ = parentContext.format_;
  domWriter_ = parentContext.domWriter_;
  formatMap_ = parentContext.formatMap_;
  escapeText_ = parentContext.escapeText_;
  escapeAttributes_ = parentContext.escapeAttributes_;
}

int XSLContext::getRank() {
  int rank = 0;
  Context *currentContext = this;

  while (true) {
    if (!currentContext->hasParent())
      break;
    ++rank;
    currentContext = &(currentContext->getParent());
  }
  return rank;
}

void XSLContext::setWriter(CharacterWriter& writer) {
  writer_ = &writer.copy();
}

void XSLContext::setDOMWriter(CharacterWriter& writer, bool isHTML) {
  domWriter_ = new DOMWriter(writer, isHTML);
}

void XSLContext::setEscaping(ConfigurationSet& configurationSet) {
  escapeText_ = configurationSet.escape_text;
  escapeAttributes_ = configurationSet.escape_attributes;
}

}
