/***************************************************************************
                          xslcontext.h  -  description
                             -------------------
    begin                : Sun Sep 26 2004
    copyright            : (C) 2004 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef XSLCONTEXT_H
#define XSLCONTEXT_H

#include <set>
#include "pointer.h"
#include "context.h"
#include "domwriter.h"
#include "nodeset.h"
#include "configurationset.h"
#include "expressionmap.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class NodeSet;
  class ConfigurationSet;
  class Template;
  
  class XSLContext : public Context  {
   public:

    XSLContext();

    CharacterWriter& getWriter() { return *writer_; }
    DOMWriter& getDOMWriter() { return *domWriter_; }
    OutputFormat& getFormat() { return *format_; }
    NodeSet& getCurrentSet() { if (currentSet_) return *currentSet_;
      else return NodeSet::failSet; }

    void setCurrentSet(NodeSet& nodeSet) {
      currentSet_ = &nodeSet; count_ = nodeSet.getLength(); }
    void setParameterMap(ExpressionMap& parameterMap) { parameterMap_ = &parameterMap; }
    void setWriter(CharacterWriter& writer);
    void setDOMWriter(CharacterWriter& writer, bool isHTML = false);
    void setParent(XSLContext& parent);
    void clearParent() { parentContext_ = 0; }
    int getRank();
    void setEscaping(ConfigurationSet& configurationSet);
    bool getEscapingText() { return escapeText_; }
    bool getEscapingAttributes() { return escapeAttributes_; }
    ExpressionMap getParameterMap() {
      if (parameterMap_) return *parameterMap_; else return ExpressionMap(); }

    Context* getParent() { return parentContext_; } // for testing
    Template& getParentTemplate() { return *parentTemplate_; }
  private:
    Pointer<CharacterWriter> writer_;
    Pointer<OutputFormat> format_;
    Pointer<DOMWriter> domWriter_;
    NodeSet *currentSet_; // not managed
    ExpressionMap *parameterMap_; // not managed
    Template *parentTemplate_;
    bool escapeText_, escapeAttributes_, elementWritten_;
  };
}


#endif
