/***************************************************************************
                          xslresources.cpp  -  description
                             -------------------
    begin                : Sat Oct 1 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xslresources.h"
#include "matchedtemplate.h"
#include "xslutility.h"
#include "domstring.h"
#include "element.h"
#include "attr.h"
#include "liststream.h"
#include "matchedtemplate.h"
#include "stringreader.h"
#include "tokenstream.h"

namespace limpid {

XSLResources::XSLResources():
  matchedMap_(new NamedTemplateMap), calledMap_(new NamedTemplateMap),
  importedMatchedMap_(new NamedTemplateMap), importedCalledMap_(new NamedTemplateMap),
  attributeSetMap_(new AttributeSetMap), cdataSectionSet_(new NameSet),
  formatMap_(new DecimalFormatMap), stepSet_(new NameSet){
}

// set a matched template in the map. If there is already an entry with the same
// key, the old matched template is replaced with a later template if the later
// template has equal or higher priority. Otherwise the later template is discarded.
Template& XSLResources::setMappedTemplate(const DOMString& key,
    Template& tmp, Pointer<NamedTemplateMap> map) {
  NamedTemplateMap::iterator p = map->find(key);

  if (p != map->end()) {
    if (p->second->getPriority() <= tmp.getPriority()) {
      map->erase(p);
      Template* newTemplate = &tmp.copy();
      map->insert(make_pair(key, Pointer<Template>(newTemplate)));
      return *newTemplate;
    }
    else return *(p->second);
  }
  else {
    Template* newTemplate = &tmp.copy();
    map->insert(make_pair(key, Pointer<Template>(newTemplate)));
    return *newTemplate;
  }
}

Template& XSLResources::getTemplate(const DOMString& key,
      const Pointer<NamedTemplateMap>& map) const {
  NamedTemplateMap::iterator p = map->find(key);
  if (p != map->end()) return *(p->second);
  else return Template::failTemplate;
}

DOMString XSLResources::getCDATASectionName(size_t index) const {
  NameSet::iterator p = cdataSectionSet_->begin();
  advance(p, index);

  if (p != cdataSectionSet_->end()) return *p;
  return "";   // gone too far
}

void XSLResources::setCDATASectionNames(const DOMString& cdsString) {
  StringReader reader(cdsString);
  TokenStream tis(reader);
  tis.setDelimiters(""); // tokenise the string

  while (true) {
    DOMString token = tis.nextToken();
    if (token.empty()) break;

    cdataSectionSet_->insert(token);
  }
}

// set attribute sets as they are encountered in document order. In the event
// that the attribute set map already has an entry with the same key, it is
// removed in favour of the new set. This has the required result of giving highest
// priority to the last sets.
void XSLResources::setAttributeSet(Element attributeSetElement) {
  DOMString key = attributeSetElement.getAttribute("name");
  if (key.empty())
    wrt::Errors << "Unnamed attribute set: " << attributeSetElement << wrt::endl;

  AttributeSetMap::iterator p = attributeSetMap_->find(key);
  if (p != attributeSetMap_->end())
    attributeSetMap_->erase(p);

  Element* newElement = static_cast<Element *>(&attributeSetElement.cloneNode(true));//new Element(attributeSetElement);

  DOMString namespaceURI = attributeSetElement.getNamespaceURI();
  DOMString prefix = attributeSetElement.getPrefix();
  DOMString attrName = "xmlns";
  if (!prefix.empty()) {
    attrName += ":";
    attrName += prefix;
  }
  newElement->setAttribute(attrName, namespaceURI);

  attributeSetMap_->insert(make_pair(key, Pointer<Element>(newElement)));
}

Element& XSLResources::getAttributeSet(DOMString& name) const {
  AttributeSetMap::iterator p = attributeSetMap_->find(name);
  if (p != attributeSetMap_->end())
    return *(p->second);
  else
    return Element::failElement;
}

}
