/***************************************************************************
                          xslresources.h  -  description
                             -------------------
    begin                : Sat Oct 1 2005
    copyright            : (C) 2005 by John Redmond
    email                : john_redmond@optusnet.com.au
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef XSLRESOURCES_H
#define XSLRESOURCES_H

#include <map>
#include "pointer.h"
#include "element.h"
#include "endnode.h"
#include "domstring.h"
#include "matchedtemplate.h"
#include "decimalformatmap.h"

/**
  *@author John Redmond
  */

namespace limpid {
  class XSLContext;
  class DOMString;
  class Template;
  class Node;

  class XSLResources {

  public:
    XSLResources();

    friend class TemplateStream;

    Template& getCalledTemplate(const DOMString& key, bool isImported) const {
      if (!isImported) return getTemplate(key, calledMap_);
      return getTemplate(key, importedCalledMap_); }
    void setMatchedTemplate(const DOMString& key, Template& tmp, bool isImported) {
      if (!isImported) setMappedTemplate(key, tmp, matchedMap_);
      else setMappedTemplate(key, tmp, importedMatchedMap_); }
    void setCalledTemplate(const DOMString& key, Template& tmp, bool isImported) {
      if (!isImported) setMappedTemplate(key, tmp, calledMap_);
      else setMappedTemplate(key, tmp, importedCalledMap_); }

    void setAttributeSet(Element attributeSetElement);
    Element& getAttributeSet(DOMString& name) const;


    const DecimalFormat& getDecimalFormat(const DOMString& key) const {
      return formatMap_->getFormat(key); }
    void setDecimalFormat(const DOMString& key, const DecimalFormat& format) {
      formatMap_->setFormat(key, format); }
    void setStep(const DOMString& stepName) { stepSet_->insert(stepName); }
    const DecimalFormatMap getDecimalFormatMap() { return *formatMap_; }
    const DOMString& getKey(size_t index) const;

    bool hasCDATASection(Node& node) const { DOMString nodeName = node.getNodeName();
      return cdataSectionSet_->find(nodeName) != cdataSectionSet_->end(); }
    void setCDATASectionNames(const DOMString& cdsString);
    bool isStep(Node& node) {
      if (node.getNodeType() != Node::ELEMENT_NODE)
        return stepSet_->find(node.getNodeName()) != stepSet_->end();
      if (node.getNodeType() != Node::END_NODE) return false;
      DOMString nodeName = static_cast<EndNode &>(node).getNodeName();
        return stepSet_->find(nodeName) != stepSet_->end(); }

    size_t getCDataCount() const { return cdataSectionSet_->size(); }
    DOMString getCDATASectionName(size_t index) const;

  private:
    typedef std::map<DOMString, Pointer<Template> > NamedTemplateMap;
    typedef std::map<DOMString, Pointer<Element> > AttributeSetMap;
    typedef std::set<DOMString> NameSet;

    Pointer<NamedTemplateMap> matchedMap_, calledMap_,
      importedMatchedMap_, importedCalledMap_;
    Pointer<AttributeSetMap>  attributeSetMap_;
    Pointer<NameSet> cdataSectionSet_;
    Pointer<DecimalFormatMap> formatMap_;
    Pointer<NameSet> stepSet_;

    Template& setMappedTemplate(const DOMString& key, Template& tmp,
      Pointer<NamedTemplateMap> map);
    bool findAndProcess(const DOMString& key, XSLContext& parentContext,
      const Pointer<NamedTemplateMap>& map) const;
    bool findAndProcessNode(const DOMString& key, XSLContext& parentContext, Node& node,
      const Pointer<NamedTemplateMap>& map) const;

    Template& getTemplate(const DOMString& key, const Pointer<NamedTemplateMap>& map) const;
  };
}

#endif
