/***************************************************************************
 *   Copyright (C) 2007 by John Redmond   *
 *   john_redmond@optusnet.com.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "helper.h"
#include "domstring.h"
#include "writer.h"
#include "stringreader.h"
#include "dombuilder.h"
#include "domwriter.h"

namespace limpid {

Helper::Helper() {}

void Helper::help() {
  DOMString helpString = "<?xml version='1.0' encoding='UTF-8'?>\n<!DOCTYPE html PUBLIC '-//W3C//DTD_XHTML 1.0 Strict//EN' 'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'>\n<html>\n <head>\n  <title>Lexxia Command Options</title>\n  <style type='text/css'>body { width: 90%; margin-left: 6mm; font: 9pt Arial, sans-serif; font-weight: 200; background: #FFFDDA; }\n\n/* divisions */\n.L1, .L2, .L3 { border: 0pt solid black; padding: 3pt; }\n.L2 { border: 0pt solid blue; }\n.L3 { border: 0pt solid red; }\n\n/* headings */\nh1, h2, h3 { color: #005073; }\nh1 { border: 0.3pt solid blue; padding: 0.3em; margin: 0.75em, 0; margin-bottom: 1em; font-size: 1.5em; background: #EBF5FC; }\nh2 { margin: 0.67em, 0;font-size: 1.4em; }\nh3 { margin: 0.63em, 0; font-size: 1.2em; }\nh1, h2 { text-align: center; }\nh3, h4 { border: 0pt solid red; margin-left: 0em; margin-top: 0.8em; margin-bottom: 0em; }\n\n/* normal text */\np { text-align: justify; margin-top: 0.4em; margin-bottom: 0em; }\n\n/* preformatted text */\npre { width: 90%; border: 0.5pt solid green; margin-left: auto; margin-right: auto; margin-top: 12pt; padding: 6px; font-size: 90%; background: white; }\n\n/* list items */\ndl { width: 90%; border: 0.5pt solid green; margin-left: auto; margin-right: auto; margin-top: 12pt; padding: 6px; font-size: 100%; background: white; }\ndt { margin-top: 0.5em; margin-left: 20px; font: 9pt 'Times New Roman', serif; font-weight: bolder; }\n\n/* tables */\ntable0 { width: 99%; border: 0.5pt solid blue; margin-left: auto; margin-right: auto; background: #D4ECFB;  }\ntable { width: 90%; border: 0.5pt solid blue; margin-left: auto; margin-right: auto; background: white;  }\ntheader { border: 0,5pt solid blue; background: #D4ECFB;}\ntheader, tr { font-size: 8pt; border: 0.5pt solid blue; }\nth, td { border: 0.5pt solid blue; padding-left: 1em; }\nth { text-align: center; background: #D4ECFB; }\n\n/* special */\n#titleblock { margin-left: auto; margin-right: auto; margin-bottom: 2em; }\n#titleblock * { text-align: center; color: #005073; }\n#titleblock a { border: 0.5pt solid blue; padding: 2pt; }\n.linkbox { border: 0pt solid blue; margin-top: 12pt; text-align: center; }\n.linkbox a { display: inline-block; border: 0.5pt solid blue; margin: 0pt; padding: 2pt; }\n\n/* medium-specific */\n@media print {\nbody { margin-left: 20mm; font: 9pt 'Times New Roman', serif; font-weight: 200; }\n.linkbox { display: none; }\n}\n</style>\n </head>\n <body>\n  <div class='L1'>\n   <h1>Lexxia Command Options</h1>\n   <div class='L3'>\n   </div>\n   <div class='L3'>\n    <h3>Getting Help at the Command Line</h3>\n    <dl>\n     <dt>The command is ''lexxia -h'' or ''lexxia -H''.</dt>\n     <dd> The contents of an HTML page (this page!) are written to the console. This can be caught by <em>less</em> or redirected to a file for later reference.</dd>\n    </dl>\n    <h3>Important Notes</h3>\n    <ul>\n     <li>For clarity, the listings below show a space between the command and parameters (if any). This space <em>must not</em> be used in actual command lines.</li>\n     <li>In a command line, it is posssible to specify combinations of Lexxia document processors. </li>\n    </ul>\n    <p>It is important to understand that there are two types of document processors:</p>\n    <dl>\n     <dt>Normal (Type N)</dt>\n     <dd>The processor processes a document passed to it. It typically has the side effect of writing to an output file, and it does <em>not</em> change the document, which is therefore available for subsequent (downstream) processors;</dd>\n     <dt>Mutator (Type M)</dt>\n     <dd>The processor either modifies the document passed to it, or makes a totally different document available to downstream processors. The processor does <em>not</em> produce output. If output is required, it should be followed by an ''-O'' or ''-o'' command (see below).</dd>\n    </dl>\n   </div>\n   <div class='L3'>\n    <h3>\n     <a name='xslt'>Document Input and Output Specifications</a>\n    </h3>\n    <p>Lexxia attempts to connect documents of different formats. The formats are deduced from the file extensions:</p>\n    <dl>\n     <dt>Reading and writing of XML and related (HTML, XHTML, XSL) files.</dt>\n     <dd>These all have essentially the same format, and are parsed by an XML parser. Special care is taken with HTML files,  because of the possibility of unbalanced tags, but no guarantee can be given;</dd>\n     <dt>Reading and writing of TXT files.</dt>\n     <dd>Text files are loaded as rather uninteresting XML documents, while dotted text files typically lead to rich structure.</dd>\n     <dt>Reading of RJ (Record Jar) files.</dt>\n     <dd>Text files in the simple, but useful, RJ format. Writing is not presently supported.</dd>\n     <dt>Reading of output from other programs.</dt>\n     <dd>Lexxia can read from a character stream (using piping). Unless it is given a hint of the format of the stream, such as with ''?txt'' to indicate a text stream, the input parser is set to XML.</dd>\n    </dl>\n    <p>Note that the parameters for input are unique, in that they do not have a dashed prefix.</p>\n   </div>\n   <div class='L3'>\n    <h3>\n     <a name='output'>System Output Specifications (all type N)</a>\n    </h3>\n    <dl>\n     <dt>?xxx</dt>\n     <dd>Hint the format expected in a character stream piped from another application, where ''xxx'' corresponds to the extension expected in a source file name. If this hint is not given, a default xml format is expected;</dd>\n     <dt>-O</dt>\n     <dd>Output in XML format to standard out (N). Useful for piping and linking to external commands. </dd>\n     <dt>-O filePath</dt>\n     <dd>Putter (Outputter) to a specified path. The format of the output is determined by the extension of the file path (xml, html and txt are most common);</dd>\n     <dt>-o</dt>\n     <dd>Output in XML format to standard out (N). Useful for piping and linking to external commands.</dd>\n     <dt>-o fileExtension</dt>\n     <dd>Output to standard out (N) in a format given by the file extension. Useful for piping and linking to external commands.</dd>\n     <dt>-V</dt>\n     <dd>Verbose operation, showing a list of input files and the total timing.</dd>\n    </dl>\n   </div>\n   <div class='L3'>\n    <h3>\n     <a name='latex'>LaTeX Building</a>\n    </h3>\n    <dl>\n     <dt>-E+output path</dt>\n     <dd>Build a LaTeX document using the default settings for XHTML (Note the ''+'' character!);</dd>\n     <dt>-E csspath+output path</dt>\n     <dd>Build a LaTeX Document using an external stylesheet;</dd>\n     <dt>-e output path</dt>\n     <dd>Build a LaTeX document in verbose node. The resulting document has labels indicating partial paths of leaf elements.</dd>\n    </dl>\n   </div>\n   <div class='L3'>\n    <h3>\n     <a name='xslt'>Stylesheet Building and Transformation (all type M)</a>\n    </h3>\n    <dl>\n     <dt>-P</dt>\n     <dd>Prototyper. Build a default XSLT stylesheet from an (inline) Schema document (M). Use <em>after</em> -1;</dd>\n     <dt>-M stylesheet path</dt>\n     <dd>Mutator. Transform the input document (M). Used to pass a new document to the next processor. There is no output. (Compare ''-T''.)</dd>\n     <dt>-T stylesheet path</dt>\n     <dd>Transfomer. The normal method for XSLT transformation of a document with output as a side effect (N). The output is to standard out, and can be redirected (with ''> file name'') or piped (such as ''|less''). (Compare ''-M''.) </dd>\n     <dt>-T stylesheet path+output path</dt>\n     <dd>Transfomer. The normal method for XSLT transformation of a document with output as a side effect (N). The output is to a defined path.</dd>\n     <dt>-T stylesheet path+output path+true or false</dt>\n     <dd>Transfomer. The normal method for XSLT transformation of a document with output as a side effect (N). The output is to a defined path; ''true'' specifies that the transformation creates an XML document, which is then copied to the output path; ''false'' (the default) specifies that the output does not involve an intermediate document. This is always faster, and obligatory if the transformation result is not an XML document. It is best simply to leave this decision to the system.  </dd>\n    </dl>\n   </div>\n   <div class='L3'>\n    <h3>\n     <a name='editing'>Editing (all type M)</a>\n    </h3>\n    <dl>\n     <dt>-A attribute name</dt>\n     <dd>Replace an element with this attribute with an element with name equal to the attribute value and copy all child nodes from the old element to the new; </dd>\n     <dt>-a</dt>\n     <dd>AttributeStripper: Remove all attributes from a document;</dd>\n     <dt>-a attribute name</dt>\n     <dd>AttributeStripper: Remove all <em>matching</em> attributes from a document;</dd>\n     <dt>-c node XPath expression</dt>\n     <dd>Cutter. Promote all children of each node with this path by inserting them before the specified node. Then remove that node;</dd>\n     <dt>-C node XPath expression</dt>\n     <dd>Cutter. Remove all elements with this path, together with their children.</dd>\n     <dt>-D</dt>\n     <dd>Document Stripper: removes siblings of the document element. Useful with XHTML documents before the ''-r'' command, as it allows trimming back to isolate content; </dd>\n     <dt>-J output path, root name, title string with underscores</dt>\n     <dd>Joiner. Add all source files to a new document with a specified root name and title string.</dd>\n     <dt>-n node XPath expression, element name</dt>\n     <dd>NewNamer. Replace all elements matching the node path with an element with the new name and transfer all descendent nodes to the new element.</dd>\n     <dt>-N node XPath expression, element name</dt>\n     <dd>NewNamer: Replace all elements matching the node path with an element with the new name, but <em>do not</em> transfer descendent nodes to the new element.</dd>\n     <dt>-r</dt>\n     <dd>RootStripper. Repeatedly remove document (root) elements until one is reached with more than a single child node.</dd>\n     <dt>-t</dt>\n     <dd>TextStripper. Remove all text nodes;</dd>\n     <dt>-z</dt>\n     <dd>TextStripper: Remove (zap) all <em>empty</em> elements;</dd>\n     <dt>-Z</dt>\n     <dd>Text Normalizer: Examine all text content and, as necessary, normalize the whitespace to remove embedded carriage return and newline characters.</dd>\n    </dl>\n   </div>\n   <div class='L3'>\n    <h3>\n     <a name='system'>System Wrapping Commands (all type M)</a>\n    </h3>\n    <dl>\n     <dt>-w</dt>\n     <dd>Wrap an XML document in an XHTML wrapper to provide head and body (M).</dd>\n     <dt>-w cssPath</dt>\n     <dd>Wrap an XML document in an XHTML wrapper to provide head and body (M). The head includes a reference to a CSS stylesheet.</dd>\n     <dt>-w cssPath1+cssPath2</dt>\n     <dd>Wrap an XML document in an XHTML wrapper to provide head and body (M). The head includes a reference to a CSS display stylesheet (cssPath1) and a printing stylesheet (cssPath2).</dd>\n     <dt>-W</dt>\n     <dd>Wrap an XML document in an XHTML wrapper to provide head and body (M).</dd>\n     <dt>-W cssPath</dt>\n     <dd>Wrap an XML document in an XHTML wrapper to provide head and body (M). The head includes a <em>copy</em> of the contents a CSS stylesheet.</dd>\n     <dt>-W cssPath1+cssPath2</dt>\n     <dd>Wrap an XML document in an XHTML wrapper to provide head and body (M). The head includes a <em>copies</em> of the contents of the CSS stylesheets.</dd>\n    </dl>\n   </div>\n   <div class='L3'>\n    <h3>\n     <a name='reporting'>Analysis and Reporting (type N)</a>\n    </h3>\n    <dl>\n     <dt>-q</dt>\n     <dd>Simple query with a Reporter (N);</dd>\n     <dt>-Q schema or DTD file path</dt>\n     <dd>Detailed query with a Reporter, including structure check against schema or DTD(N).</dd>\n    </dl>\n   </div>\n   <div class='L3'>\n    <h3>\n     <a name='tools'>Lexxia Analysis Tools (all type M)</a>\n    </h3>\n    <dl>\n     <dt>-1</dt>\n     <dd>SchemaBuilder. Build a Schema document from the input file (M);</dd>\n     <dt>-2</dt>\n     <dd>SchemaInspector. Process a Schema document and build a document listing all element paths in a document (M).  Use <em>after</em> -1;</dd>\n     <dt>-23</dt>\n     <dd>SchemaInspector. Process a Schema document and build a document listing all element paths in a document (the path length is limited in this case to 3) (M).  Use <em>after</em> -1;</dd>\n     <dt>-3</dt>\n     <dd>XMLMapper. Build an XSLT stylesheet to convert the input file to XML (M). Use <em>after</em> -1 -2;</dd>\n     <dt>-4</dt>\n     <dd>SchemaMapper. Build an XSLT stylesheet to convert the input file to LaTeX (M). Use <em>after</em> -1 -2. Now deprecated.</dd>\n    </dl>\n   </div>\n  </div>\n </body>\n</html>";
  helpString.change("\\n", "\n\n");
  wrt::Console << helpString << wrt::endl;
  wrt::Console.flush();
}


}
